;*******************************************************************************************
;   Beschreibung:	Kommunikation mit Hauptcontroller fr Tasteneingaben und Display ber 
;					2-Leiter bus. Interner 16MHZ Oscillator																								 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	Handbox_P16F1827.asm						                               		 *
;    Datum:        	03.02.2020                                                            *
;    Version: 			V2.0                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;	Subtraktion: STATUS,C = 0, wenn Ergebniss <0, = 1, wenn Ergebniss 0 oder >0*
;	Addition: STATUS,C = 0, wenn Ergebniss <256, = 1, wenn Ergebniss >255*
;
;
;
;																										 *
;													 *
;                                                                                          *
;*******************************************************************************************


	list      p=16f1827		;definiert den verwendeten Prozessor
	#include <p16f1827.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
   __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_OFF & _BORV_19 & _LVP_OFF
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus


;===========================
;  DEFINITIONEN
;===========================

#define IN_THEIZ		PORTA, 0		;(pin 17) Eingang Heizungstemperatur
#define IN_TA			PORTA, 1		;(pin 18) Eingang Umgebungstemperatur
#define SI_CLK			PORTA, 2		;(pin 1) Serieller Takt fr Display
#define SI_DAT			PORTA, 3		;(pin 2) Serielle Daten zum Display 
#define DAT_TX			PORTA, 4		;(pin 3) Daten zum Hauptcontroller
#define IN_NCA5			PORTA, 5		;(pin 4)
#define DISP_EN			PORTA, 6		;(pin 15) Mit der positiven Flanke von DISP_EN werden die parallelen Daten bernommen
#define DISP_RS			PORTA, 7		;(pin 16) Unterscheidung zwischen Befehl und Daten
#define DAT_RX			PORTB, 0		;(pin 6) Daten vom Hauptcontroller
#define OUT_PWM			PORTB, 1		;(pin 7) Ausgang PWM Regelung der Display Heizung
#define IN_MENUE		PORTB, 2		;(pin 8)
#define IN_OK			PORTB, 3		;(pin 9)
#define IN_LEFT			PORTB, 4		;(pin 10)
#define IN_RIGHT		PORTB, 5		;(pin 11)
#define IN_UP			PORTB, 6		;(pin 12), PGC
#define IN_DOWN			PORTB, 7		;(pin 13), PGD

;===========================
;  KONSTANTEN
;===========================


;===========================
;  VARIABLEN BANK 0
;===========================

LOOP_1			EQU 0x20		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x21		;Register fr Daten, die an das Display, das Schiebereg. oder den DAC gesendet werden sollen
SHIFT_CNT		EQU	0x22		;Zhler fr die serielle Datenausgabe (Display, Schiebereg., DAC)
TEMP_01			EQU 0x23		;
TEMP_02			EQU 0x24		;
TEMP_03			EQU 0x25		;
TEMP_04			EQU 0x26		;
TEMP_05			EQU 0x27		;
TEMP_06			EQU 0x28		;
TEMP_07			EQU 0x29		;
BEFEHL			EQU 0x2A		;Register zur Zwischenspeicherung der Tastaturbefehle
BEFEHL_A		EQU 0x2B		;Register zur Zwischenspeicherung der alten Tastaturbefehle
TIMER_H			EQU 0x2C		;Register zur Erweiterung Timer 1; 8 Bit
AD_A			EQU 0x2D		;Register fr AD Wert Aussentemperatur
AD_H			EQU 0x2E		;Register fr AD Wert Heizungstemperatur
T_A				EQU 0x2F		;Register fr Umgebungstemperatur
T_H				EQU 0x30		;Register fr Heizungstemperatur
T_S				EQU 0x31		;Register fr Sollwert Temperatur
PWM_IST			EQU 0x32		;Register fr PMW Wert
I_TEIL_ALT		EQU 0x33		;Register fr den alten I-Anteil der Temperaturregelung
I_TEIL_VZ		EQU 0x34		;Register fr das Vorzeichen des alten I-Anteil: 0=+, 1=-
STAT_PWM		EQU 0x35		;Register fr PMW STATUS: Bit0: Heizung Ein/Aus, Bit1: Fhlerbruch Heizung
								; Bit2: Kurzschluss Fhler Heizung, Bit3: Fhlerbruch Umgebungstemp.
								; Bit4: Kurzschluss Fhler Umgebungstemp.
BEFEHL_B		EQU 0x36		;Register 
LOOP_INT		EQU 0x37		;Zhler fr Interrupt Wartezeit
CONTRAST		EQU 0x39		;Wert fr Kontrast Display (0-63)
MENUE_CNT_MAX	EQU 0x3A		;max. Anzahl Menpunkte
MENUE_COUNT		EQU 0x3B		;vertikaler Menpunkt Zhler
TEMP_DIG_CNT	EQU 0x3C		;horizontaler Menpunkt Zhler
TEMP_DIG_STAT	EQU 0x3D
TEMP_CNT_MAX	EQU 0x3E
MENUE_ADR		EQU 0x3F		;Adresse, die vom Master gesendet wird fr Handbox Men
MENUE_STAT		EQU 0x40		;Adresse, die vom Master gesendet wird fr Handbox Men

;===========================
;  VARIABLEN EEPROM
;===========================

		ORG 	0xF000			;Adresse des EEPROM fr den Brenner 

		de		0x01			;Adresse 00,	Heizung: 0=Aus, 1=Ein
		de		0x28			;Adresse 01,	Contrast Wert 0-63(40)

;__________________________________________________________________________________________

		ORG     0x0000			; Adresse, an die nach einem Reset gesprungen wird
		PAGESEL Start
  		goto    Start           ; Springe zum Programmanfang

		ORG     0x0004          ; Adresse, an die nach einem Interrupt gesprungen wird

		; Ab hier kann der Code fr die Interruptroutinen stehen

		BANKSEL PORTA				;Select BANK 0
		movlw d'8'					;es sollen 8 Bits empfangen werden
		movwf SHIFT_CNT				;Zhler fr die bertragenen Bits
		nop
		nop
		nop
		nop
		nop
		nop
		movlw d'3'
		btfsc DAT_RX
		goto send_tastbef

read_data
		bcf DISP_EN				;Enable auf low setzen
		movlw d'3'
		movwf LOOP_INT
		decfsz LOOP_INT
		goto $-1	
		btfss DAT_RX
		bcf DISP_RS
		btfsc DAT_RX
		bsf DISP_RS
		movlw d'1'
		movwf LOOP_INT
		decfsz LOOP_INT
		goto $-1	
		clrf MENUE_ADR
		btfsc DISP_RS
		bsf MENUE_ADR,0
disp_loop
		nop		
		bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden Flanke bernommen werden knnen
		bcf STATUS,C
		rlf MENUE_ADR
		btfss DAT_RX				;
		bcf SI_DAT
		btfsc DAT_RX
		bsf SI_DAT					;
		btfsc SI_DAT
		bsf MENUE_ADR,0
		bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten vom Display bernommen.
		nop
		decfsz SHIFT_CNT			;reduziere Bit Zhler um 1
		goto disp_loop				;noch keine 8 Bit bertragen, zurck zur Schleife
		bcf MENUE_STAT,0			;8 Bit bertragen, lsche Bit0 von menue_stat Register
		btfss STATUS,C				;prfen des ausgeschobenen Bits (DISP_RS)
		bsf MENUE_STAT,0			;bit ist 0, d.h. Befehl, setze Bit0 von menue_stat Register
		bsf DISP_EN					;Enable auf high setzen, mit der positiven Flanke werden Daten bertragen 
		goto interupt_end

send_tastbef	
		movwf SHIFT_CNT				;In W steht 3, =>3 Bits sollen bertragen werden
shift_dat_tx
		rlf BEFEHL_B,f				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
		btfss STATUS,C				;Prfen, welchen Wert das Carryflag hat
		bcf DAT_TX					;setze Datenleitung auf Low
		btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
		bsf DAT_TX					;setze Datenleitung auf High
		movlw d'2'
		movwf LOOP_INT
		decfsz LOOP_INT
		goto $-1	
		decfsz SHIFT_CNT,f			;1 Bit wurde bertragen -> Zhler um 1 verringern	
		goto shift_dat_tx			;noch keine 8 Bit bertragen, zurck zur Schleife
		movlw d'2'
		movwf LOOP_INT
		decfsz LOOP_INT
		goto $-1	
		bsf DAT_TX

interupt_end
		bcf INTCON,INTF				;RB0 Interupt Flag lschen
		retfie                    	;Rcksprung vom Interrupt

		

;__________________________________________________________________________________________

Start						; Beginn des Programms
	; Initialisierungen
	BANKSEL PORTA			;Select BANK 0
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB

	BANKSEL LATA			;Select BANK 2
	clrf LATA
	clrf LATB


	BANKSEL TRISA			;Select BANK 1
	movlw b'01111010'
	movwf OSCCON			;Interner Oszillator, 16 MHz
	movlw b'00000010'     	;PortB pull-up on, Interupt RB0 bei negativer Flanke, TMR0 Prescaler 1:8
	movwf OPTION_REG 
	clrf INTCON

	movlw b'01100000'		;bit 7=0: linksbndig, bit 6-4=101: f/64, bit 1-0:+Vref =Vdd
	movwf ADCON1

	movlw b'00100011'		;setze RA2, RA3, RA4, RA6, RA7, als Ausgang, den Rest als Eingang
	movwf TRISA
	movlw b'11111101'		;setze RB1 als Ausgang, den Rest als Eingang
	movwf TRISB

	BANKSEL ANSELA			;Select BANK 3
	movlw b'00000011'		;Pin RA0 und RA1 sind analoge Eingnge (AN0,AN1)
	movwf ANSELA
	clrf ANSELB				;Alle Pins von Port B sind digitale I/O Ports

	BANKSEL TRISA			;Select BANK 1
	movlw b'00000001'		;ADC einschalten
	movwf ADCON0			;

	BANKSEL WPUA			;Select BANK 4
	clrf WPUA
	movlw 0xFF				;Alle Eingnge von Port B haben Pull-Up
	movwf WPUB				

	BANKSEL PORTA			;Select BANK 0
	bsf DAT_TX

	movlw 0x00				;EEPROM Adresse fr Heizung Ein/aus
	call readEE				;Unterprogramm zum Lesen aus dem internen EEPROM
	movwf STAT_PWM
	movlw 0x01				;EEPROM Adresse fr Contrast
	call readEE				;Unterprogramm zum Lesen aus dem internen EEPROM
	movwf CONTRAST			;Schreibe den Wert ins Contrast Register
	call init_display

	clrf MENUE_ADR			;Lsche Register
	clrf MENUE_STAT
	clrf PWM_IST			;Lsche Register fr PMW Wert
	clrf I_TEIL_ALT			;Lsche Register fr den alten I-Anteil der Temperaturregelung
	clrf I_TEIL_VZ			;Lsche Register fr das Vorzeichen des alten I-Anteil: 0=+, 1=-
	movlw d'100'			;Sollwert ist 15C => Wert = (15C + 35) x 2 = 100
	movwf T_S				; 
	clrf TMR1L
	clrf TMR1H
	clrf TIMER_H
	bcf PIR1,TMR1IF			;lsche Timer 1 Interupt Flag
	clrf T1CON
	bsf T1CON,TMR1ON		;Timer 1 On
	clrf BEFEHL_B

	call disp_stat_heiz
	bcf INTCON,INTF			;lsche RB0 Interupt Flag
	bsf INTCON,INTE			;erlaube RB0 Interupt
	bsf INTCON,GIE			;globale Interupts erlauben

;Testbereich_Anfang

;Testbereich_Ende

main
	movfw PORTB			;prfen ob sich der Status einer Taste verndert hat: kopiere Port B ins Befehl Register 
	movwf BEFEHL
	bsf BEFEHL,0		;setze Bits 1:0 (Keine Tasten)
	bsf BEFEHL,1
	movlw 0xFF			;hole den alten Status der Tasten nach W
	subwf BEFEHL,w		;subtrahiere den neuen Status der Tasten in W
	btfsc STATUS,Z		;prfen ob null, d.h. keine nderung
	goto $+3
	call transmit_data	;nicht null, sende den neuen Status zum Master
	goto $+2
	bsf DAT_TX

	btfss MENUE_STAT,0	;prfen, ob letzter Datensatz vom Master ein Befehl oder Daten waren
	goto $+5			; waren Daten, weiter in der Hauptprogrammschleife
	movlw 0xF0			;prfen, ob letzter Befehl vom Master <240 war (Befehl >=240 = Men)
	subwf MENUE_ADR,w	;
	btfsc STATUS,C		;
	goto menue_handbox	;Befehl >=240 => Men

	btfsc STAT_PWM,0	;prfen ob Heizung ausgeschaltet sein soll
	goto $+3
	bcf OUT_PWM
	goto main

	btfss PIR1,TMR1IF	;Prfen, ob Timer 1 bergelaufen ist
	goto main
	bcf PIR1,TMR1IF		;lschen des Timer 1 berlauf Flag
	incfsz TIMER_H		;erhhe Timer 1 Erweiterungsregister und prfe, ob 0
	goto $+2			;Timer noch nicht 0, weiter bei Prfung, ob PWM Zeit abgelaufen ist
	goto $+8			;Timer ist 0, weiter bei Temperaturmessung/Berechnung neuer PWM Wert
	movfw PWM_IST		;Subtrahiere PWM Wert vom aktuellen Timer Wert in W
	subwf TIMER_H,w
	btfss STATUS,C		;Prfen, ob PWM > Timer
	goto main			;Timer hat noch nicht PWM Wert erreicht, weiter bei Hauptprogrammschleife
	bcf OUT_PWM			;Timer hat PWM Wert erreicht oder berschritten, Ausschalten der Heizung
	goto main
	call temp_regelung
	goto main


menue_handbox
	bcf INTCON,GIE			;globale Interupts verbieten
	bcf OUT_PWM				;sicherheitshalber Heizung ausschalten
	movlw 0x02				;das Einstellungsmen hat 2 Menpunkte
	movwf MENUE_CNT_MAX
	movlw 0x01
	movwf MENUE_COUNT		;vertikalen Menpunkt Zhler auf 1 setzen
	movwf TEMP_DIG_CNT		;horizontalen Menpunkt Zhler auf 1 setzen
	call disp_einstell
	call dig_einst_hor
menu_einstell_loop
	btfsc TEMP_DIG_STAT,2
	call disp_einstell
	btfsc TEMP_DIG_STAT,0
	call dig_einst_hor

	call tast_befehl		;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;Prfen ob "Men" Taste gedrckt ist (high ist)	
	goto menue_end			;
	btfsc BEFEHL,5			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_cnt		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)	
	goto menu_einstell_loop	;

	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENUE_COUNT,w	;
	btfsc STATUS,Z		
	goto heizung_set		; Aufruf des Programmteils heizung_set
	movlw d'2'				;
	subwf MENUE_COUNT,w	;
	btfsc STATUS,Z		
	goto contrast_set		; Aufruf des Programmteils contrast_set

heizung_set
	movlw d'3'				;Prfen, ob horizontaler Menpunkt 3 (Test) gewhlt ist
	subwf TEMP_DIG_CNT,w	;
	btfsc STATUS,Z
	goto heizung_test		;Menpunkt 3 (Test) ist gewhlt
	movlw d'2'				;Prfen, ob horizontaler Menpunkt 2 (Aus) gewhlt ist
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS,Z			;
	goto $+3				;Menpunkt 1 (Ein) ist gewhlt
	bcf STAT_PWM,0			;Menpunkt 2 (Aus) ist gewhlt
	goto $+2
	bsf STAT_PWM,0			;Menpunkt 1 (Ein) ist gewhlt
	clrf TEMP_01
	btfsc STAT_PWM,0		;kopiere Wert, der ins EEPROM geschrieben werden soll, nach TEMP Register
	incf TEMP_01
	movlw 0x00				;Adresse, in die der Wert geschrieben werden soll
	call writeEE
	goto menue_end

heizung_test
	call cursor_off			;Cursor ausschalten
	call clear_display		;Display lschen
	movlw 0x46
	movwf T_A
	movwf T_H
	call disp_adval			;Aufruf Unterprogramm zur Anzeige von Sollwert, Istwert Umgebung/Heizung, PMW-Wert
	movlw d'250'
	call wait
heiz_test_loop
	call tast_befehl		;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto menue_end			;Taste gedrckt, gehe zum Ende des Menpunktes
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto menue_end			;Taste gedrckt, gehe zum Ende des Menpunktes
	btfss PIR1,TMR1IF		;Prfen, ob Timer 1 bergelaufen ist
	goto heiz_test_loop
	bcf PIR1,TMR1IF			;lschen des Timer 1 berlauf Flag
	incfsz TIMER_H			;erhhe Timer 1 Erweiterungsregister und prfe, ob 0
	goto $+2				;Timer noch nicht 0, weiter bei Prfung, ob PWM Zeit abgelaufen ist
	goto $+7				;Timer ist 0, weiter bei Temperaturmessung/Berechnung neuer PWM Wert
	movfw PWM_IST			;Subtrahiere PWM Wert vom aktuellen Timer Wert in W
	subwf TIMER_H,w
	btfss STATUS,C			;Prfen, ob PWM > Timer
	goto heiz_test_loop		;Timer hat noch nicht PWM Wert erreicht, weiter bei Programmschleife
	bcf OUT_PWM				;Timer hat PWM Wert erreicht oder berschritten, Ausschalten der Heizung
	goto heiz_test_loop

	call temp_regelung
	call disp_adval	
	goto heiz_test_loop



contrast_set				;Menpunkt 8.6.2. Kontrast Einstellung
	movfw CONTRAST			;Kontrastwert in Temp Register sichern
	movwf TEMP_04
	call disp_set_contrast	;Anzeigen des Kontrast Werte
contr_set_loop
	btfsc TEMP_DIG_STAT,0	;prfen, ob Wert sich nicht gendert hat
	call disp_set_contrast	;Wert hat sich gendert, Anzeigen des neuen Kontrast Wertes
	call tast_befehl		;Aufruf des Unterprogramms zum laden der Tastenzustnde nach Befehl
	btfsc BEFEHL,0			;prfen, ob Men Taste nicht gedrckt ist
	goto contr_set_end1
	btfsc BEFEHL,1			;prfen, ob OK Taste nicht gedrckt ist
	goto contr_set_end
	btfsc BEFEHL,4			;prfen, ob Up Taste nicht gedrckt ist
	call inc_contrast
	btfsc BEFEHL,5			;prfen, ob Down Taste nicht gedrckt ist
	call dec_contrast
	goto contr_set_loop
contr_set_end				;Einstellungen sollen bernommen werden
	movfw CONTRAST			;Speichern des Kontrast Wertes im internen EEPROM
	movwf TEMP_01			;zu speichernder Wert
	movlw 0x01				;EEPROM Adresse
	call writeEE
	goto menue_end
contr_set_end1				;Einstellungen sollen nicht bernommen werden
	movfw TEMP_04			;Rcksichern des in Temp Register gespeicherten Kontrast Wertes
	movwf CONTRAST
	goto menue_end

menue_end
	movlw d'250'			;Wartezeit von 250ms
	call wait				;
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	clrf TEMP_DIG_STAT
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays	
	call cursor_off			;Cursor ausschalten
	clrf BEFEHL_B			;lsche Register
	clrf MENUE_STAT
	clrf MENUE_ADR
	bcf DAT_TX
	movlw d'10'				;Wartezeit von 10ms
	call wait				;
	bsf DAT_TX
	bcf INTCON,INTF			;lsche RB0 Interupt Flag
	bsf INTCON,INTE			;erlaube RB0 Interupt
	bsf INTCON,GIE			;globale Interupts erlauben
	goto main

menue_1up
	incf MENUE_COUNT		;Erhhe Menpunktzhler
	movfw MENUE_CNT_MAX		;hole maximale Anzahl nach W und addiere 1
	addlw 0x01
	subwf MENUE_COUNT,w		;prfen, ob maximale Anzahl um 1 berschritten wurde
	btfss STATUS,Z		
	goto menue_1updn_end	;nein, keine berschreitung
	movlw d'1'				;ja, maximale Anzahl wurde berschritten, rcksetzen des Zhlers auf 1
	movwf MENUE_COUNT
	goto menue_1updn_end
menue_1dn
	decfsz MENUE_COUNT		;Reduziere Menpunktzhler und prfe ob 0
	goto menue_1updn_end	;noch nicht 0
	movfw MENUE_CNT_MAX		;Menpunktzhler ist 0, rcksetzen des Zhlers auf maximale Anzahl Menpunkte
	movwf MENUE_COUNT
menue_1updn_end
	movlw d'1'				;lade horizontalen Zhler mit Wert 1
	movwf TEMP_DIG_CNT		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit fr update Cursorposition in Menpunkten Goto, Position und Speed
	bsf TEMP_DIG_STAT,2		;setze das Statusbit fr update Display
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	return

inc_dig_cnt
	movfw TEMP_CNT_MAX		;
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z			;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4				;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf TEMP_DIG_CNT
	goto incdec_cnt_end
	incf TEMP_DIG_CNT
	goto incdec_cnt_end
dec_dig_cnt
	decfsz TEMP_DIG_CNT		;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto incdec_cnt_end		;wenn ja, berspringe diesen Befehl	
	movfw TEMP_CNT_MAX		;
	movwf TEMP_DIG_CNT
incdec_cnt_end
	bsf TEMP_DIG_STAT,0		;setze das Status bit
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	return

dig_einst_hor				;Unterprogramm zum setzen des Cursor im Einstell Untermen 
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_einst_hor2		;Ergebniss >0<, weiter bei nchster Prfung
	movlw 0xC2				;lade Adresse ins W-Register (3.Zeichen in der 2.Reihe)
	goto dig_einst_hor_end	;springe zum Ende des Unterprogramms
dig_einst_hor2
	movlw d'2'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_einst_hor3		;Ergebniss >0<, weiter bei nchster Prfung
	movlw 0xC6				;lade Adresse ins W-Register (7.Zeichen in der 2.Reihe)
	goto dig_einst_hor_end	;springe zum Ende des Unterprogramms
dig_einst_hor3
	movlw 0xCA				;lade Adresse ins W-Register (11.Zeichen in der 2.Reihe)
dig_einst_hor_end
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf TEMP_DIG_STAT,0		;lsche das Status bit
	movlw d'250'			;Wartezeit von 250ms
	call wait
	return

inc_contrast
	incf CONTRAST
	movlw d'64'
	subwf CONTRAST,w
	btfss STATUS,C
	goto incdec_contr_end
	movlw d'63'
	movwf CONTRAST
	goto incdec_contr_end
dec_contrast
	decf CONTRAST
	movlw d'24'
	subwf CONTRAST,w
	btfsc STATUS,C
	goto incdec_contr_end
	movlw d'24'
	movwf CONTRAST
incdec_contr_end
	call init_display
	bsf TEMP_DIG_STAT,0
	return


;*******************************************************************************************
; Unterprogramme zur Heizungsregelung
;*******************************************************************************************

temp_regelung
	bcf T1CON,TMR1ON	;Timer 1 Off
	bcf OUT_PWM			;Ausschalten der Heizung

	BANKSEL TRISA		;Select BANK 1
	bcf	ADCON0,2		;Aussentemperatur messen: ADC-Eingang AN0 auswhlen, ADCHS0=0
	call temp_messen	;Unterprogramm zum Auslesen des AD Wandlers
	movwf AD_A			;kopiere den AD Wert (8 Bit) fr die Aussentemperatur ins entsprechende Register
	btfss STAT_PWM,1	;prfen, ob Fehlerbit fr "Fhlerbruch" gesetzt ist
	goto $+4			;nein, weiter bei nchster Prfung
	bcf STAT_PWM,1		;
	bsf STAT_PWM,3		;setze Fehlerbit fr "Fhlerbruch Aussentemperatur"
	goto pwm_off		;Heizung ausschalten
	btfss STAT_PWM,2	;prfen, ob Fehlerbit fr "Kurzschluss Fhler" gesetzt ist
	goto $+4			;nein, weiter bei Auswertung des AD Wertes
	bcf STAT_PWM,2		;
	bsf STAT_PWM,4		;setze Fehlerbit fr "Kurzschluss Fhler Aussentemperatur"
	goto pwm_off		;Heizung ausschalten
	movlw 0x07			;setzen des Programmzhlers fr Temperaturtabelle im Bereich 0x701...0x7E0
	movwf PCLATH
	movfw AD_A			;hole den AD Wert (8 Bit) fr die Aussentemperatur nach W
	addlw d'235'		;subtrahiere Wert 21 (bzw. addiere Wert 235)
	call tab_temperatur	;hole den entsprechenden Temperaturwert aus der Temperaturtabelle
	movwf T_A			;(Umrechnung der Tabellenwerte in Temperatur: Tabellenwert /2 minus 35)

	BANKSEL TRISA		;Select BANK 1
	bsf	ADCON0,2		;Heizungstemperatur messen: ADC-Eingang AN1 auswhlen, ADCHS0=1
	call temp_messen	;Unterprogramm zum Auslesen des AD Wandlers
	movwf AD_H			;kopiere den AD Wert (8 Bit) fr die Heizungstemperatur ins entsprechende Register
	btfsc STAT_PWM,1	;prfen, ob Fehlerbit fr "Fhlerbruch Heizung" gesetzt ist
	goto pwm_off		;Heizung ausschalten
	btfsc STAT_PWM,2	;prfen, ob Fehlerbit fr "Kurzschluss Fhler Heizung" gesetzt ist
	goto pwm_off		;ja, Heizung ausschalten
	movlw 0x07			;setzen des Programmzhlers fr Temperaturtabelle im Bereich 0x701...0x7E0
	movwf PCLATH
	movfw AD_H			;hole den AD Wert (8 Bit) fr die Heizungstemperatur nach W
	addlw d'235'		;subtrahiere Wert 21 (bzw. addiere Wert 235)
	call tab_temperatur	;hole den entsprechenden Temperaturwert aus der Temperaturtabelle
	movwf T_H			;(Umrechnung der Tabellenwerte in Temperatur: Tabellenwert /2 minus 35)
;Sollwert Temperatur T_S berechnen: T_S = 15C + 15C - T_A fr T_A < 15C, T_S max = 50C
	movlw 0x64			;hole Temperaturwert 15C nach W: (15+35)*2 = 100 = 0x64
	subwf T_A,w			;Differenz Auentemperatur minus 15C in W
	btfsc STATUS,C		;
	goto pwm_off		;Aussentemperatur >=15C, Ausschalten der Heizung
	clrf T_S			;Aussentemperatur <15C, Lsche Sollwertregister
	subwf T_S,w			;subtrahiere W von 0
	movwf PWM_IST		;Differenz aus W sichern in PWM Register (wird spter noch gebraucht)
	addlw 0x64			;addiere 15C
	movwf T_S			;sichern in Sollwert Register
	sublw d'170'		;prfen ob Sollwert > max (max = 50C): subtrahiere Sollwert von Wert 170 (50C)
	btfsc STATUS,C		;
	goto $+3			;Sollwert <= 50C
	movlw d'170'		;berechneter Sollwert > 50C, schreibe 50C ins Sollwert Register
	movwf T_S			;

;PMW Wert berechnen: PMW Wert = Basiswert + P-Anteil + I-Anteil
	movfw PWM_IST		;Basiswert = (15C - T_A) x 6
	bcf STATUS,C
	rlf PWM_IST,f
	btfss STATUS,C
	goto $+3
	movlw 0xFF
	goto $+4
	addwf PWM_IST,w
	btfsc STATUS,C
	movlw 0xFF
	movwf PWM_IST
p_anteil
	movfw T_H			;P-Anteil = (T soll - T heizung) x 4
	subwf T_S,w
	movwf TEMP_01
	btfss STATUS,C
	goto p_anteil_n		;Ergebnis ist negativ
	bcf STATUS,C
	rlf TEMP_01,w
	addwf PWM_IST
	btfss STATUS,C
	goto i_anteil
	movlw 0xFF
	movwf PWM_IST
	goto i_anteil
p_anteil_n
	bcf STATUS,C
	rlf TEMP_01,w
	addwf PWM_IST
	btfsc STATUS,C
	goto i_anteil
	clrf PWM_IST
i_anteil
	movfw T_H			;I-Anteil = (T soll - T heizung) x 0,5 +/- I-Anteil Alt
	subwf T_S,w
	movwf TEMP_01
	btfsc STATUS,C
	goto i_anteil_p		;Ergebnis ist positiv
	movfw TEMP_01
	clrf TEMP_01
	subwf TEMP_01,f
	bsf I_TEIL_VZ,1
i_anteil_p
	bcf STATUS,C
	rrf TEMP_01,w
	movlw d'0'
	subwf I_TEIL_VZ,w
	btfsc STATUS,Z
	goto i_anteil_pp
	movlw d'1'
	subwf I_TEIL_VZ,w
	btfsc STATUS,Z
	goto i_anteil_pm
	movlw d'2'
	subwf I_TEIL_VZ,w
	btfsc STATUS,Z
	goto i_anteil_mp
i_anteil_mm
	bsf I_TEIL_VZ,0
	goto $+2
i_anteil_pp
	bcf I_TEIL_VZ,0
	movfw TEMP_01
	addwf I_TEIL_ALT,f
	btfss STATUS,C
	goto i_anteil_end
	movlw 0xFF
	movwf I_TEIL_ALT
	goto i_anteil_end
i_anteil_pm
	movfw I_TEIL_ALT
	subwf TEMP_01,f
	btfsc STATUS,C
	goto $+7			;Ergebnis ist positiv, d.h. der I-Anteil = positiv
	bsf I_TEIL_VZ,0		;Ergebnis ist negativ
	movfw TEMP_01
	clrf TEMP_01
	subwf TEMP_01,w
	movwf I_TEIL_ALT
	goto i_anteil_end
	bcf I_TEIL_VZ,0		;Ergebnis ist positiv
	movfw TEMP_01
	movwf I_TEIL_ALT
	goto i_anteil_end
i_anteil_mp
	movfw TEMP_01
	subwf I_TEIL_ALT,f
	btfsc STATUS,C
	goto $+6			;Ergebnis ist positiv, d.h. der I-Anteil = positiv
	bsf I_TEIL_VZ,0		;Ergebnis ist negativ
	movfw I_TEIL_ALT
	clrf I_TEIL_ALT
	subwf I_TEIL_ALT,f
	goto i_anteil_end
	bcf I_TEIL_VZ,0		;Ergebnis ist positiv
i_anteil_end
	bcf I_TEIL_VZ,1	
	movfw I_TEIL_ALT
	btfsc I_TEIL_VZ,0
	goto i_anteil_end_n	;Vorzeichen ist negativ
	addwf PWM_IST,w
	btfss STATUS,C
	goto pwm_on
	movfw PWM_IST
	sublw 0xFF
	movwf I_TEIL_ALT
	movlw 0xFF
	goto pwm_on
i_anteil_end_n
	subwf PWM_IST,w
	btfsc STATUS,C
	goto pwm_on
	movfw PWM_IST
	movwf I_TEIL_ALT	
	movlw 0x00
pwm_on
	movwf PWM_IST	
	movfw PWM_IST
	btfsc STATUS,Z
	goto pwm_off
	bsf OUT_PWM			;Heizung einschalten
	bsf T1CON,TMR1ON	;Timer 1 On
	return
pwm_off
	clrf PWM_IST
	bcf OUT_PWM			;Heizung ausschalten
	bsf T1CON,TMR1ON	;Timer 1 On
	return


temp_messen
	BANKSEL PORTA		;Select BANK 0
	movlw d'40'			;Acquisition delay = 30 us, Zeit zum Einstellen des AD Wandlers, gfordert: 4 us
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	BANKSEL TRISA		;Select BANK 1	
	bsf	ADCON0,1		; ADC starten
	btfsc ADCON0,1		; prfen ob der ADC fertig ist?
	goto $-1			; nein, weiter warten
	movlw d'21'			;minimaler AD Wert
	subwf ADRESH,w		;subtrahiere minimalen AD Wert von den oberen 8 Bit des AD Wertes
	BANKSEL PORTA		;Select BANK 0
	btfss STATUS,C		;prfen ob Ergebnis positiv, d.h. AD Wert >= min
	bsf STAT_PWM,2		;Ergebnis negativ, d.h. AD Wert < min, setze Fehlerbit fr "Fhler Kurzschluss"
	BANKSEL TRISA		;Select BANK 1	
	movlw d'245'		;maximaler AD Wert plus 1
	subwf ADRESH,w		;subtrahiere minimalen AD Wert von den oberen 8 Bit des AD Wertes
	BANKSEL PORTA		;Select BANK 0
	btfsc STATUS,C		;prfen ob Ergebnis negativ, d.h. AD Wert < max +1
	bsf STAT_PWM,1		;Ergebnis positiv, d.h. AD Wert >= max +1, setze Fehlerbit fr "Fhlerbruch"
	BANKSEL TRISA		;Select BANK 1	
	movfw ADRESH		;AD Wert im gltigen Bereich, obere 8 Bit auslesen
	BANKSEL PORTA		;Select BANK 0
	return

;*******************************************************************************************
; Unterprogramme zum Lesen/Schreiben des internen EEPROM
;*******************************************************************************************

readEE
	BANKSEL EEADRL 		;Select BANK 3
	movwf EEADRL 		;Data Memory
						;Address to read
	bcf EECON1,CFGS 	;Deselect Config space
	bcf EECON1,EEPGD	;Point to DATA memory
	bsf EECON1,RD 		;EE Read
	movf EEDATL,W 		;W = EEDATL
	BANKSEL PORTA		;Select BANK 0
	return

writeEE
	BANKSEL EEADRL 		;Select BANK 3
	movwf EEADRL 		;Data Memory Address to write
	BANKSEL PORTA		;Select BANK 0
	movfw TEMP_01		;
	BANKSEL EEADRL 		;Select BANK 3
	movwf EEDATL 		;Data Memory Value to write
	bcf EECON1,CFGS 	;Deselect Configuration space
	bcf EECON1,EEPGD 	;Point to DATA memory
	bsf EECON1,WREN 	;Enable writes
	bcf INTCON,GIE 		;Disable INTs.
	movlw 55h 			;
	movwf EECON2 		;Write 55h
	movlw 0AAh 			;
	movwf EECON2 		;Write AAh
	bsf EECON1,WR 		;Set WR bit to begin write
	bsf INTCON,GIE 		;Enable Interrupts
	bcf EECON1,WREN 	;Disable writes
	btfsc EECON1,WR 	;Wait for write to complete
	goto $-2 			;Done
	BANKSEL PORTA		;Select BANK 0
	return




;*******************************************************************************************
; Unterprogramme zur Anzeige auf dem Display
;*******************************************************************************************

init_display					;Unterprogramm zur Initialisierung des Displays
	movlw d'100'				;geforderte Wartezeit des Displays von 40ms
	call wait 		
	movlw 0x38					;Befehl: Function Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x39					;Befehl: Function Set, (muss zweimal bertragen werden)
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x14					;Befehl: Bias Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 	

	movfw CONTRAST				;kopiere Kontrastwert in Temp Register
	movwf TEMP_02
	bcf TEMP_02,4				;lsche Bits 4+5, da hier nur Bits 0-3 bentigt werden
	bcf TEMP_02,5	
	movfw TEMP_02				;addiere Bits 0-3 und 0x70
	addlw 0x70					;Befehl: Contrast Set
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		

	clrf TEMP_02				;kopiere Bits 4+5 von Contrast Register nach Bits 0+1 des Temp Register
	btfsc CONTRAST,4
	bsf TEMP_02,0
	btfsc CONTRAST,5
	bsf TEMP_02,1
	movfw TEMP_02				;addiere Bits 0-1 und 0x50
	addlw 0x50					;Befehl: Power/Icon/Contrast Contr.
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		

	movlw 0x69					;Befehl: Follower Control
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl			;
	movlw d'1'					;geforderte Wartezeit des Displays von 26,3us
	call wait 		
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl			;
	return

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'2'					;geforderte Wartezeit des Displays von 1,08ms
	call wait 		
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call disp_befehl			;
	movlw d'40'					;Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'40'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

disp_einstell				;Anzeige Untermen Einstellungen
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x01
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_2
	movlw d'3'				;Anzeige hat 3 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_heizung_set
	goto disp_einstell_end
disp_einstell_2
	movlw 0x02
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_end1
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_contrast_set
	goto disp_einstell_end1
disp_einstell_end1
	call cursor_off			;ausschalten des cursor in den Menpunkten 4-6
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf TEMP_DIG_STAT,2		;lsche das Statusbit
	return
disp_einstell_end
	movlw 0xC2
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	call cursor_on
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf TEMP_DIG_STAT,2		;lsche das Statusbit
	return

disp_stat_heiz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_heizung			;Unterprogramm zum Schreiben von "Heizung"
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	btfss STAT_PWM,0
	goto $+3
	call disp_ein				;Unterprogramm zum Schreiben von "Ein"
	return
	call disp_aus				;Unterprogramm zum Schreiben von "Aus"
	return

disp_heizung_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_heizung			;Unterprogramm zum Schreiben von "Heizung"
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_ein				;Unterprogramm zum Schreiben von "Ein"
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	call disp_aus				;Unterprogramm zum Schreiben von "Aus"
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_contrast_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	call disp_contrast			;Unterprogramm zum Schreiben von "Kontrast"
	return

disp_set_contrast
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call disp_contrast			;Unterprogramm zum Schreiben von "Kontrast"
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen	
	movlw 0x8A
	call disp_befehl
	movfw CONTRAST
	call convert_dig
	movfw TEMP_02
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_03
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movlw 0x8B
	call disp_befehl
	call cursor_on
	bcf TEMP_DIG_STAT,0
	movlw d'250'
	call wait
	return

disp_contrast					;Unterprogramm zum Schreiben von "Kontrast"
	movlw A'K'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_heizung					;Unterprogramm zum Schreiben von "Heizung"
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_ein						;Unterprogramm zum Schreiben von "Ein"
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_aus						;Unterprogramm zum Schreiben von "Aus"
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return


disp_adval
	movlw 0x80
	call disp_befehl
	movlw A'T'
	call disp_daten
	movlw A'S'
	call disp_daten
	movfw T_S
	call conv_temperatur
	call disp_daten				;Vorzeichen +/-
	movfw TEMP_01
	call convert_dig
	movfw TEMP_02
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_03
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movlw A'.'					;Punkt
	call disp_daten				;bertrage das Zeichen
	clrw
	btfsc TEMP_06,7
	addlw d'5'
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw A'P'
	call disp_daten
	movlw A'M'
	call disp_daten
	movlw A'W'
	call disp_daten
	movlw A':'
	call disp_daten
	movfw PWM_IST
	call convert_dig
	movfw TEMP_01
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_02
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_03
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten

	movlw 0xC0
	call disp_befehl
	movlw A'T'
	call disp_daten
	movlw A'U'
	call disp_daten

	btfss STAT_PWM,3			;prfen, ob Fehlerbit fr Fhlerbruch Umgebungstemp. gesetzt ist
	goto $+4					;nein, weiter bei nchster Prfung
	call disp_fbr				;ja, schreibe Fhlerbruch (F-Br)
	bcf STAT_PWM,3				;lsche Fehlerbit
	goto disp_adval_th			;weiter bei Heizung
	btfss STAT_PWM,4			;prfen, ob Fehlerbit fr Kurzschluss Fhler Umgebungstemp. gesetzt ist
	goto $+4					;nein, weiter bei schreiben der Temperatur
	call disp_fks				;ja, schreibe Kurzschluss Fhler (F-Ks)
	bcf STAT_PWM,4				;lsche Fehlerbit
	goto disp_adval_th			;weiter bei Heizung

	movfw T_A
	call conv_temperatur
	call disp_daten				;Vorzeichen +/-
	movfw TEMP_01
	call convert_dig
	movfw TEMP_02
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_03
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movlw A'.'					;Punkt
	call disp_daten				;bertrage das Zeichen
	clrw
	btfsc TEMP_06,7
	addlw d'5'
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
disp_adval_th
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw A'T'
	call disp_daten
	movlw A'H'
	call disp_daten

	btfss STAT_PWM,1			;prfen, ob Fehlerbit fr Fhlerbruch Heizungstemp. gesetzt ist
	goto $+4					;nein, weiter bei nchster Prfung
	call disp_fbr				;ja, schreibe Fhlerbruch (F-Br)
	bcf STAT_PWM,1				;lsche Fehlerbit
	return						;Rcksprung
	btfss STAT_PWM,2			;prfen, ob Fehlerbit fr Kurzschluss Fhler Heizungstemp. gesetzt ist
	goto $+4					;nein, weiter bei schreiben der Temperatur
	call disp_fks				;ja, schreibe Kurzschluss Fhler (F-Ks)
	bcf STAT_PWM,2				;lsche Fehlerbit
	return						;Rcksprung

	movfw T_H
	call conv_temperatur
	call disp_daten				;Vorzeichen +/-
	movfw TEMP_01
	call convert_dig
	movfw TEMP_02
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movfw TEMP_03
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten
	movlw A'.'					;Punkt
	call disp_daten				;bertrage das Zeichen
	clrw
	btfsc TEMP_06,7
	addlw d'5'
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


conv_temperatur
	movwf TEMP_01
	movlw d'70'
	subwf TEMP_01
	btfsc STATUS,C
	goto $+7			;positiv
	movlw A'-'
	movwf TEMP_07
	movfw TEMP_01
	clrf TEMP_01
	subwf TEMP_01,f
	goto $+3
	movlw A'+'
	movwf TEMP_07
	clrf TEMP_06
	bcf STATUS,C
	rrf TEMP_01
	rrf TEMP_06
	movfw TEMP_07
	return

disp_fbr						;Schreibe Fhlerbruch (F-Br)
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw A'F'
	call disp_daten
	movlw A'-'
	call disp_daten
	movlw A'B'
	call disp_daten
	movlw A'r'
	call disp_daten
	return

disp_fks						;Schreibe Kurzschluss Fhler (F-Ks)
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw A'F'
	call disp_daten
	movlw A'-'
	call disp_daten
	movlw A'K'
	call disp_daten
	movlw A's'
	call disp_daten
	return


disp_befehl						;Unterprogramm zum Schreiben eines Befehls an das Display 
	bcf DISP_EN					;Enable auf low setzen
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call send_data				;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	goto disp_bef_dat_end
disp_daten						;Unterprogramm zum Schreiben der Daten an das Display 
	bcf DISP_EN					;Enable auf low setzen
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call send_data				;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
disp_bef_dat_end
	bsf DISP_EN					;Enable auf high setzen, mit der positiven Flanke werden Daten bertragen 
	movlw d'50'					;geforderte Wartezeit (min. 26,3 us)
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	return

send_data
	movwf SHIFT_REG				;Die zu bertragenden Daten werden aus dem W-Register in das Reg. SHIFT_REG kopiert
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen Bits
shift_daten
	bcf SI_CLK					;Takt auf Low setzen, damit die Daten bei der steigenden
									;Flanke bernommen werden knnen
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf SI_DAT					;Carryflag = 1 -> Pin SI_DAT auf Highpegel setzen
	btfss STATUS,C				;Nochmal prfen, da immer nur ein Befehl bersprungen wird
	bcf SI_DAT					;Carryflag = 0 -> Pin SI_DAT auf Lowpegel setzen
	nop							;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	bsf SI_CLK					;mit dieser steigenden Flanke werden die Daten bernommen
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shift_daten			;nchstes bit bertragen
	return						;Rcksprung aus dem Unterprogramm

wait
	movwf LOOP_1
wait_loop
	movlw d'250'				;loop_16 mu 16 Takten entsprechen fr 1ms: 16MHz /4/250/16Takte
	movwf TEMP_01
loop_16
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	decfsz TEMP_01
	goto loop_16
	decfsz LOOP_1
	goto wait_loop
	return


convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_03			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_01			;lsche Temp Register
	clrf TEMP_02			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_03,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_01			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_03,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_03,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_02			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_03,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movfw TEMP_03
	return


;*******************************************************************************************
; Unterprogramme zur TX Kommunikation
;*******************************************************************************************

tast_befehl
	movfw PORTB					;Bits BEFEHL Register:
	movwf BEFEHL_A				;bit 0 = Menue	bit 1 = OK		bit 2 = Left	bit 3 = Right
								;bit 4 = Up		bit 5 = Down
	clrf BEFEHL
	btfss BEFEHL_A,2			;Menue
	bsf BEFEHL,0
	btfss BEFEHL_A,3			;OK
	bsf BEFEHL,1
	btfss BEFEHL_A,4			;Left
	bsf BEFEHL,2
	btfss BEFEHL_A,5			;Right
	bsf BEFEHL,3
	btfss BEFEHL_A,6			;Up
	bsf BEFEHL,4
	btfss BEFEHL_A,7			;Down
	bsf BEFEHL,5
	return

transmit_data
	movfw BEFEHL				;Bits BEFEHL Register:
	movwf BEFEHL_A				;bit 0 = Menue	bit 1 = OK		bit 2 = Left	bit 3 = Right
								;bit 4 = Up		bit 5 = Down
	clrf BEFEHL
	movlw b'00000000'
	btfss BEFEHL_A,2			;Menue
	movwf BEFEHL
	movlw b'00100000'
	btfss BEFEHL_A,3			;OK
	movwf BEFEHL
	movlw b'01000000'
	btfss BEFEHL_A,4			;Left
	movwf BEFEHL
	movlw b'01100000'
	btfss BEFEHL_A,5			;Right
	movwf BEFEHL
	movlw b'10000000'
	btfss BEFEHL_A,6			;Up
	movwf BEFEHL
	movlw b'10100000'
	btfss BEFEHL_A,7			;Down
	movwf BEFEHL
	movfw BEFEHL
	movwf BEFEHL_B

	bcf DAT_TX

	return


	ORG     0x06FF			; 

tab_temperatur		;Umrechnung der Tabellenwerte in Temperatur: Tabellenwert /2 minus 35
	addwf PCL,f
	retlw 0xFC		;DP 0: AD Wert = 21, T = 91 C
	retlw 0xF8		;DP 1: AD Wert = 22, T = 89 C
	retlw 0xF5		;DP 2: AD Wert = 23, T = 87,5 C
	retlw 0xF2		;DP 3: AD Wert = 24, T = 86 C
	retlw 0xEF		;DP 4: AD Wert = 25, T = 84,5 C
	retlw 0xEC		;DP 5: AD Wert = 26, T = 83 C
	retlw 0xEA		;DP 6: AD Wert = 27, T = 82 C
	retlw 0xE7		;DP 7: AD Wert = 28, T = 80,5 C
	retlw 0xE5		;DP 8: AD Wert = 29, T = 79,5 C
	retlw 0xE2		;DP 9: AD Wert = 30, T = 78 C
	retlw 0xE0		;DP 10: AD Wert = 31, T = 77 C
	retlw 0xDE		;DP 11: AD Wert = 32, T = 76 C
	retlw 0xDC		;DP 12: AD Wert = 33, T = 75 C
	retlw 0xDA		;DP 13: AD Wert = 34, T = 74 C
	retlw 0xD8		;DP 14: AD Wert = 35, T = 73 C
	retlw 0xD6		;DP 15: AD Wert = 36, T = 72 C
	retlw 0xD4		;DP 16: AD Wert = 37, T = 71 C
	retlw 0xD2		;DP 17: AD Wert = 38, T = 70 C
	retlw 0xD0		;DP 18: AD Wert = 39, T = 69 C
	retlw 0xCE		;DP 19: AD Wert = 40, T = 68 C
	retlw 0xCD		;DP 20: AD Wert = 41, T = 67,5 C
	retlw 0xCB		;DP 21: AD Wert = 42, T = 66,5 C
	retlw 0xC9		;DP 22: AD Wert = 43, T = 65,5 C
	retlw 0xC8		;DP 23: AD Wert = 44, T = 65 C
	retlw 0xC6		;DP 24: AD Wert = 45, T = 64 C
	retlw 0xC5		;DP 25: AD Wert = 46, T = 63,5 C
	retlw 0xC3		;DP 26: AD Wert = 47, T = 62,5 C
	retlw 0xC2		;DP 27: AD Wert = 48, T = 62 C
	retlw 0xC0		;DP 28: AD Wert = 49, T = 61 C
	retlw 0xBF		;DP 29: AD Wert = 50, T = 60,5 C
	retlw 0xBD		;DP 30: AD Wert = 51, T = 59,5 C
	retlw 0xBC		;DP 31: AD Wert = 52, T = 59 C
	retlw 0xBB		;DP 32: AD Wert = 53, T = 58,5 C
	retlw 0xBA		;DP 33: AD Wert = 54, T = 58 C
	retlw 0xB8		;DP 34: AD Wert = 55, T = 57 C
	retlw 0xB7		;DP 35: AD Wert = 56, T = 56,5 C
	retlw 0xB6		;DP 36: AD Wert = 57, T = 56 C
	retlw 0xB4		;DP 37: AD Wert = 58, T = 55 C
	retlw 0xB3		;DP 38: AD Wert = 59, T = 54,5 C
	retlw 0xB2		;DP 39: AD Wert = 60, T = 54 C
	retlw 0xB1		;DP 40: AD Wert = 61, T = 53,5 C
	retlw 0xB0		;DP 41: AD Wert = 62, T = 53 C
	retlw 0xAF		;DP 42: AD Wert = 63, T = 52,5 C
	retlw 0xAE		;DP 43: AD Wert = 64, T = 52 C
	retlw 0xAC		;DP 44: AD Wert = 65, T = 51 C
	retlw 0xAB		;DP 45: AD Wert = 66, T = 50,5 C
	retlw 0xAA		;DP 46: AD Wert = 67, T = 50 C
	retlw 0xA9		;DP 47: AD Wert = 68, T = 49,5 C
	retlw 0xA8		;DP 48: AD Wert = 69, T = 49 C
	retlw 0xA7		;DP 49: AD Wert = 70, T = 48,5 C
	retlw 0xA6		;DP 50: AD Wert = 71, T = 48 C
	retlw 0xA5		;DP 51: AD Wert = 72, T = 47,5 C
	retlw 0xA4		;DP 52: AD Wert = 73, T = 47 C
	retlw 0xA3		;DP 53: AD Wert = 74, T = 46,5 C
	retlw 0xA2		;DP 54: AD Wert = 75, T = 46 C
	retlw 0xA1		;DP 55: AD Wert = 76, T = 45,5 C
	retlw 0xA0		;DP 56: AD Wert = 77, T = 45 C
	retlw 0x9F		;DP 57: AD Wert = 78, T = 44,5 C
	retlw 0x9E		;DP 58: AD Wert = 79, T = 44 C
	retlw 0x9D		;DP 59: AD Wert = 80, T = 43,5 C
	retlw 0x9C		;DP 60: AD Wert = 81, T = 43 C
	retlw 0x9C		;DP 61: AD Wert = 82, T = 43 C
	retlw 0x9B		;DP 62: AD Wert = 83, T = 42,5 C
	retlw 0x9A		;DP 63: AD Wert = 84, T = 42 C
	retlw 0x99		;DP 64: AD Wert = 85, T = 41,5 C
	retlw 0x98		;DP 65: AD Wert = 86, T = 41 C
	retlw 0x97		;DP 66: AD Wert = 87, T = 40,5 C
	retlw 0x96		;DP 67: AD Wert = 88, T = 40 C
	retlw 0x95		;DP 68: AD Wert = 89, T = 39,5 C
	retlw 0x95		;DP 69: AD Wert = 90, T = 39,5 C
	retlw 0x94		;DP 70: AD Wert = 91, T = 39 C
	retlw 0x93		;DP 71: AD Wert = 92, T = 38,5 C
	retlw 0x92		;DP 72: AD Wert = 93, T = 38 C
	retlw 0x91		;DP 73: AD Wert = 94, T = 37,5 C
	retlw 0x90		;DP 74: AD Wert = 95, T = 37 C
	retlw 0x90		;DP 75: AD Wert = 96, T = 37 C
	retlw 0x8F		;DP 76: AD Wert = 97, T = 36,5 C
	retlw 0x8E		;DP 77: AD Wert = 98, T = 36 C
	retlw 0x8D		;DP 78: AD Wert = 99, T = 35,5 C
	retlw 0x8C		;DP 79: AD Wert = 100, T = 35 C
	retlw 0x8C		;DP 80: AD Wert = 101, T = 35 C
	retlw 0x8B		;DP 81: AD Wert = 102, T = 34,5 C
	retlw 0x8A		;DP 82: AD Wert = 103, T = 34 C
	retlw 0x89		;DP 83: AD Wert = 104, T = 33,5 C
	retlw 0x88		;DP 84: AD Wert = 105, T = 33 C
	retlw 0x88		;DP 85: AD Wert = 106, T = 33 C
	retlw 0x87		;DP 86: AD Wert = 107, T = 32,5 C
	retlw 0x86		;DP 87: AD Wert = 108, T = 32 C
	retlw 0x85		;DP 88: AD Wert = 109, T = 31,5 C
	retlw 0x85		;DP 89: AD Wert = 110, T = 31,5 C
	retlw 0x84		;DP 90: AD Wert = 111, T = 31 C
	retlw 0x83		;DP 91: AD Wert = 112, T = 30,5 C
	retlw 0x82		;DP 92: AD Wert = 113, T = 30 C
	retlw 0x82		;DP 93: AD Wert = 114, T = 30 C
	retlw 0x81		;DP 94: AD Wert = 115, T = 29,5 C
	retlw 0x80		;DP 95: AD Wert = 116, T = 29 C
	retlw 0x7F		;DP 96: AD Wert = 117, T = 28,5 C
	retlw 0x7F		;DP 97: AD Wert = 118, T = 28,5 C
	retlw 0x7E		;DP 98: AD Wert = 119, T = 28 C
	retlw 0x7D		;DP 99: AD Wert = 120, T = 27,5 C
	retlw 0x7D		;DP 100: AD Wert = 121, T = 27,5 C
	retlw 0x7C		;DP 101: AD Wert = 122, T = 27 C
	retlw 0x7B		;DP 102: AD Wert = 123, T = 26,5 C
	retlw 0x7A		;DP 103: AD Wert = 124, T = 26 C
	retlw 0x7A		;DP 104: AD Wert = 125, T = 26 C
	retlw 0x79		;DP 105: AD Wert = 126, T = 25,5 C
	retlw 0x78		;DP 106: AD Wert = 127, T = 25 C
	retlw 0x78		;DP 107: AD Wert = 128, T = 25 C
	retlw 0x77		;DP 108: AD Wert = 129, T = 24,5 C
	retlw 0x76		;DP 109: AD Wert = 130, T = 24 C
	retlw 0x75		;DP 110: AD Wert = 131, T = 23,5 C
	retlw 0x75		;DP 111: AD Wert = 132, T = 23,5 C
	retlw 0x74		;DP 112: AD Wert = 133, T = 23 C
	retlw 0x73		;DP 113: AD Wert = 134, T = 22,5 C
	retlw 0x73		;DP 114: AD Wert = 135, T = 22,5 C
	retlw 0x72		;DP 115: AD Wert = 136, T = 22 C
	retlw 0x71		;DP 116: AD Wert = 137, T = 21,5 C
	retlw 0x70		;DP 117: AD Wert = 138, T = 21 C
	retlw 0x70		;DP 118: AD Wert = 139, T = 21 C
	retlw 0x6F		;DP 119: AD Wert = 140, T = 20,5 C
	retlw 0x6E		;DP 120: AD Wert = 141, T = 20 C
	retlw 0x6E		;DP 121: AD Wert = 142, T = 20 C
	retlw 0x6D		;DP 122: AD Wert = 143, T = 19,5 C
	retlw 0x6C		;DP 123: AD Wert = 144, T = 19 C
	retlw 0x6C		;DP 124: AD Wert = 145, T = 19 C
	retlw 0x6B		;DP 125: AD Wert = 146, T = 18,5 C
	retlw 0x6A		;DP 126: AD Wert = 147, T = 18 C
	retlw 0x69		;DP 127: AD Wert = 148, T = 17,5 C
	retlw 0x69		;DP 128: AD Wert = 149, T = 17,5 C
	retlw 0x68		;DP 129: AD Wert = 150, T = 17 C
	retlw 0x67		;DP 130: AD Wert = 151, T = 16,5 C
	retlw 0x67		;DP 131: AD Wert = 152, T = 16,5 C
	retlw 0x66		;DP 132: AD Wert = 153, T = 16 C
	retlw 0x65		;DP 133: AD Wert = 154, T = 15,5 C
	retlw 0x65		;DP 134: AD Wert = 155, T = 15,5 C
	retlw 0x64		;DP 135: AD Wert = 156, T = 15 C
	retlw 0x63		;DP 136: AD Wert = 157, T = 14,5 C
	retlw 0x62		;DP 137: AD Wert = 158, T = 14 C
	retlw 0x62		;DP 138: AD Wert = 159, T = 14 C
	retlw 0x61		;DP 139: AD Wert = 160, T = 13,5 C
	retlw 0x60		;DP 140: AD Wert = 161, T = 13 C
	retlw 0x60		;DP 141: AD Wert = 162, T = 13 C
	retlw 0x5F		;DP 142: AD Wert = 163, T = 12,5 C
	retlw 0x5E		;DP 143: AD Wert = 164, T = 12 C
	retlw 0x5D		;DP 144: AD Wert = 165, T = 11,5 C
	retlw 0x5D		;DP 145: AD Wert = 166, T = 11,5 C
	retlw 0x5C		;DP 146: AD Wert = 167, T = 11 C
	retlw 0x5B		;DP 147: AD Wert = 168, T = 10,5 C
	retlw 0x5B		;DP 148: AD Wert = 169, T = 10,5 C
	retlw 0x5A		;DP 149: AD Wert = 170, T = 10 C
	retlw 0x59		;DP 150: AD Wert = 171, T = 9,5 C
	retlw 0x58		;DP 151: AD Wert = 172, T = 9 C
	retlw 0x58		;DP 152: AD Wert = 173, T = 9 C
	retlw 0x57		;DP 153: AD Wert = 174, T = 8,5 C
	retlw 0x56		;DP 154: AD Wert = 175, T = 8 C
	retlw 0x55		;DP 155: AD Wert = 176, T = 7,5 C
	retlw 0x55		;DP 156: AD Wert = 177, T = 7,5 C
	retlw 0x54		;DP 157: AD Wert = 178, T = 7 C
	retlw 0x53		;DP 158: AD Wert = 179, T = 6,5 C
	retlw 0x52		;DP 159: AD Wert = 180, T = 6 C
	retlw 0x52		;DP 160: AD Wert = 181, T = 6 C
	retlw 0x51		;DP 161: AD Wert = 182, T = 5,5 C
	retlw 0x50		;DP 162: AD Wert = 183, T = 5 C
	retlw 0x4F		;DP 163: AD Wert = 184, T = 4,5 C
	retlw 0x4E		;DP 164: AD Wert = 185, T = 4 C
	retlw 0x4E		;DP 165: AD Wert = 186, T = 4 C
	retlw 0x4D		;DP 166: AD Wert = 187, T = 3,5 C
	retlw 0x4C		;DP 167: AD Wert = 188, T = 3 C
	retlw 0x4B		;DP 168: AD Wert = 189, T = 2,5 C
	retlw 0x4A		;DP 169: AD Wert = 190, T = 2 C
	retlw 0x4A		;DP 170: AD Wert = 191, T = 2 C
	retlw 0x49		;DP 171: AD Wert = 192, T = 1,5 C
	retlw 0x48		;DP 172: AD Wert = 193, T = 1 C
	retlw 0x47		;DP 173: AD Wert = 194, T = 0,5 C
	retlw 0x46		;DP 174: AD Wert = 195, T = 0 C
	retlw 0x45		;DP 175: AD Wert = 196, T = -0,5 C
	retlw 0x45		;DP 176: AD Wert = 197, T = -0,5 C
	retlw 0x44		;DP 177: AD Wert = 198, T = -1 C
	retlw 0x43		;DP 178: AD Wert = 199, T = -1,5 C
	retlw 0x42		;DP 179: AD Wert = 200, T = -2 C
	retlw 0x41		;DP 180: AD Wert = 201, T = -2,5 C
	retlw 0x40		;DP 181: AD Wert = 202, T = -3 C
	retlw 0x3F		;DP 182: AD Wert = 203, T = -3,5 C
	retlw 0x3E		;DP 183: AD Wert = 204, T = -4 C
	retlw 0x3D		;DP 184: AD Wert = 205, T = -4,5 C
	retlw 0x3C		;DP 185: AD Wert = 206, T = -5 C
	retlw 0x3B		;DP 186: AD Wert = 207, T = -5,5 C
	retlw 0x3A		;DP 187: AD Wert = 208, T = -6 C
	retlw 0x39		;DP 188: AD Wert = 209, T = -6,5 C
	retlw 0x38		;DP 189: AD Wert = 210, T = -7 C
	retlw 0x37		;DP 190: AD Wert = 211, T = -7,5 C
	retlw 0x36		;DP 191: AD Wert = 212, T = -8 C
	retlw 0x35		;DP 192: AD Wert = 213, T = -8,5 C
	retlw 0x34		;DP 193: AD Wert = 214, T = -9 C
	retlw 0x33		;DP 194: AD Wert = 215, T = -9,5 C
	retlw 0x32		;DP 195: AD Wert = 216, T = -10 C
	retlw 0x31		;DP 196: AD Wert = 217, T = -10,5 C
	retlw 0x30		;DP 197: AD Wert = 218, T = -11 C
	retlw 0x2F		;DP 198: AD Wert = 219, T = -11,5 C
	retlw 0x2D		;DP 199: AD Wert = 220, T = -12,5 C
	retlw 0x2C		;DP 200: AD Wert = 221, T = -13 C
	retlw 0x2B		;DP 201: AD Wert = 222, T = -13,5 C
	retlw 0x2A		;DP 202: AD Wert = 223, T = -14 C
	retlw 0x28		;DP 203: AD Wert = 224, T = -15 C
	retlw 0x27		;DP 204: AD Wert = 225, T = -15,5 C
	retlw 0x26		;DP 205: AD Wert = 226, T = -16 C
	retlw 0x24		;DP 206: AD Wert = 227, T = -17 C
	retlw 0x23		;DP 207: AD Wert = 228, T = -17,5 C
	retlw 0x21		;DP 208: AD Wert = 229, T = -18,5 C
	retlw 0x20		;DP 209: AD Wert = 230, T = -19 C
	retlw 0x1E		;DP 210: AD Wert = 231, T = -20 C
	retlw 0x1C		;DP 211: AD Wert = 232, T = -21 C
	retlw 0x1B		;DP 212: AD Wert = 233, T = -21,5 C
	retlw 0x19		;DP 213: AD Wert = 234, T = -22,5 C
	retlw 0x17		;DP 214: AD Wert = 235, T = -23,5 C
	retlw 0x15		;DP 215: AD Wert = 236, T = -24,5 C
	retlw 0x13		;DP 216: AD Wert = 237, T = -25,5 C
	retlw 0x11		;DP 217: AD Wert = 238, T = -26,5 C
	retlw 0x0F		;DP 218: AD Wert = 239, T = -27,5 C
	retlw 0x0D		;DP 219: AD Wert = 240, T = -28,5 C
	retlw 0x0A		;DP 220: AD Wert = 241, T = -30 C
	retlw 0x08		;DP 221: AD Wert = 242, T = -31 C
	retlw 0x05		;DP 222: AD Wert = 243, T = -32,5 C
	retlw 0x02		;DP 223: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 224: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 225: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 226: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 227: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 228: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 229: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 230: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 231: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 232: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 233: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 234: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 235: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 236: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 237: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 238: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 239: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 240: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 241: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 242: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 243: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 244: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 245: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 246: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 247: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 248: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 249: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 250: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 251: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 252: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 253: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 254: AD Wert = 244, T = -34 C
	retlw 0x02		;DP 255: AD Wert = 244, T = -34 C


	END                	; Ende des Programms

