;*******************************************************************************************
;   Beschreibung: Teleskop Steuerung, Goto Funktion, interne Positionsencoder, 
;   externe Encoder optional, Positionswahl manuell oder aus externen EEPROMs: 
;   100 Eigene + Kataloge: Messier, NGC, BSC, DCF-77 Sternzeituhr optional
;   Takt : 16 MHz
;								 *							 *
;*******************************************************************************************
;                                                                                          	*
;    Dateiname:	 	Teleskop_St_P16F876A.asm						                   	 	*
;    Datum:        	10.01.2024                                                            	*
;    Version: 		V22                                                                 		*
;                                                                                          	*
;    Autor:       	Werner Hirsch                                                       	*
;    Internet:      www.wernerhirsch.de                                                     *
;    e-mail:		wernerhirschhh@aol.com                                                  *
;                                                                                          	*
;*******************************************************************************************
;																														 *
;	Bankumschaltungen:
;	BANK_0:	BCF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_1:	BSF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_2:	BCF STATUS,RP0
;			BSF STATUS,RP1
;	BANK_3:	BSF STATUS,RP0
;			BSF STATUS,RP1														 *
;
;	Subtraktion: STATUS,C = 0, wenn Ergebniss <0, = 1, wenn Ergebniss 0 oder >0*
;	Addition: STATUS,C = 0, wenn Ergebniss <256, = 1, wenn Ergebniss >255*
;*******************************************************************************************


	list      p=16f876a			;definiert den verwendeten Prozessor
	#include <p16f876a.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define OUT_RA_SMOT		PORTA, 0		;PIN RA0 (pin 2)Ausgang Clock Schrittmotor RA
#define OUT_RA_ENABLE	PORTA, 1		;PIN RA1 (pin 3)Ausgang Enable RA
#define OUT_RA_RL		PORTA, 2		;PIN RA2 (pin 4)Ausgang Drehrichtung RA
#define OUT_DE_ENABLE	PORTA, 3		;PIN RA3 (pin 5)Ausgang Enable DE
#define IN_DCF			PORTA, 4		;PIN RA4 (pin 6)Eingang DCF Empfnger
#define OUT_RA_HL		PORTA, 5		;PIN RA5 (pin 7)Ausgang High/Low Geschwindigkeit RA
#define IN_OST			PORTB, 0		;PIN RB0 (pin 21)Eingang fr ST4 OST 
#define IN_NORD			PORTB, 1		;PIN RB1 (pin 22)Eingang fr ST4 NORD
#define IN_SUED			PORTB, 2		;PIN RB2 (pin 23)Eingang fr ST4 SUED
#define IN_WEST			PORTB, 3		;PIN RB3 (pin 24)Eingang fr ST4 WEST
#define IN_TAKT_SMRA	PORTB, 4		;PIN RB4 (pin 25)Eingang fr Encodertakt RA (externe Encoder) 
#define IN_RA_RL		PORTB, 5		;PIN RB5 (pin 26)Eingang fr Drehrichtung RA (externe Encoder)
#define IN_TAKT_SMDE	PORTB, 6		;PIN RB6 (pin 27), PGC, Eingang fr Encodertakt DE (externe Encoder)
#define IN_DE_RL		PORTB, 7		;PIN RB7 (pin 28), PGD, Eingang fr Drehrichtung DE (externe Encoder)
#define OUT_DE_SMOT		PORTC, 0		;PIN RC0 (pin 11)Ausgang Clock Schrittmotor DE 
#define OUT_DE_RL		PORTC, 1		;PIN RC1 (pin 12)Ausgang Drehrichtung DE 
#define OUT_DE_HL		PORTC, 2		;PIN RC2 (pin 13)Ausgang High/Low Geschwindigkeit DE 
#define SCL_IO			PORTC, 3		;PIN RC3 (pin 14)I2C Clock 
#define SDA_IO			PORTC, 4		;PIN RC4 (pin 15)I2C Daten 
#define IN_NC_RC5		PORTC, 5		;PIN RC5 (pin 16)
#define DAT_RX			PORTC, 6		;PIN RC6 (pin 17)Eingang fr serielle Daten von Handbox
#define DAT_TX			PORTC, 7		;PIN RC7 (pin 18)Ausgang fr serielle Daten zur Handbox 



;===========================
;  VARIABLEN BANK 0
;===========================

COUNT_L_SZ		EQU	0x20		;Zhler fr Low Byte Frequenzeinstellung Sternzeit
COUNT_H_SZ		EQU	0x21		;Zhler fr High Byte Frequenzeinstellung Sternzeit
DIGIT_SZ		EQU	0x22		;6 Sekunden-Zhler fr Sternzeit
COUNT_L_UZ		EQU	0x23		;Zhler fr Low Byte Frequenzeinstellung Uhrzeit
COUNT_H_UZ		EQU	0x24		;Zhler fr High Byte Frequenzeinstellung Uhrzeit
DIGIT_UZ		EQU	0x25		;2 Sekunden-Zhler fr Uhrzeit

RA_LOW_A		EQU	0x26        ;Register fr RA, Zhlerwert A Mikroschritt (Stundenachse)
RA_LOW_B		EQU	0x27        ;Register fr RA, Zhlerwert B Mikroschritt (Stundenachse)
RA_LOW_C		EQU	0x28        ;Register fr RA, Zhlerwert C Mikroschritt (Stundenachse)
RA_LOW_TEMP		EQU	0x29        ;Zhler fr RA Wert A, Mikroschritt (Stundenachse)
COUNT_L_RA_TEMP	EQU	0x2A        ;Zhler fr RA Wert B/C, Mikroschritt (Stundenachse)
RA_MAX_TEMP		EQU 0x2B		;Zhler fr RA Wert A, Vollschritt (Stundenachse)
DE_LOW			EQU	0x2C        ;Register fr DE, Zhlerwert A Mikroschritt (Deklinationsachse)
DE_LOW_TEMP		EQU	0x2D        ;Zhler fr DE Wert A, Mikroschritt (Deklinationsachse)
DE_MAX_TEMP		EQU 0x2E		;Zhler fr RA Wert A, Vollschritt (Deklinationsachse)

;Register fr Variable Stundenachse
RA_NORM_A		EQU 0x2F		;Register fr RA Nachfhrgeschwindigkeit Zhler A, St/So/Lu
RA_NORM_B		EQU 0x30		;Register fr RA Nachfhrgeschwindigkeit Zhler B, St/So/Lu
RA_NORM_C		EQU 0x31		;Register fr RA Nachfhrgeschwindigkeit Zhler C, St/So/Lu
RA_HIGH_L		EQU	0x32		;Register fr RA High speed
RA_ENC_VS_REST	EQU	0x33        ;Register fr Rest bei Berechnung Encoderwert Vollschritt
RA_ENC_A		EQU	0x34		;Zhlerwert A fr Encodertakt bei Mikroschritt (Stundenachse)
RA_ENC_B		EQU	0x35		;Zhlerwert B fr Encodertakt bei Mikroschritt (Stundenachse)
RA_ENC_C		EQU 0x36		;Zhlerwert C fr Encodertakt bei Mikroschritt (Stundenachse)
COUNT_ENC_RA	EQU 0x37		;Zhler A fr Encodertakt (Stundenachse)
LOOP_ENC_RA		EQU 0x38		;Zhler B/C fr Encodertakt (Stundenachse)
;Register fr Variable Deklinationsachse
DE_NORM_L		EQU	0x39		;Normal Geschwindigkeit
DE_HIGH_L		EQU	0x3A		;High speed
DE_ENC_VS_REST	EQU	0x3B        ;Register fr Rest bei Berechnung Encoderwert Vollschritt
DE_ENC_A		EQU	0x3C		;Zhlerwert A fr Encodertakt (Deklinationsachse)
DE_ENC_B		EQU	0x3D		;Zhlerwert B fr Encodertakt (Deklinationsachse)
DE_ENC_C		EQU 0x3E		;Zhlerwert C fr Encodertakt (Deklinationsachse)
COUNT_ENC_DE	EQU	0x3F        ;Zhler A fr Encodertakt (Deklinationsachse)
LOOP_ENC_DE		EQU 0x40		;Zhler B/C fr Encodertakt (Deklinationsachse)

LOOP_1			EQU 0x41		;Zhler fr Wartezeit
SHIFT_REG		EQU	0x42		;Datum, welches an das Display gesendet werden soll
SHIFT_CNT		EQU	0x43		;Zhler fr die Datenausgabe
NEU_DISP		EQU 0x44		;Status Register fr Display Optionen Status Sternzeit
								;bit 0 = Display Sternzeit,	bit 1 = Display Uhrzeit, bit 2 = Display Rektaeszension
								;bit 3 = Display Deklination, bit 4 = Display Menue_1, bit 5 = Anzeige Uhrzeit+Sternzeit = 0 / RA+DE = 1
								;bit 6 = Display Menue_Parameter, bit 7 = Timer 0 berlauf,
MENUE_CNT_MAX	EQU 0x45		;
MENUE_COUNT		EQU 0x46		;
SEK_SZ			EQU	0x47		;Register fr Sekunden Anzeige, Sternzeit
MIN_SZ			EQU	0x48		;Register fr Minuten Anzeige, Sternzeit
STD_SZ			EQU 0x49		;Register fr Stunden Anzeige, Sternzeit
SEK_UZ			EQU	0x4A		;Register fr Sekunden Anzeige, Uhrzeit
MIN_UZ			EQU	0x4B		;Register fr Minuten Anzeige, Uhrzeit
STD_UZ			EQU 0x4C		;Register fr Stunden Anzeige, Uhrzeit
SEK_RA			EQU	0x4D		;Register fr Sekunden Anzeige, RA
MIN_RA			EQU	0x4E		;Register fr Minuten Anzeige, RA
STD_RA			EQU 0x4F		;Register fr Stunden Anzeige, RA
MIN_DE_01		EQU	0x50		;Register fr Minuten Anzeige 0,1er, DE
MIN_DE			EQU	0x51		;Register fr Minuten Anzeige, DE
GRD_DE			EQU 0x52		;Register fr Grad Anzeige, DE
PM_DE			EQU 0x53		;Register fr Vorzeichen +/-, DE

TEMP_DIG_CNT	EQU 0x54		;Zhler fr das zu ndernde Digit in den Menpunkten
TEMP_CNT_MAX	EQU 0x55		;Max Zhlerwert fr das zu ndernde Digit  in den Menpunkten
TEMP_DIG_ADR	EQU 0x56		;Display Adressregister fr das zu ndernde Digit  in den Menpunkten
TEMP_DIG_VAL	EQU 0x57		;Display Wert fr das zu ndernde Digit  in den Menpunkten

TEMP_DIG_STAT	EQU 0x58		;Status Register	bit 0: Anzeige update des zu ndernden Digit in den Menpunkten
								;					bit 1: Zwischenspeicherung Up/Down: Down = 0, Up = 1, 
BEFEHL			EQU 0x59		;Register fr Tastenbefehl vom Slave Controller (Handbox)
								;bit 0 = Menue		bit 1 = OK/speed	bit 2 = Left		bit 3 = Right
								;bit 4 = Up			bit 5 = Down
STAT_POS		EQU 0x5A		;Statusregister fr Positionen: 
								;	Bit 0 wird gesetzt wenn DE Pos <90
								;	Bit 1 wird gesetzt bei +180 DE Pos.
								;	Bit 2 wird gesetzt bei 0 DE Pos.
								;	bit 3 wird gesetzt, wenn RA GOTO Position erreicht ist
								;	bit 4 wird gesetzt, wenn DE GOTO Position erreicht ist
								;	bit 5 Goto Enable Bit RA
								;	bit 6 Goto Enable Bit DE
STAT_MOTOR		EQU 0x5B		;Statusregister fr Motoren: 
								;	Bit 0: Motor RA: 0=Aus, 1=Ein
								;	Bit 1: Motor DE: 0=Aus, 1=Ein
								;	Bit 2: Drehrichtung Motor RA: 0=Normal, 1=Invertiert
								;	bit 3: Drehrichtung Motor DE: 0=Normal, 1=Invertiert
								;	bit 4: Bit fr Wechsel Zhler B/C RA Normalbetrieb
								;	bit 5: Bit fr Wechsel Zhler B/C RA Encoder
								;	bit 6: Bit fr Wechsel Zhler B/C DE Encoder
								;	bit 7: Encoder: 0=Extern, 1=Intern
SPEED			EQU 0x5C		;Register fr Geschwindigkeiten:
								;	bit 0 = H/L DE/RA, 0 = max,	1 = normal
								;	bit 1 = Statusbit fr Vernderung H/L,St,So,Lu
								;	bit 2 = RA_X0,  0 = null,   1 = normal
								;	bit 3 = RA_X2,	0 = x2, 	1 = normal
								;	bit 4 = solar	0 = solar,	1 = stellar
								;	bit 5 = lunar	0 = lunar,	1 = stellar
								;	bit 6 = DE_Up/Down
								;	bit 7 = 
DIG_REG			EQU 0x5D		;Register fr Wahl der Menupunkte:
								;	bit 0 = dig_temp Aufruf
								;	bit 1 = dig_temp_para Aufruf
								;	bit 2 = dig_temp_kat Aufruf
								;	bit 3 = 
								;	bit 4 = 
								;	bit 5 = 
								;	bit 6 = 
								;	bit 7 = 
MENUE_ADR		EQU 0x5E		;Register fr Menroutine
STAT_ENCODER	EQU 0x5F		;Register fr Status der Encoder; Bit 1:0 nicht belegt
								;	Bit 2 : Drehrichtung Encoder RA int: 0 = links, 1 = rechts
								;	Bit 3 : Drehrichtung Encoder DE int: 0 = down, 1 = up
								;	Bit 4 : nicht belegt
								;	Bit 5 : Alter Status Encodertakt RA ext: 0/1
								;	Bit 6 : Alter Status Encodertakt DE ext: 0/1
								;	Bit 7 : nicht belegt
ENCODER_NEU		EQU 0x60		;Register fr neuen Status der externen Encoder
RA_HS_RAMPE		EQU	0x61		;High speed: Register fr Beschleunigungs-/Bremsrampe
DE_HS_RAMPE		EQU	0x62		;High speed: Register fr Beschleunigungs-/Bremsrampe
STAT_RAMPE		EQU	0x63		;High speed: Status Register fr Beschleunigungs-/Bremsrampe
								;	bit 0 : RA, 0=Beschleunigungsrampe, 1=Bremsrampe
								;	bit 1 : RA, =1 wenn Ende Bremsrampe erreicht
								;	bit 2 : DE, 0=Beschleunigungsrampe, 1=Bremsrampe
								;	bit 3 : DE, =1 wenn Ende Bremsrampe erreicht
								;	bit 4 : 
								;	bit 5 : 
								;	bit 6 : RA, Kopie Timer1 berlauf Bit
								;	bit 7 : DE, Kopie Timer1 berlauf Bit
MIN_GOTO_01		EQU	0x64		;DE Goto Register (0-180), 10tel Minuten
MIN_GOTO		EQU	0x65		;DE Goto Register (0-180), Minuten
GRD_GOTO		EQU	0x66		;DE Goto Register (0-180), Grad
NEU1_DISP		EQU 0x67		;Status Register fr Display 
								;bit 0 = SZ Min, bit 1 = SZ Std, bit 2 = UZ Min
								;bit 3 = UZ Std, bit 4 = RA Min, bit 5 = RA Std
								;bit 6 = DE Min, bit 7 = DE Grd,
SEK_RA_ALT		EQU 0x68		;Status Register fr Sekunden RA, alter Wert 

SEK_TEMP_1		EQU	0x6B		;Temporres Register fr Sekunden Anzeige, 1er und Startpunkt Bremsrampe Goto RA Sek
MIN_TEMP_1		EQU	0x6C		;Temporres Register fr Minuten Anzeige, 1er und Startpunkt Bremsrampe Goto RA Min
STD_TEMP_1		EQU 0x6D		;Temporres Register fr Stunden Anzeige, 1er und Startpunkt Bremsrampe Goto RA Std
MIN_TEM_1		EQU	0x6E		;Temporres Register fr Minuten Anzeige, 1er und Startpunkt Bremsrampe Goto DE Min
GRD_TEM_1		EQU 0x6F		;Temporres Register fr Grad Anzeige, 1er und Startpunkt Bremsrampe Goto DE Grad
SEK_TEMP		EQU	0x70		;Temporres Register fr Sekunden Anzeige, 10er
MIN_TEMP		EQU	0x71		;Temporres Register fr Minuten Anzeige, 10er
STD_TEMP		EQU 0x72		;Temporres Register fr Stunden Anzeige, 10er
MIN_TEM_01		EQU	0x73		;Temporres Register fr Minuten Anzeige, 0,1er
MIN_TEM			EQU	0x74		;Temporres Register fr Minuten Anzeige, 10er
GRD_TEM			EQU 0x75		;Temporres Register fr Grad Anzeige, 10er
PM_TEM			EQU 0x76		;Temporres Register fr Vorzeichen +/-
TEMP_13			EQU	0x77		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x78		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x79		;Allgemeines Zwischenspeicherregister
TEMP_16			EQU 0x7A		;Allgemeines Zwischenspeicherregister
TEMP_17			EQU 0x7B		;Allgemeines Zwischenspeicherregister
TEMP_18			EQU 0x7C		;Allgemeines Zwischenspeicherregister
w_temp        	EQU	0x7D        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x7E        ; der Interruptroutine zu sichern
pclath_temp		EQU	0x7F

;===========================
;  KONSTANTE
;===========================



;===========================
;  VARIABLEN BANK 1 im Bereich 0xB0 bis 0xEF
;===========================
DATEN_1			EQU 0xB0		;Daten 1-16: Register fr Daten vom externen EEPROM, Katalognummer H-Byte
DATEN_2			EQU 0xB1		;Katalognummer L-Byte
DATEN_3			EQU 0xB2		;
DATEN_4			EQU 0xB3		;
DATEN_5			EQU 0xB4		;
DATEN_6			EQU 0xB5		;
DATEN_7			EQU 0xB6		;
DATEN_8			EQU 0xB7		;
DATEN_9			EQU 0xB8		;
DATEN_10		EQU 0xB9		;Helligkeit m
ADR_DEV			EQU	0xBA		;Deviseadresse des EEPROM zum Lesen/Schreiben
ADR_H			EQU	0xBB		;High-Byte der Speicheradresse im EEPROM
ADR_L			EQU	0xBC		;Low-Byte der Speicheradresse im EEPROM
ADR_H_MIN		EQU	0xBD		;High-Byte der Speicheradresse im EEPROM
ADR_L_MIN		EQU	0xBE		;Low-Byte der Speicheradresse im EEPROM
ADR_H_MAX		EQU	0xBF		;High-Byte der Speicheradresse im EEPROM
ADR_L_MAX		EQU	0xC0		;Low-Byte der Speicheradresse im EEPROM
KAT_H			EQU	0xC1		;Zwischenspeicher fr Katalognummer EEPROM, High Byte
KAT_L			EQU	0xC2		;Zwischenspeicher fr Katalognummer EEPROM, Low Byte
KAT_5			EQU 0xC3		;Zwischenspeicher fr Katalognummer EEPROM, 100.000er
KAT_4			EQU 0xC4		;Zwischenspeicher fr Katalognummer EEPROM, 10.000er
KAT_3			EQU	0xC5		;Zwischenspeicher fr Katalognummer EEPROM, 1.000er
KAT_2			EQU	0xC6		;Zwischenspeicher fr Katalognummer EEPROM, 100er
KAT_1			EQU	0xC7		;Zwischenspeicher fr Katalognummer EEPROM, 10er
KAT_0			EQU	0xC8		;Zwischenspeicher fr Katalognummer EEPROM, 1er
ADR_H1			EQU	0xC9		;High-Byte der Speicheradresse im EEPROM
ADR_L1			EQU	0xCA		;Low-Byte der Speicheradresse im EEPROM
ZAHL_A0			EQU 0xCB		;Rechenregister A, Bit 0-7
ZAHL_A1			EQU 0xCC		;Rechenregister A, Bit 8-15
ZAHL_A2			EQU 0xCD		;Rechenregister A, Bit 16-23
ZAHL_A3			EQU 0xCE		;Rechenregister A, Bit 24-31
ZAHL_B0			EQU 0xCF		;Rechenregister B, Bit 0-7
ZAHL_B1			EQU 0xD0		;Rechenregister B, Bit 8-15
ZAHL_B2			EQU 0xD1		;Rechenregister B, Bit 16-23
ZAHL_B3			EQU 0xD2		;Rechenregister B, Bit 24-31
ZAHL_C0			EQU 0xD3		;Rechenregister C, Bit 0-7
ZAHL_C1			EQU 0xD4		;Rechenregister C, Bit 8-15
ZAHL_C2			EQU 0xD5		;Rechenregister C, Bit 16-23
ZAHL_C3			EQU 0xD6		;Rechenregister C, Bit 24-31
ZAHL_C4			EQU 0xD7		;Rechenregister C, Bit 32-39
ZAHL_C5			EQU 0xD8		;Rechenregister C, Bit 40-47
ZAHL_C6			EQU 0xD9		;Rechenregister C, Bit 48-55
ZAHL_C7			EQU 0xDA		;Rechenregister C, Bit 56-63
TBIT_H			EQU 0xDB		;
TBIT_L			EQU 0xDC		;
DCF_BIT			EQU 0xDD		;
DCF_MESS_STAT	EQU 0xDE		;
DCF_PRUEFBIT	EQU 0xDF		;
DCF_COUNT		EQU 0xE0		;
TAG_UZ			EQU 0xE1		;Tag
WTAG_UZ			EQU 0xE2		;Wochentag
MONAT_UZ		EQU 0xE3		;Monat
JAHR_UZ			EQU 0xE4		;Jahr
ZZ_UZ			EQU 0xE5		;Zeitzone
DCF_H_MAX		EQU 0xE6		;Maximalwert fr DCF Bit "High"
DCF_H_MIN		EQU 0xE7		;Minimalwert fr DCF Bit "High"
DCF_L_MAX		EQU 0xE8		;Maximalwert fr DCF Bit "Low"
DCF_L_MIN		EQU 0xE9		;Minimalwert fr DCF Bit "Low"
LOOP_2			EQU 0xEA		;Zhler allgemein

;===========================
;  VARIABLEN BANK 2 im Bereich 0x0120 bis 0x016F
;===========================
;DATEN_03TEMP	EQU 0x0120		;Allgemeines Zwischenspeicherregister


;__________________________________________________________________________________________
;
		ORG 	H'2100'			;Adresse des EEPROM fr den Brenner 
;Konstanten Stundenachse
		de		0x7A			;Adresse 0x00,	Solar: RA, Zhler A
		de		0x77			;Adresse 0x01,	Solar: RA, Zhler B
		de		0x09			;Adresse 0x02,	Solar: RA, Zhler C
		de		0x79			;Adresse 0x03,	Stellar: RA, Zhler A
		de		0x42			;Adresse 0x04,	Stellar: RA, Zhler B
		de		0xB9			;Adresse 0x05,	Stellar: RA, Zhler C
		de		0x7E			;Adresse 0x06,	Lunar: RA, Zhler A
		de		0x1C			;Adresse 0x07,	Lunar: RA, Zhler B
		de		0x0F			;Adresse 0x08,	Lunar: RA, Zhler C
		de		0x05			;Adresse 0x09,	High speed: RA, Zhler A
		de		0x40			;Adresse 0x0A,	Zhlerwert A fr Encoder RA bei Mikroschritt
		de		0x01			;Adresse 0x0B,	Zhlerwert B fr Encoder RA bei Mikroschritt
		de		0x00			;Adresse 0x0C,	Zhlerwert C fr Encoder RA bei Mikroschritt
;Konstanten Deklinationsachse
		de		0x62			;Adresse 0x0D,	Normal Geschwindigkeit: DE, Zhler A
		de		0x05			;Adresse 0x0E,	High speed: DE, Zhler A
		de		0x40			;Adresse 0x0F,	Zhlerwert A fr Encoder RA bei Mikroschritt
		de		0x01			;Adresse 0x10,	Zhlerwert B fr Encoder RA bei Mikroschritt
		de		0x00			;Adresse 0x11,	Zhlerwert C fr Encoder RA bei Mikroschritt
;Konstanten Position
		de		0x00			;Adresse 0x12,	Ost=0, West=1
		de		0x09			;Adresse 0x13,	Winkelgrad
		de		0x35			;Adresse 0x14,	Winkelminuten 
		de		0x24			;Adresse 0x15,	Winkelsekunden
;Konstanten DCF Bitlnge Min/Max
		de		0xD6			;Adresse 0x16,	Maximalwert fr DCF Bit "High"
		de		0xAF			;Adresse 0x17,	Minimalwert fr DCF Bit "High"
		de		0x75			;Adresse 0x18,	Maximalwert fr DCF Bit "Low" 
		de		0x4E			;Adresse 0x19,	Minimalwert fr DCF Bit "Low"
;Konstanten Nachfhrgeschwindigkeit
		de		0x01			;Adresse 0x1A,	Stellar = 1, Solar =2, Lunar = 3
;Register STAT_MOTOR
		de		0x83			;Adresse 0x1B,	10000011
;Konstanten Stundenachse, Berechnung Parameter
		de		0x02			;Adresse 0x1C,	bersetzungsverhltnis: 172800, hex: 0x02A300, H-byte
		de		0xA3			;Adresse 0x1D,	bersetzungsverhltnis: 172800, hex: 0x02A300, M-byte
		de		0x00			;Adresse 0x1E,	bersetzungsverhltnis: 172800, hex: 0x02A300, L-byte
		de		0x08			;Adresse 0x1F,	n-max 800/min, 1/100stel: 8, hex: 0x08
;Konstanten Deklinationsachse, Berechnung Parameter
		de		0x03			;Adresse 0x20,	bersetzungsverhltnis: 216000, hex: 0x034BC0, H-byte
		de		0x4B			;Adresse 0x21,	bersetzungsverhltnis: 216000, hex: 0x034BC0, M-byte
		de		0xC0			;Adresse 0x22,	bersetzungsverhltnis: 216000, hex: 0x034BC0, L-byte
		de		0x08			;Adresse 0x23,	n-max 800/min, 1/100stel: 8, hex: 0x08
;__________________________________________________________________________________________

		ORG     0			; Adresse, an die nach einem Reset gesprungen wird
		clrf    PCLATH          ; Rcksetzen der Page-Bits
  		goto    Start           ; Springe zum Programmanfang

		ORG     4           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	status_temp     ; Sichere den Inhalt des STATUS-Registers
		movf    PCLATH,w        ; Verschiebe PCLATH-Register ins W-Register
		movwf	pclath_temp     ; Sichere den Inhalt des PCLATH-Registers


Int_serv 
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4				;
	bcf STATUS,RP0				;Bank 0
	bcf STATUS,RP1
	btfsc INTCON,TMR0IF			;prfen ob Timer0 Interupt bit low ist
	call timer0_irp_routine		;wenn ja, berspringe diesen Befehl
	btfsc STAT_MOTOR,7			;prfen ob externe Encoder aktiviert sind
	goto $+3					;nein, nicht aktiviert
	btfsc INTCON,RBIF			;ja, aktiviert, prfen ob RBIF Interupt bit low ist
	call enc_ext_irp_routine	;wenn ja, berspringe diesen Befehl


		movf    pclath_temp,w     	; hole die Kopie des PCLATH-Registers
		movwf	PCLATH            	; Sichere die Daten wieder zurck in das PCLATH-Register
		movf    status_temp,w     	; hole die Kopie des STATUS-Registers
		movwf	STATUS            	; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w          	; Sichere die Daten wieder zurck in das W-Register
		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

Start							; Beginn des Programms
	; Initialisierungen
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	movlw b'11110101'
	movwf PORTA
	movlw b'11111111'
	movwf PORTB
	movlw b'11111111'
	movwf PORTC
	bsf STATUS,RP0			;Select BANK_1
	bcf STATUS,RP1
	movlw b'00000111'		;alle Pins von Port A sind digitale Ein- oder Ausgnge
	movwf ADCON1
	movlw b'11010000'		;setze RA0 bis RA3 und RA5 als Ausgang und RA4 und RA6 bis RA7 als Eingang
	movwf TRISA
	movlw b'11111111'		;setze RB0 bis RB7 als Eingang
	movwf TRISB
	movlw b'01111000'		;setze RC3 bis RC6 als Eingang, den Rest als Ausgang
	movwf TRISC
	movlw B'00000000'		;pull-up, internen Takt zhlen, bit 3: TMR0 Vorteiler 2:1
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	bsf DAT_TX

	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	bcf PCLATH,4			;Select Page 0

	movlw 0x1F				;Laden der Konstanten fr den Sternzeittakt
	movwf COUNT_H_SZ		;
	movlw 0x6F
	movwf COUNT_L_SZ
	movlw 0x06				;Setzen des Zhlers, Digit:= 6	
	movwf DIGIT_SZ			;
	movlw 0x1F				;Laden der Konstanten fr den Uhrzeittakt
	movwf COUNT_H_UZ		;
	movlw 0x84
	movwf COUNT_L_UZ
	movlw 0x02				;Setzen des Zhlers, Digit:= 2	
	movwf DIGIT_UZ			;
	movlw d'16'				;Takt Encoder Mikroschritt : 16 Mikroschritte = 1 Vollschritt
	movwf RA_ENC_VS_REST	;
	movwf DE_ENC_VS_REST	;
	movlw 0xFF				;setze bit 0-7 auf high (=255) in Register Display Status, SPEED und TEMP_DIG_STAT Register
	movwf NEU_DISP			;
	movwf NEU1_DISP			;
	movwf SPEED				;
	movwf TEMP_DIG_STAT		;
	clrf BEFEHL
	clrf STAT_ENCODER

	bsf PCLATH,3			;Select Page 3	
	bsf PCLATH,4			;
	call lese_parameter		;laden der Parameter aus dem internen EEPROM in die F-Register
	bcf PCLATH,3			;Select Page 0	
	bcf PCLATH,4			;
	btfsc STAT_MOTOR,0
	bsf OUT_RA_ENABLE
	btfsc STAT_MOTOR,1
	bsf OUT_DE_ENABLE	

	movlw d'15'				;Rcksetzen der Rampenregister mit Startwerten
	movwf RA_HS_RAMPE
	movwf DE_HS_RAMPE
	clrf STAT_RAMPE
							;Initialisierung der I2C Schnittstelle zum Lesen der externen EEPROMs
	bsf STATUS, RP0			;Bank 1 
    movlw d'9'				;clock = 16000/(4*(9+1)) = 400kHz 
    movwf SSPADD			;fr I2C 
	clrf SSPCON2			;alle Bits auf 0 setzen
    bcf STATUS, RP0			;Bank 0 
	movlw B'00001000'		;master mode, clock=Fosc/(4*(SSPADD+1)) 
    movwf SSPCON			; 

	clrf TMR0
	movlw b'01100000'		;bit0-2 interupt flags, bit3 RBIE enable, bit4 INTE disable,
	movwf INTCON			;bit5 TMR0IE enable, bit6 PEIE enable, bit7 GIE disable
	btfss STAT_MOTOR,7		;prfen ob Bit high ist (Decoder: low = extern, high = intern)
	bsf INTCON,RBIE			;low, externe Encoder, enable RB Port Change Interrupt
	bsf PCLATH,3			;Select Page 1
	call Zeit_Null			;Uhr und Sternzeit werden auf 0 gesetzt
	bcf PCLATH,3			;Select Page 0
	movlw b'00100001'		;Timer1: Prescaler 1:4, Timer1 ON, (65,5ms)
	movwf T1CON
	movlw b'01001110'		;Timer2: Prescaler: 16, Timer2 ON, Postscaler: 10, (10,24ms)
	movwf T2CON
	bsf INTCON,GIE			;Globale Interrupts erlauben
	
;Testbereich_Anfang

;Testbereich_Ende

main							;Beginn der Hauptschleife
	btfss PIR1,TMR2IF			;prfen, ob Timer 2 bergelaufen ist (10ms)
	goto $+7					;
	bcf PIR1,TMR2IF		
	clrf BEFEHL
	btfss DAT_RX				;prfen, ob Slave daten senden will
	call tast_befehl			;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus der Handbox
	btfsc BEFEHL,0				;prfen ob Bit fr Befehl Menue low ist
	goto menue_1				;Bit0 ist gesetzt, springe zum Programmteil "Menue"

	btfss IN_OST				;ST-4 Befehle in Befehlsregister kopieren
	bsf BEFEHL,2
	btfss IN_WEST				
	bsf BEFEHL,3
	btfss IN_NORD				
	bsf BEFEHL,4
	btfss IN_SUED				
	bsf BEFEHL,5

	btfsc SPEED,1				;prfen ob sich die Geschwindigkeit verndert hat
	goto speed_set				;Bit1 ist gesetzt, gehe zum Programmteil speed set

	btfsc BEFEHL,1				;Prfen ob Taste "OK" fr Motortakt nicht gedrckt ist (low ist)
	call hl_ff					;Bit1 ist gesetzt, Aufruf des Unterprogramms fr H/L Flip Flop
	btfss SPEED,0				;Prfen ob Bit fr Motortakt high ist, d.h. Normalgeschwindigkeit
	goto ra_hl					;Bit ist low, weiter bei RA Highspeed

	bsf OUT_RA_HL				;setze Ausgang auf high
	btfss BEFEHL,2				;Prfen ob Pfeiltaste "Left" gedrckt ist (high ist)
	goto $+4					;"Left" ist nicht gedrckt, weiter bei $+
	bcf SPEED,2					;lsche bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	bsf SPEED,3					;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)		
	goto de_up_down
	btfss BEFEHL,3				;Prfen ob Pfeiltaste "Right" gedrckt ist (high ist)
	goto $+4					;"Right" ist nicht gedrckt, weiter bei $+
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bcf SPEED,3					;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)			
	goto de_up_down
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf SPEED,2					;setze bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	bsf SPEED,3					;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)		

de_up_down
	bsf OUT_DE_HL				;setze Ausgang auf high
	btfss BEFEHL,4				;Prfen ob Pfeiltaste "Up" gedrckt ist (high ist)
	goto $+4					;wenn ja berspringe diesen Befehl, wenn nein weiter bei $+4
	bcf SPEED,6					;lsche Bit fr Vernderung DE => Bewegung
	call set_de_up				;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto speed_set
	btfss BEFEHL,5				;Prfen ob Pfeiltaste "Down" gedrckt ist (high ist)
	goto $+4					;wenn ja berspringe diesen Befehl, wenn nein weiter bei $+4
	bcf SPEED,6					;lsche Bit fr Vernderung DE => Bewegung
	call set_de_down			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto speed_set
	bsf SPEED,6					;setze bit fr Vernderung DE => keine Bewegung
	goto speed_set

ra_hl
	btfss BEFEHL,2				;Prfen ob Pfeiltaste "Left" nicht gedrckt ist (low ist)
	goto $+3					;"Left" ist gedrckt, weiter bei $+
	call set_ra_links			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto ra_highspeed
	btfss BEFEHL,3				;Prfen ob Pfeiltaste "Right" nicht gedrckt ist (low ist)
	goto $+3					;"Right" ist gedrckt, weiter bei $+
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto ra_highspeed
	btfss OUT_RA_HL				;prfen, ob Normalgeschwindigkeit (1) oder max. Geschwindigkeit (0)
	bsf STAT_RAMPE,0			;max. Geschw., setze Bit fr Bremsrampe. Bit0: RA, 0=Beschleunigungsrampe, 1=Bremsrampe
	btfss STAT_RAMPE,0			;Normalgeschwindigkeit, prfen, ob Bit fr Bremsrampe gesetzt ist
	goto ra_hl_end				;Bit fr Bremsrampe nicht gesetzt => Rcksetzen auf Normalgeschwindigkeit
	btfsc STAT_RAMPE,1			;Bit fr Bremsrampe ist gesetzt, prfen, ob Bit fr Ende Bremsrampe nicht gesetzt ist
	goto ra_hl_end				;Bit fr Ende Bremsrampe ist gesetzt => Rcksetzen auf Normalgeschwindigkeit
ra_highspeed
	bcf OUT_RA_HL				;setze Ausgang auf low
	goto de_hl					;gehe zur nchsten Befehlsabfrage
ra_hl_end
	bsf OUT_RA_HL				;setze Ausgang auf high (Normalgeschwindigkeit)
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf SPEED,2					;setze bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	bsf SPEED,3					;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)		
	movlw d'15'					;Rcksetzen des Rampenregisters mit Normalgeschwindigkeitswert 
	movwf RA_HS_RAMPE
	bcf STAT_RAMPE,0
	bcf STAT_RAMPE,1
	bcf STAT_RAMPE,6

de_hl
	bcf OUT_DE_HL				;setze Ausgang auf low
	btfss BEFEHL,4				;Prfen ob Pfeiltaste "Up" gedrckt ist (high ist)
	goto $+3					;wenn ja berspringe diesen Befehl, wenn nein weiter bei $+4
	call set_de_up				;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto de_highspeed
	btfss BEFEHL,5				;Prfen ob Pfeiltaste "Down" gedrckt ist (high ist)
	goto $+3					;wenn ja berspringe diesen Befehl, wenn nein weiter bei $+4
	call set_de_down			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto de_highspeed
	btfss SPEED,6
	bsf STAT_RAMPE,2
	btfss STAT_RAMPE,2	
	goto de_hl_end
	btfsc STAT_RAMPE,3
	goto de_hl_end
de_highspeed
	bcf SPEED,6					;lsche Bit fr Vernderung DE => Bewegung
	goto speed_set
de_hl_end
	bsf SPEED,6					;setze Bit fr Vernderung DE => keine Bewegung
	movlw d'15'					;Rcksetzen des Rampenregisters mit Normalgeschwindigkeitswert 
	movwf DE_HS_RAMPE
	bcf STAT_RAMPE,2
	bcf STAT_RAMPE,3
	bcf STAT_RAMPE,7


speed_set
	btfss SPEED,3				;Prfen ob das Bit fr RA_2fach gesetzt ist
	goto ra_x2					;wenn ja berspringe diesen Befehl

	movfw RA_NORM_A				;Lade Register mit Nachfhrgeschwindigkeits Werten
	movwf RA_LOW_A				;
	movfw RA_NORM_B				;
	movwf RA_LOW_B				;
	movfw RA_NORM_C				;
	movwf RA_LOW_C				;
	goto update_disp			;Weiter bei Update Display

ra_x2
	bcf STATUS,C				;lsche carry bit, damit es beim rotieren nicht strt
	movfw RA_NORM_A				;
	rrf RA_NORM_A,w				;teilen durch 2
	movwf RA_LOW_A
	clrf RA_LOW_C

update_disp
	bsf PCLATH,4				;Select Page 2
	btfsc SPEED,1				;prfe Status bit fr genderte Geschwindigkeiten
	call disp_speed				;Aufruf des Unterprogramms zur Anzeige von St,So,Lu, blank/MAX
	bcf PCLATH,4				;Select Page 0
	btfss NEU_DISP,5			;prfe ob bit fr Anzeige RA/DE gesetzt ist
	goto disp_zeit				;wenn ja berspringe diesen befehl, wenn nein gehe zu disp_pos
	goto disp_pos
disp_zeit
	bsf PCLATH,4				;Select Page 2
	btfsc NEU_DISP,1			;prfe Status bit fr Display Uhrzeit (ob Uhrzeit sich gendert hat)
	call disp_uz				;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige Uhrzeit
	btfsc NEU_DISP,0			;prfe Status bit fr Display Sternzeit (ob Sternzeit sich gendert hat)
	call disp_sz				;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige Sternzeit
	bcf PCLATH,4				;Select Page 0
	goto main
disp_pos
	bsf PCLATH,4				;Select Page 2
	btfsc NEU_DISP,2			;prfe Status bit fr Display RA (ob RA sich gendert hat)
	call disp_ra				;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige RA
	btfsc NEU_DISP,3			;prfe Status bit fr Display DE (ob DE sich gendert hat)
	call disp_de				;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige DE
	bcf PCLATH,4				;Select Page 0
	goto main


menue_1
	movlw 0x08				;Men 1 hat 8 Menpunkte
	movwf MENUE_CNT_MAX
	movlw 0x01
	movwf MENUE_COUNT		;vertikalen Menpunkt Zhler auf 1 setzen
	movwf TEMP_DIG_CNT		;horizontalen Menpunkt Zhler auf 1 setzen
	bsf PCLATH,4			;Select Page 2
	call disp_menue_1
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	call dig_temp_kat
	bcf PCLATH,3			;Select Page 0
menue_1_loop
	bsf PCLATH,4			;Select Page 2
	btfsc NEU_DISP,4
	call disp_menue_1
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	btfsc TEMP_DIG_STAT,0
	call dig_temp_kat
	bcf PCLATH,3			;Select Page 0
	clrf BEFEHL
	btfsc DAT_RX			;prfen, ob Slave daten senden will
	goto $-1	
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus dem Slave
	btfsc BEFEHL,0			;Prfen ob "Men" Taste gedrckt ist (high ist)	
	goto menue_end			;
	btfsc BEFEHL,5			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_cnt		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)	
	goto menue_1_loop		;


	movlw d'1'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_goto_set		; Aufruf des Programmteils menu_goto_set
	movlw d'2'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_pos_set		; Aufruf des Programmteils menu_pos_set
	movlw d'3'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_speed_set		; Aufruf des Programmteils menu_speed_set
	movlw d'4'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_motor_set		; Aufruf des Programmteils menu_motor_set
	movlw d'5'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_anzeige_set	; Aufruf des Programmteils menu_anzeige_set
	movlw d'6'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_uz_set		; Aufruf des Programmteils menu_uz_set
	movlw d'7'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_loeschen		; Aufruf des Programmteils menu_loeschen
	movlw d'8'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_einstell_set	; Aufruf des Programmteils menu_einstell
	goto menue_end



menu_goto_set				;Menpunkt 1
	movlw d'0'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	bcf STAT_POS,3			;lsche Statusbits fr Erreichen der GOTO Position RA/DE
	bcf STAT_POS,4			;
	bcf STAT_POS,5			;lsche GOTO Enable Bits
	bcf STAT_POS,6			;
	movlw d'1'
	subwf TEMP_DIG_CNT,w
	btfsc STATUS,Z
	goto menu_goto_man
	bsf PCLATH,3			;Select Page 1
	call menu_eeprom
	bcf PCLATH,3			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 
menu_goto_man
	movlw d'1'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	bsf PCLATH,4			;Select Page 2
	call disp_set_goto		;
	bcf PCLATH,4			;Select Page 0
	bsf PCLATH,3			;Select Page 1
	call menu_rade_man		;Aufruf des Unterprogramms zum setzen der Parameter fr RA/DE manuell
	bcf PCLATH,3			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 


copy_goto_man
	call conv_dig_rade		;Konvertierung der einzelnen Digits in Werte fr RA/DE


copy_goto_eepr				;In diesem Programmteil wird die Drehrichtung RA ermittelt
	movfw GRD_TEM			;Kopieren der DE Goto Werte in Temp Register
	movwf TEMP_18
	movfw MIN_TEM			;
	movwf TEMP_17
	movfw MIN_TEM_01		;
	movwf TEMP_16
	movfw PM_TEM
	bsf PCLATH,4			;Select Page 2	
	call conv_de_disp_int	;Unterprogramm zum Konvertieren von DE Werten -90=>0=>+90 in 0=>180
	bcf PCLATH,4			;Select Page 0
	movfw TEMP_18			;Kopieren der konvertierten DE Werte in die Goto Register
	movwf GRD_GOTO			;
	movfw TEMP_17
	movwf MIN_GOTO			;
	movfw TEMP_16
	movwf MIN_GOTO_01

	movlw d'15'				;Rcksetzen der Rampenregister mit Startwerten
	movwf RA_HS_RAMPE
	movwf DE_HS_RAMPE
	clrf STAT_RAMPE

	btfss STAT_MOTOR,0		;Prfen ob Bit fr RA Motor "Ein" gesetzt ist (=> Enable)
	goto loop_test_end		;Bit ist nicht gesetzt, Abbruch
	bsf PCLATH,3			;Select Page 1
	call ra_richtung
	call de_richtung
	bcf PCLATH,3			;Select Page 0	

	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_off			;Cursor ausschalten
	movlw d'250'
	call wait				;Wartezeit von 250ms
	bcf PCLATH,4			;Select Page 0
	movlw 0xFF
	movwf NEU_DISP			;setze Status Bit Register fr Display auf Wert 255 (11111111)
	movwf NEU1_DISP
ra_set_highspeed
	btfss STAT_POS,3		;prfen ob Statusbit fr Erreichen GOTO Position RA gesetzt ist
	goto $+4				;Statusbit fr Erreichen GOTO Position RA nicht gesetzt,weiter
	bsf SPEED,2				;setze bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	bsf SPEED,3				;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)	
	goto de_set_highspeed	;Statusbit ist gesetzt, weiter bei DE	
	bsf STAT_POS,5			;Statusbit nicht gesetzt, setze Goto Enable Bit RA
	bcf OUT_RA_HL			;setze Ausgang auf low => Max. Geschwindigkeit RA
de_set_highspeed
	btfsc STAT_POS,4		;prfen ob Statusbit fr Erreichen GOTO Position DE nicht gesetzt ist
	goto goto_loop			;Statusbit ist gesetzt, weiter bei LOOP
	bcf SPEED,6				;Statusbit nicht gesetzt, lsche bit fr DE Up/Down
	bsf STAT_POS,6			;Statusbit nicht gesetzt, setze Goto Enable Bit DE
	bcf OUT_DE_HL			;setze Ausgang auf low => Max. Geschwindigkeit DE
goto_loop
	clrf BEFEHL
	btfss DAT_RX			;prfen, ob Slave daten senden will
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus dem Schieberegister 74HC165
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto loop_test_end		;wenn ja berspringe diesen befehl, wenn nein springe ans Ende 
	bsf PCLATH,4			;Select Page 2
	btfsc NEU_DISP,2		;prfe Status bit fr Display RA (ob RA sich gendert hat)
	call disp_ra			;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige RA
	bcf PCLATH,4			;Select Page 0
	btfss NEU_DISP,1		;prfe Status bit fr Display Uhrzeit (Anzeige fr DE soll nur 1 pro sek upgedatet werden)
	goto $+5				;wenn ja, berspringe diesen Befehl
	bcf NEU_DISP,1			;lsche das Status bit
	bsf PCLATH,4			;Select Page 2
	btfsc NEU_DISP,3		;prfe Status bit fr Display DE (ob DE sich gendert hat)
	call disp_de			;wenn nein, berspringe diesen Befehl, wenn ja Aufruf Unterprogramm zur Anzeige DE
	bcf PCLATH,4			;Select Page 0

	btfsc OUT_RA_HL			;prfen, ob Ausgang low ist	
	bsf STAT_POS,3			;Ausgang ist high, setze Statusbit fr Erreichen der RA GOTO Position	

	btfsc OUT_DE_HL			;prfen, ob Ausgang low ist	
	bsf STAT_POS,4			;setze Statusbit fr Erreichen der DE GOTO Position	

loop_test
	btfss STAT_POS,3		;prfen ob Goto Position RA erreicht ist
	goto goto_loop			;wenn ja berspringe diesen Befehl
	btfss STAT_POS,4		;prfen ob Goto Position DE erreicht ist
	goto goto_loop			;wenn ja berspringe diesen Befehl
loop_test_end
	bsf OUT_RA_HL
	bsf STAT_RAMPE,1		;setze Bit fr Ende Bremsrampe RA
	bsf SPEED,6				;setze Bit
	bsf OUT_DE_HL
	bsf STAT_RAMPE,3		;setze Bit fr Ende Bremsrampe DE
	call set_ra_rechts		;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	goto abfr_speich


menu_pos_set				;Menpunkt 2
	movlw d'2'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'
	subwf TEMP_DIG_CNT,w
	btfsc STATUS,Z
	goto menu_pos_man
	bsf PCLATH,3			;Select Page 1
	call menu_eeprom
	bcf PCLATH,3			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 
menu_pos_man
	movlw d'3'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	bsf PCLATH,4			;Select Page 2
	call disp_set_pos		;
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	call menu_rade_man		;Aufruf des Unterprogramms zum setzen der Parameter fr RA/DE manuell
	bcf PCLATH,3			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 


copy_pos_man				; Kopiere Temp register in RA + DE Register
	call conv_dig_rade		;Konvertierung der einzelnen Digits in Werte fr RA/DE

copy_pos_eepr
	movfw STD_TEMP			;kopiere die temporren Werte in die aktiven Register
	movwf STD_RA
	movfw MIN_TEMP
	movwf MIN_RA
	movfw SEK_TEMP
	movwf SEK_RA
	movfw GRD_TEM
	movwf TEMP_18
	movfw MIN_TEM
	movwf TEMP_17
	bcf STATUS,C			;lschen des Carry Bis, damit es beim verschieben nicht strt
	rrf MIN_TEM_01,f		;teilen durch 2, mit diesem Befehl und dem nchsten Befehl wird sichergestellt, das der Wert 0,2,4,6,8 betrgt
	bcf STATUS,C			;lschen des Carry Bis, damit es beim verschieben nicht strt
	rlf MIN_TEM_01,f		;multiplizieren mt 2,
	movfw MIN_TEM_01
	movwf TEMP_16
	movfw PM_TEM
	bsf PCLATH,4				;Select Page 2	
	call conv_de_disp_int		;Unterprogramm zum Konvertieren von DE Werten -90=>0=>+90 in 0=>180
	bcf PCLATH,4				;Select Page 0
	movfw TEMP_18				;Kopieren der konvertierten DE Werte in die Goto Register
	movwf GRD_DE				;
	movfw TEMP_17
	movwf MIN_DE				;
	movfw TEMP_16
	movwf MIN_DE_01


copy_pos
	clrf STAT_POS			;Lschen des Registers und prfen ob bits fr DE Endpositionen 0/+90/-90 gesetzt werden mssen 
	movlw d'90'				;prfen, ob Grad DE <90
	subwf GRD_DE,w
	btfss STATUS,C
	bsf STAT_POS,0			;<90, setze Status Bit
	movfw GRD_DE			;hole GRD_DE ins W-Register und prfe ob 0. Wenn 0 Prfung des nchsten Wertes
	btfss STATUS,Z
	goto $+8
	movfw MIN_DE			;hole MIN_DE ins W-Register und prfe ob 0. Wenn 0 Prfung des nchsten Wertes
	btfss STATUS,Z
	goto $+5
	movfw MIN_DE_01			;hole MIN_DE_01 ins W-Register und prfe ob 0. Wenn 0 setze Statusbit fr 0 Endposition
	btfss STATUS,Z
	goto $+2
	bsf STAT_POS,2			;setze Statusbit fr 0 Endposition
	movlw d'180'			;prfen ob 180 Position erreicht ist 
	subwf GRD_DE,w
	btfss STATUS,Z
	goto copy_pos1
	bsf STAT_POS,1			;setze Statusbit fr +180 Endposition
copy_pos1
	goto abfr_speich

abfr_speich					;Nach Goto oder Position wird automatisch gefragt, ob die Position gespeichert werden soll
	bsf PCLATH,4			;Select Page 2, Die Abfrage soll aber nur 3-4 Sek erfolgen um dann wieder ins normale 
	call disp_speich		; Programm zu springen.
	movlw d'250'			;Wartezeit von 250ms
	call wait				;
	bcf PCLATH,4			;Select Page 0
	bcf NEU_DISP,1			;lsche Sekunden berlauf Bit
	movlw 0x03				;lade Zhler mit Wert 3
	movwf LOOP_1
abfr_loop
	clrf BEFEHL
	btfss DAT_RX			;prfen, ob Slave daten senden will
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus dem Schieberegister 74HC165
	btfsc BEFEHL,1			;prfen ob Bit fr Befehl OK gesetzt ist
	goto menu_speichern		;Bit fr Befehl OK ist gesetzt, weiter bei Menpunkt Speichern
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Men gesetzt ist
	goto menue_end			;Bit fr Befehl Men ist gesetzt, d.h. Beendigung des Mens
	btfss NEU_DISP,1		;Prfen, ob eine Sekunde abgelaufen ist
	goto abfr_loop			;Sekunde noch nicht um, zurck zur Schleife
	bcf NEU_DISP,1			;Sekunde ist um, lsche Sekunden berlauf Bit
	decfsz LOOP_1			;Reduziere 3 Sekundenzhler und prfe, ob 0
	goto abfr_loop			;Zhler noch nicht 0, zurck zur Schleife
	goto menue_end			;Zhler ist 0, Beendigung des Mens

menu_speichern
	movlw d'14'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	clrf DIG_REG			;
	bsf DIG_REG,2			;setze bit 2 in Register 
	movlw d'4'				;Cursor auf 1er Katalognummer setzen
	movwf TEMP_DIG_CNT
	bsf STATUS,RP0			;Select BANK_1	

	movfw DATEN_3			;Sichern der Daten
	movwf TBIT_H
	movfw DATEN_4
	movwf TBIT_L
	movfw DATEN_5
	movwf DCF_BIT
	movfw DATEN_6
	movwf DCF_MESS_STAT
	movfw DATEN_7
	movwf DCF_PRUEFBIT
	movfw DATEN_8
	movwf DCF_COUNT
	movfw DATEN_9
	movwf TAG_UZ
	movfw DATEN_10
	movwf WTAG_UZ
	movfw STD_TEMP
	movwf MONAT_UZ
	movfw MIN_TEMP
	movwf JAHR_UZ
	movfw SEK_TEMP
	movwf DCF_H_MAX
	movlw A'+'				;Prfen ob Vorzeichen positiv ist
	subwf PM_TEM,w			;
	btfss STATUS,Z			;
	bsf GRD_TEM,7			;wenn ja berspringe diesen Befehl, wenn nein (Vorzeichen negativ) setze Bit 7
	movfw GRD_TEM
	movwf DCF_H_MIN
	movfw MIN_TEM
	movwf DCF_L_MAX
	movfw MIN_TEM_01
	movwf DCF_L_MIN
	movlw b'10100000'
	movwf ADR_DEV
	clrf ADR_H
	clrf ADR_L
suche_freie_adr
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,4			;Select Page 2	
	call Lese_EEPROM_0
	bcf PCLATH,4			;Select Page 0	
	btfss TEMP_13,0
	goto menu_speichern_end	;
	bsf STATUS,RP0			;Select BANK_1	
	movlw 0x10
	addwf ADR_L,f			;Low Byte um 16 erhhen
	btfss STATUS,C
	goto $+2
	incf ADR_H				;Bei berlauf High Byte um 1 erhhen
	movlw 0x06				;Prfen, ob maximale Adresse (dez: 1599, hex:063F ) berschritten ist
	subwf ADR_H,w
	btfss STATUS,Z
	goto suche_freie_adr
	movlw 0x40
	subwf ADR_L,w
	btfss STATUS,Z
	goto suche_freie_adr
	goto copy_speich_end
menu_speichern_end
	bsf STATUS,RP0			;Select BANK_1	
	movfw ADR_L
	movwf KAT_L
	movfw ADR_H
	movwf KAT_H
	movlw b'10100000'
	movwf ADR_DEV
	call conv_adr_kat		;Adresse durch 16 teilen => Katalognummer, 4x nach rechts schieben

	movfw KAT_H
	movwf ZAHL_C1
	movfw KAT_L
	movwf ZAHL_C0
	clrf ZAHL_C2
	bsf PCLATH,3			;Select Page 3		
	bsf PCLATH,4			;
	call convert_hex_dig
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2	
	call disp_kat_no
	call Lese_EEPROM_1
	call disp_ee
	bcf PCLATH,4			;Select Page 0		
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte

copy_speichern
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_H
	btfss STATUS,Z
	goto copy_speich_end	;H-Byte >0<, Abbruch Speichern	
	movlw d'100'			;Maximale Katalognr. = 99, prfen, ob L-Byte <100
	subwf KAT_L,w
	btfsc STATUS,C			
	goto copy_speich_end	;L-Byte >=100, Abbruch Speichern
	clrf DATEN_1			;Kopiere zwischengespeicherte Katalog-Nr nach DATEN Register und Adress Register
	clrf ADR_H
	movfw KAT_L			;
	movwf DATEN_2
	movwf ADR_L

	call conv_kat_adr		;Katalognummer mit 16 multiplizieren => Adresse, 4x nach links schieben
	movfw TBIT_H			;Rcksichern der Daten in die Schreibregister fr das externe EEPROM
	movwf DATEN_3
	movfw TBIT_L
	movwf DATEN_4
	movfw DCF_BIT
	movwf DATEN_5
	movfw DCF_MESS_STAT
	movwf DATEN_6
	movfw DCF_PRUEFBIT
	movwf DATEN_7
	movfw DCF_COUNT
	movwf DATEN_8
	movfw TAG_UZ
	movwf DATEN_9
	movfw WTAG_UZ
	movwf DATEN_10
	movfw MONAT_UZ
	movwf STD_TEMP	
	movfw JAHR_UZ
	movwf MIN_TEMP	
	movfw DCF_H_MAX
	movwf SEK_TEMP	
	movfw DCF_H_MIN
	movwf GRD_TEM	
	movfw DCF_L_MAX
	movwf MIN_TEM	
	movfw DCF_L_MIN
	movwf MIN_TEM_01	
	movlw b'10100000'
	movwf ADR_DEV	
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,4			;Select Page 2
	call Schreibe_EEPROM	;Daten werden ins EEPROM geschrieben
	bcf PCLATH,4			;Select Page 0
copy_speich_end				;
	bcf STATUS,RP0			;Select BANK_0	
	goto menue_end
	


menu_speed_set				;Menpunkt 3
	bsf SPEED,4				;setze Bit 4+5 fr solar/lunar auf 1
	bsf SPEED,5				;
	bsf SPEED,1				;setze Statusbit fr Update Display mit Speed Werten

	movfw TEMP_DIG_CNT		;Kopiere die Geschwindigkeit in Temp. Reg. (stellar=1, solar=2, lunar=3)
	movwf TEMP_13
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_parameter1	;Lesen und kopieren der Parameter Zhler A/B/C fr st/so/lu aus internem EEPROM
	call schreibe_speed		;schreibe die Geschwindigkeit (stellar=1, solar=2, lunar=3) ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menue_end

menu_motor_set				;Menpunkt 4
	movlw d'4'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	clrf DIG_REG			;
	bsf DIG_REG,6			;setze bit 6 in Register 
	movfw STAT_MOTOR		;Kopiere Motor Statusregister in Temp register
	movwf TEMP_14
	bsf PCLATH,4			;Select Page 2
	call disp_set_motor		;
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_motor	
	movfw TEMP_14			;Kopiere Temp register in Motor Statusregister
	movwf STAT_MOTOR
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_mot_stat	;Aufruf Unterprogramm zum schreiben der Motor Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	btfss STAT_MOTOR,0		;prfen ob Bit fr RA Motor "Ein" gesetzt ist
	bcf OUT_RA_ENABLE		;Bit ist nicht gesetzt, d.h. Motor "Aus"
	btfsc STAT_MOTOR,0		;prfen ob Bit fr RA Motor "Ein" nicht gesetzt ist
	bsf OUT_RA_ENABLE		;Bit ist gesetzt, d.h. Motor "Ein"
	btfss STAT_MOTOR,1		;prfen ob Bit fr DE Motor "Ein" gesetzt ist
	bcf OUT_DE_ENABLE		;Bit ist nicht gesetzt, d.h. Motor "Aus"
	btfsc STAT_MOTOR,1		;prfen ob Bit fr DE Motor "Ein" nicht gesetzt ist
	bsf OUT_DE_ENABLE		;Bit ist gesetzt, d.h. Motor "Ein"
	goto menue_end

menu_anzeige_set			;Menpunkt 5
	movlw d'1'				;wenn Zhler = 1 setze Bit fr Anzeige Ra/De
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebnis = 0	
	goto $+5				;Ergebnis ist <0>, weiter bei $+5
	bsf NEU_DISP,5			;setze Bit in Display Statusregister (Anzeige RA/DE)
	bsf NEU_DISP,2			;setze Bit in Display Statusregister fr RA
	bsf NEU_DISP,3			;setze Bit in Display Statusregister fr DE
	goto menue_end			;gehe zur Beendigung des Mens
	bcf NEU_DISP,5			;lsche Bit in Display Statusregister (Anzeige SZ/UZ)
	bsf NEU_DISP,0			;setze Bit in Display Statusregister fr Sternzeit
	bsf NEU_DISP,1			;setze Bit in Display Statusregister fr Uhrzeit
	goto menue_end			;gehe zur Beendigung des Mens


menu_uz_set					;Menpunkt 6
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	btfsc TEMP_DIG_CNT,1	;wenn Zhler =2, gehe zu update der Uhr ber DCF 77 Signal
	goto update_uhr
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	movlw d'5'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'8'				;Anzeige hat 8 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	clrf DIG_REG			;
	bsf DIG_REG,3			;setze bit 3 in Register 
	bsf STATUS,RP0			;Select BANK_1		
	clrf DATEN_1			;Zeitzone
	clrf DATEN_2			;Minute
	clrf DATEN_3			;Stunde
	clrf DATEN_5			;Sekunde
	movlw 0x01
	movwf DATEN_4			;Tag
	movwf DATEN_6			;Monat
	clrf DATEN_7			;Jahr
	movlw A'+'				;Vorzeichen fr Zeitzone
	movwf DATEN_8
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_set_uz
	bcf PCLATH,4			;Select Page 0

	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_uz						;Kopiere Temp register in Uhrzeit Register
	bcf INTCON,GIE			;verbiete globale Interrupts
	movlw 0x1F				;Laden der Konstanten fr den Uhrzeittakt, damit Zhler mit voller Sekunde beginnt
	movwf COUNT_H_UZ		;
	movlw 0x84
	movwf COUNT_L_UZ
	movlw 0x02				;Setzen des Zhlers, Digit:= 2	
	movwf DIGIT_UZ			;
	bsf STATUS,RP0			;Select BANK_1, kopieren der temporren Register in aktive Register
	movlw A'+'				;prfen, ob Vorzeichen Zeitzone = +
	subwf DATEN_8,w
	btfss STATUS,Z
	bsf DATEN_1,7			;Vorzeichen Zeitzone = -, setze Bit 7 in Zeitzonen Register
	movfw DATEN_3
	bcf STATUS,RP0			;Select BANK_0	
	movwf STD_UZ
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_2
	bcf STATUS,RP0			;Select BANK_0	
	movwf MIN_UZ
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_5
	bcf STATUS,RP0			;Select BANK_0	
	movwf SEK_UZ			;kopiere den kombinierten Wert ins aktive Register
	bsf INTCON,GIE			;erlaube globale Interrupts
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call sternzeit			;Aufruf des Unterprogramms zur Sternzeitberechnung	
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	bcf NEU_DISP,5			;lsche Bit fr Anzeige UZ/SZ	
	goto menue_end


menu_loeschen				;Menpunkt 7
	movlw d'6'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'2'				;Katalog "Own" whlen
	movwf TEMP_DIG_CNT
	bsf PCLATH,3			;Select Page 1
	call menu_eeprom
	bcf PCLATH,3			;Select Page 0
	bsf STATUS,RP0			;Select BANK_1	
	movlw b'10100000'
	movwf ADR_DEV
	movlw 0x06				;lade hchste Adresse (dez: 1584, hex:0630, Katalognr.99 ) in Adressregister
	movwf ADR_H
	movlw 0x30
	movwf ADR_L
suche_daten
	bcf STATUS,RP0			;Select BANK_0	
	bsf PCLATH,4			;Select Page 2	
	call Lese_EEPROM_0
	bcf PCLATH,4			;Select Page 0	
	btfsc TEMP_13,0
	goto menu_speichern_end	;
	bsf STATUS,RP0			;Select BANK_1	
	movfw ADR_H				;prfen, ob H-Byte schon 0 ist
	btfss STATUS,Z
	goto $+4
	movfw ADR_L				;prfen, ob L-Byte auch 0 ist
	btfsc STATUS,Z
	goto copy_loesch_end
	movlw 0x10
	subwf ADR_L,f			;Low Byte um 16 erhhen
	btfsc STATUS,C
	goto suche_daten
	decf ADR_H				;Bei berlauf High Byte um 1 erhhen
	goto suche_daten
	goto copy_loesch_end

copy_loeschen
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_H
	btfss STATUS,Z
	goto copy_loesch_end	;H-Byte >0<, Abbruch Speichern	
	movlw d'100'			;Maximale Katalognr. = 99, prfen, ob L-Byte <100
	subwf KAT_L,w
	btfsc STATUS,C			
	goto copy_loesch_end	;L-Byte >=100, Abbruch Speichern
	clrf DATEN_1			;Kopiere zwischengespeicherte Katalog-Nr nach DATEN Register und Adress Register
	clrf ADR_H
	movfw KAT_L			;
	movwf DATEN_2
	movwf ADR_L

	call conv_kat_adr		;Katalognummer mit 16 multiplizieren => Adresse, 4x nach links schieben

	movlw b'10100000'
	movwf ADR_DEV	
	clrf DATEN_3
	clrf DATEN_4
	clrf DATEN_5
	clrf DATEN_6
	clrf DATEN_7
	clrf DATEN_8
	clrf DATEN_9
	clrf DATEN_10
	bcf STATUS,RP0			;Select BANK_0	
	clrf STD_TEMP			;
	clrf MIN_TEMP			;
	clrf SEK_TEMP			;
	clrf GRD_TEM			;
	clrf MIN_TEM			;
	clrf MIN_TEM_01			;
	bsf PCLATH,4			;Select Page 2
	call Schreibe_EEPROM	;Daten werden ins EEPROM geschrieben
	bcf PCLATH,4			;Select Page 0
copy_loesch_end				;
	bcf STATUS,RP0			;Select BANK_0	
	goto menu_loeschen


menu_einstell_set			;Menpunkt 8: Einstellungen
	movlw 0x06				;das Einstellungsmen hat 6 Menpunkte
	movwf MENUE_CNT_MAX
	movlw 0x01
	movwf MENUE_COUNT		;vertikalen Menpunkt Zhler auf 1 setzen
	movwf TEMP_DIG_CNT		;horizontalen Menpunkt Zhler auf 1 setzen
	bsf PCLATH,4			;Select Page 2
	call disp_einstell
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	call dig_einst_hor
	bcf PCLATH,3			;Select Page 0
menu_einstell_loop
	bsf PCLATH,4			;Select Page 2
	btfsc NEU_DISP,4
	call disp_einstell
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;
	btfsc TEMP_DIG_STAT,0
	call dig_einst_hor
	bcf PCLATH,3			;Select Page 0
	clrf BEFEHL
	btfsc DAT_RX			;prfen, ob Slave daten senden will
	goto $-1	
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus dem Slave
	btfsc BEFEHL,0			;Prfen ob "Men" Taste gedrckt ist (high ist)	
	goto menue_end			;
	btfsc BEFEHL,5			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	call menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	call menue_1up			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_cnt		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)	
	goto menu_einstell_loop	;

	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	bcf PCLATH,4			;Select Page 0
	movlw d'1'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_standort_set	; Aufruf des Programmteils menu_standort_set
	movlw d'2'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_para_set		; Aufruf des Programmteils menu_para_set
	movlw d'3'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_dcfpara_set	; Aufruf des Programmteils menu_dcfpara_set
	movlw d'4'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_encoder_set	; Aufruf des Programmteils menu_encoder_set
	movlw d'5'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_q1hz_set		; Aufruf des Programmteils menu_q1hz_set
	movlw d'6'				;
	subwf MENUE_COUNT, w	;
	btfsc STATUS, Z		
	goto menu_handbox_set	; Aufruf des Programmteils menu_handbox_set
	goto menue_end

menu_standort_set			;Menpunkt 8.1
	movlw d'7'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	clrf DIG_REG			;
	bsf DIG_REG,4			;setze bit 4 in Register 
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_standort		;Unterprogramm zum lesen der Standortdaten (geogr. Lnge) aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call disp_set_standort
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_standort
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_standort	;Unterprogramm zum schreiben der Standortdaten (geogr. Lnge) in das interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menue_end	

menu_para_set				;Menpunkt 8.2
	clrf DIG_REG			;
	movlw d'1'
	subwf TEMP_DIG_CNT,w
	btfsc STATUS,Z
	goto menu_para_ra_aut	;Automatisch ist gewhlt
	bsf DIG_REG,1			;setze bit 1 in Register 
	movlw d'3'				;Anzeige hat 3 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
menu_pa_st_set
	movlw d'8'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_pa_st			;Aufruf des Unterprogramms zum Lesen der Stellar Werte aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call disp_past_set
	call disp_para_dat
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_st					;Kopiere Temp register in Stellar Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_st		;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menu_pa_so_set

menu_pa_so_set
	movlw d'9'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_pa_so			;Aufruf des Unterprogramms zum Lesen der Stellar Werte aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_paso_set
	call disp_para_dat
	bsf TEMP_DIG_STAT,0
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_so					;Kopiere Temp register in Solar Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_so		;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menu_pa_lu_set

menu_pa_lu_set
	movlw d'10'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_pa_lu			;Aufruf des Unterprogramms zum Lesen der Stellar Werte aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_palu_set
	call disp_para_dat
	bsf TEMP_DIG_STAT,0
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_lu					;Kopiere Temp register in Lunar Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_lu		;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menu_pa_ramxde_set

menu_pa_ramxde_set
	movlw d'11'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movfw RA_HIGH_L			;lade RA Max Wert in Temp Register
	movwf STD_TEMP
	movfw DE_NORM_L			;lade DE Norm Wert in Temp Register
	movwf MIN_TEMP
	movfw DE_HIGH_L			;lade DE Max Wert in Temp Register
	movwf SEK_TEMP
	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_ramxde_set
	call disp_para_dat
	bsf TEMP_DIG_STAT,0
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_ramxde				;Kopiere Temp register in RA max, DE norm, DE max Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_ramxde	;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menu_pa_patra_set

menu_pa_patra_set
	movlw d'12'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movfw RA_ENC_A			;lade RA Encoder Wert n-1 in Temp Register
	movwf STD_TEMP
	movfw RA_ENC_B			;lade RA Encoder Wert n in Temp Register
	movwf MIN_TEMP
	movfw RA_ENC_C			;lade Wert fr Loops fr wechsel n/n-1 in Temp Register
	movwf SEK_TEMP
	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_patra_set
	call disp_para_dat
	bsf TEMP_DIG_STAT,0
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_patra				;Kopiere Temp register in RA Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_patra	;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menu_pa_patde_set

menu_pa_patde_set
	movlw d'13'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movfw DE_ENC_A			;lade DE Encoder Wert n-1 in Temp Register
	movwf STD_TEMP
	movfw DE_ENC_B			;lade DE Encoder Wert n in Temp Register
	movwf MIN_TEMP
	movfw DE_ENC_C			;lade Wert fr Loops fr wechsel n/n-1 in Temp Register
	movwf SEK_TEMP
	bsf PCLATH,4			;Select Page 2
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call disp_patde_set
	call disp_para_dat
	bsf TEMP_DIG_STAT,0
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_patde				;Kopiere Temp register in DE Register
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_pa_patde	;Aufruf des Unterprogramms zum Schreiben der Daten ins interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menue_end

menu_para_ra_aut			;Unterprogramm zur automatischen Berechnung der Parameter, RA - Achse
	movlw d'7'				;Anzeige hat 7 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movlw d'16'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	clrf DIG_REG			;
	bsf DIG_REG,7			;setze bit 7 in Register 
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call lese_ratio_ra		;Unterprogramm zum Lesen des bersetzungsverhltnis Iges und n-max aus dem internen EEPROM
	call convert_hex_dig	;Umwandeln der 24 Bit Zahl in die 6 einzelnen Digits des Display fr Iges
							; die 6 Digits stehen in den Registern Kat 5 bis Kat 0, n-max in ZZ UZ
	bcf PCLATH,3			;Select Page 2
	call disp_set_para_ra
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_ra_aut
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call convert_dig_hex	;Umwandeln der 6 einzelnen Digits des Display fr Iges in eine 24 Bit Zahl
	call schreibe_ratio_ra	;schreibe bersetzungsverhltnis Iges und n-max in das interne EEPROM
	bsf STATUS,RP0			;Select BANK_1
	movfw ZAHL_C0			;Sichern des Ergebnis in KAT_0-2 Registern
	movwf KAT_0
	movfw ZAHL_C1
	movwf KAT_1
	movfw ZAHL_C2
	movwf KAT_2
	call calc_para_ra
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4

menu_para_de_aut			;Unterprogramm zur automatischen Berechnung der Parameter, DE - Achse
	movlw d'7'				;Anzeige hat 7 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movlw d'17'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	clrf DIG_REG			;
	bsf DIG_REG,7			;setze bit 7 in Register 
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4
	call lese_ratio_de		;Unterprogramm zum Lesen des bersetzungsverhltnis Iges und n-max aus dem internen EEPROM
	call convert_hex_dig	;Umwandeln der 24 Bit Zahl in die 6 einzelnen Digits des Display fr Iges
							; die 6 Digits stehen in den Registern Kat 5 bis Kat 0, n-max in ZZ UZ
	bcf PCLATH,3			;Select Page 2
	call disp_set_para_de
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_pa_de_aut
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call convert_dig_hex	;Umwandeln der 6 einzelnen Zahlen des Display fr Iges in eine 24 Bit Zahl
	call schreibe_ratio_de	;schreibe bersetzungsverhltnis Iges und n-max in das interne EEPROM
	bsf STATUS,RP0			;Select BANK_1
	movfw ZAHL_C0			;Sichern des Ergebnis in KAT_0-2 Registern
	movwf KAT_0
	movfw ZAHL_C1
	movwf KAT_1
	movfw ZAHL_C2
	movwf KAT_2
	call calc_para_de
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4
	goto menue_end

menu_dcfpara_set			;Menpunkt 8.3: DCF Parameter, Min/Max "Bitlngen"
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movlw d'15'				;Adressregister fr Menroutine
	movwf MENUE_ADR
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	clrf DIG_REG			;
	bsf DIG_REG,5			;setze bit 5 in Register 
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call lese_dcfpara		;Unterprogramm zum lesen der Min/Max "Bitlngen" aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call disp_set_dcfpara
	bcf PCLATH,4			;Select Page 0
	goto menu_routine		;Aufruf des Unterprogramms zum Einstellen der Werte 

copy_dcfpara
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call schreibe_dcfpara	;Unterprogramm zum schreiben der Min/Max "Bitlngen" in das interne EEPROM
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menue_end	

menu_encoder_set			;Menpunkt 8.4: Wahl Encoder "Intern/Extern"
	bcf INTCON,RBIF			;lsche RB Port Change Interrupt Flag
	btfss TEMP_DIG_CNT,0	;prfen ob Zhler =1 (Encoder Intern)
	goto $+4				;nicht 1, d.h. Encoder Extern ist gewhlt
	bsf STAT_MOTOR,7		;setze Bit
	bcf INTCON,RBIE			;disable RB Port Change Interrupts
	goto $+3
	bcf STAT_MOTOR,7		;lsche Bit
	bsf INTCON,RBIE			;enable RB Port Change Interrupts
	movfw STAT_MOTOR
	movwf TEMP_13
	bcf TEMP_13,4
	bcf TEMP_13,5
	bcf TEMP_13,6
	movlw 0x1B
	bsf PCLATH,3			;Select Page 3
	bsf PCLATH,4			;
	call writeEE			;
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	goto menue_end		

menu_q1hz_set				;Menpunkt 8.5: Ermittlung der Oszillator Abweichung in ppm mittels DCF77 Signal
	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto quarz1hz_set		

menu_handbox_set			;Menpunkt 8.6: Einstellungen fr Heizung und Kontrast der Handbox
	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 
	movlw 0xF0				;Mit diesem Befehl wird in der Handbox das Handbox Men aufgerufen.
	call disp_befehl	
	bcf PCLATH,4			;Select Page 0
	btfsc DAT_RX			;prfen, ob Slave daten senden will
	goto $-1
	goto menue_end		

menue_end
	movlw 0xFF				;setze bit 0-7 auf high (=255) in Display Statusregister, wenn Bit 5 gesetzt ist
	movwf NEU1_DISP			;
	btfss NEU_DISP,5		;prfen, ob das Bit fr Anzeige Ra/De gesetzt ist
	goto $+3				;Bit 5 ist nicht gesetzt, rcksichern des Bits bei $+3
	movwf NEU_DISP			;Bit 5 ist gesetzt, d.h. Anzeige Ra/De
	goto $+3				;Bit 5 ist gesetzt
	movwf NEU_DISP			;
	bcf NEU_DISP,5			;rcksichern des Bits, Anzeige UZ/SZ
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	clrf TEMP_DIG_STAT
	clrf DIG_REG			;lsche Register
	bsf SPEED,1				;setze Statusbit fr Update Display mit Speed Werten
	movlw d'250'			;Wartezeit von 250ms
	bsf PCLATH,4			;Select Page 2
	call wait				;
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays	
	call cursor_off			;Cursor ausschalten
	bcf PCLATH,4			;Select Page 0
	goto main

menue_1up
	incf MENUE_COUNT		;Erhhe Menpunktzhler
	movfw MENUE_CNT_MAX		;hole maximale Anzahl nach W und addiere 1
	addlw 0x01
	subwf MENUE_COUNT,w		;prfen, ob maximale Anzahl um 1 berschritten wurde
	btfss STATUS,Z		
	goto menue_1updn_end	;nein, keine berschreitung
	movlw d'1'				;ja, maximale Anzahl wurde berschritten, rcksetzen des Zhlers auf 1
	movwf MENUE_COUNT
	goto menue_1updn_end
menue_1dn
	decfsz MENUE_COUNT		;Reduziere Menpunktzhler und prfe ob 0
	goto menue_1updn_end	;noch nicht 0
	movfw MENUE_CNT_MAX		;Menpunktzhler ist 0, rcksetzen des Zhlers auf maximale Anzahl Menpunkte
	movwf MENUE_COUNT
menue_1updn_end
	movlw d'1'				;lade horizontalen Zhler mit Wert 1
	movwf TEMP_DIG_CNT		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit fr update Cursorposition in Menpunkten Goto, Position und Speed
	bsf NEU_DISP,4			;setze das Statusbit fr update Display
	movlw d'250'			;Wartezeit von 250ms
	bsf PCLATH,4			;Select Page 2
	call wait 
	bcf PCLATH,4			;Select Page 0
	return

set_ra_rechts
	bcf INTCON,GIE				;Interrupts verbieten
	btfss STAT_MOTOR,2			;Prfen ob Bit fr invertierte Drehrichtung RA gesetzt ist
	bsf OUT_RA_RL				;Bit ist nicht gesetzt, d.h. normale Drehrichtung => rechts
	btfsc STAT_MOTOR,2			;Prfen ob Bit fr invertierte Drehrichtung RA nicht gesetzt ist
	bcf OUT_RA_RL				;Bit ist gesetzt, d.h. invertierte Drehrichtung => links
	bsf INTCON,GIE				;Interrupts erlauben
	return

set_ra_links
	bcf INTCON,GIE				;Interrupts verbieten
	bcf SPEED,2					;lsche bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	btfss STAT_MOTOR,2			;Prfen ob Bit fr invertierte Drehrichtung RA gesetzt ist
	bcf OUT_RA_RL				;Bit ist nicht gesetzt, d.h. normale Drehrichtung => links
	btfsc STAT_MOTOR,2			;Prfen ob Bit fr invertierte Drehrichtung RA nicht gesetzt ist
	bsf OUT_RA_RL				;Bit ist gesetzt, d.h. invertierte Drehrichtung => rechts
	bsf INTCON,GIE				;Interrupts erlauben
	return

set_de_up
	bcf INTCON,GIE				;Interrupts verbieten
	bsf SPEED,7					;setze Bit (Bit 7: 0 = down, 1 = up)
	btfss STAT_MOTOR,3			;Prfen ob Bit fr invertierte Drehrichtung RA gesetzt ist
	bsf OUT_DE_RL				;Bit ist nicht gesetzt, d.h. normale Drehrichtung => rechts
	btfsc STAT_MOTOR,3			;Prfen ob Bit fr invertierte Drehrichtung RA nicht gesetzt ist
	bcf OUT_DE_RL				;Bit ist gesetzt, d.h. invertierte Drehrichtung => links
	bsf INTCON,GIE				;Interrupts erlauben
	return

set_de_down
	bcf INTCON,GIE				;Interrupts verbieten
	bcf SPEED,7					;lsche Bit (Bit 7: 0 = down, 1 = up)
	btfss STAT_MOTOR,3			;Prfen ob Bit fr invertierte Drehrichtung RA gesetzt ist
	bcf OUT_DE_RL				;Bit ist nicht gesetzt, d.h. normale Drehrichtung => links
	btfsc STAT_MOTOR,3			;Prfen ob Bit fr invertierte Drehrichtung RA nicht gesetzt ist
	bsf OUT_DE_RL				;Bit ist gesetzt, d.h. invertierte Drehrichtung => rechts
	bsf INTCON,GIE				;Interrupts erlauben
	return

menu_routine
	bsf PCLATH,4			;Select Page 2
	call cursor_on
	bcf PCLATH,4			;Select Page 0
	bsf TEMP_DIG_STAT,0		;setze Statusbit
menu_routine_loop
	bsf PCLATH,3			;Select Page 1
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call incdec_test		;wenn ja berspringe diesen Befehl (dig_temp, dig_temp_para)
	bcf PCLATH,3			;Select Page 0
	clrf BEFEHL
	btfsc DAT_RX			;prfen, ob Slave daten senden will
	goto $-1
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle aus dem Schieberegister 74HC165
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto menue_end			;wenn ja berspringe diesen befehl, wenn nein weiter bei Beendigung Men
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_cnt		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_cnt		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1	
	btfsc BEFEHL,4			;Prfen ob "UP" Taste nicht gedrckt ist (low ist)
	bsf TEMP_DIG_STAT,1		;"UP" Taste ist gedrckt,setze Statusbit fr "UP"
	btfsc BEFEHL,4			;Prfen ob "UP" Taste nicht gedrckt ist (low ist)
	call incdec_test		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,5			;Prfen ob "DOWN" Taste nicht gedrckt ist (low ist)
	call incdec_test		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto menu_routine_loop	;nein, zurck zur Schleife 
	bcf STATUS,RP0			;Select Bank 0
	goto menu_routine1		;wenn OK gegrckt, weiter bei entsprechendem Unterprogramm gem MENUE_ADR


inc_dig_cnt
	movfw TEMP_CNT_MAX		;
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z			;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4				;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf TEMP_DIG_CNT
	goto incdec_cnt_end
	incf TEMP_DIG_CNT
	goto incdec_cnt_end
dec_dig_cnt
	decfsz TEMP_DIG_CNT		;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto incdec_cnt_end		;wenn ja, berspringe diesen Befehl	
	movfw TEMP_CNT_MAX		;
	movwf TEMP_DIG_CNT
incdec_cnt_end
	bsf TEMP_DIG_STAT,0		;setze das Status bit
	bsf PCLATH,4			;Select Page 2
	movlw d'250'			;Wartezeit von 250ms
	call wait 		
	bcf PCLATH,4			;Select Page 0
	return		

conv_kat_adr
	movlw 0x04				;Katalognummer mit 16 multiplizieren => Adresse, 4x nach links schieben
	movwf LOOP_2
	bcf STATUS,C
	rlf ADR_L
	rlf ADR_H
	decfsz LOOP_2
	goto $-4
	return

conv_adr_kat
	movlw 0x04				;Adresse durch 16 teilen => Katalognummer, 4x nach rechts schieben
	movwf LOOP_2
	bcf STATUS,C
	rrf KAT_H
	rrf KAT_L
	decfsz LOOP_2
	goto $-4
	return

conv_dig_rade				;Konvertierung der einzelnen Digits in Werte fr RA/DE
	movfw STD_TEMP
	movwf TEMP_14
	movfw STD_TEMP_1
	bsf PCLATH,4			;Select Page 2	
	call combine_2dig		;Aufruf des Unterprogramms zum kombinieren von z.B. 1,2 zu 12
	movwf STD_TEMP			;kopiere den kombinierten Wert ins aktive Register
	movfw MIN_TEMP
	movwf TEMP_14
	movfw MIN_TEMP_1
	call combine_2dig		;Aufruf des Unterprogramms zum kombinieren von z.B. 1,2 zu 12
	movwf MIN_TEMP			;kopiere den kombinierten Wert ins aktive Register
	movfw SEK_TEMP
	movwf TEMP_14
	movfw SEK_TEMP_1
	call combine_2dig		;Aufruf des Unterprogramms zum kombinieren von z.B. 1,2 zu 12
	movwf SEK_TEMP			;kopiere den kombinierten Wert ins aktive Register
	movfw GRD_TEM
	movwf TEMP_14
	movfw GRD_TEM_1
	call combine_2dig		;Aufruf des Unterprogramms zum kombinieren von z.B. 1,2 zu 12
	movwf GRD_TEM			;kopiere den kombinierten Wert ins aktive Register
	movfw MIN_TEM
	movwf TEMP_14
	movfw MIN_TEM_1
	call combine_2dig		;Aufruf des Unterprogramms zum kombinieren von z.B. 1,2 zu 12
	bcf PCLATH,4			;Select Page 0	
	movwf MIN_TEM			;kopiere den kombinierten Wert ins aktive Register
	bcf STATUS,C			;lschen des Carry Bis, damit es beim verschieben nicht strt
	rrf MIN_TEM_01,f		;teilen durch 2, mit diesem Befehl und dem nchsten Befehl wird sichergestellt, das der Wert 0,2,4,6,8 betrgt
	bcf STATUS,C			;lschen des Carry Bis, damit es beim verschieben nicht strt
	rlf MIN_TEM_01,f		;multiplizieren mt 2,
	return

;*******************************************************************************************
;	Interrupt Routinen
;*******************************************************************************************

timer0_irp_routine
	bcf	INTCON, TMR0IF			; Interupt-Flag lschen
	bsf NEU_DISP,7				;setze Statusbit fr Timer 0 berlauf
	btfss STAT_MOTOR,0			;Prfen ob Bit fr RA Motor "Ein" gesetzt ist (=> Enable)
	goto deklination			;Bit ist nicht gesetzt, weiter
	btfss OUT_RA_HL				;prfen ob Ausgang high ist
	goto ra_vollschritt			;wenn wahr berspringe diesen Befehl, wenn nicht wahr (low) weiter bei ra_Vollschritt
	decfsz RA_LOW_TEMP			;Reduziere Zhler und prfe ob 0
	goto deklination			;wenn ja berspringe diesen Befehl
	btfss SPEED,2				;Prfen ob das Bit fr x0 gesetzt ist
	goto deklination			;wenn ja berspringe diesen Befehl
	btfsc OUT_RA_SMOT			;prfen ob Motorausgang low ist
	goto $+3					;wenn ja berspringe diesen Befehl und setze Ausgang auf high, wenn nein setze Ausgang auf low bei $+3
	bsf	OUT_RA_SMOT				;
	goto $+2					;weiter bei Rcksetzen des Zhlers
	bcf	OUT_RA_SMOT				;

	movfw RA_LOW_A				;Zhler A = 0, Rcksetzen des Zhlers mit Wert A
	movwf RA_LOW_TEMP
	movfw RA_LOW_C				;hole Zhler C nach W
	btfsc STATUS,Z				;prfen ob Zhler C <0>
	goto encoder_int_ra			;Zhler C =0
	btfsc STAT_MOTOR,4			;Zhler C <0>, Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	incf RA_LOW_TEMP			;Zhler C wird verwendet, Erhhe Zhler A um 1
	decfsz COUNT_L_RA_TEMP		;reduziere Zhler B/C und prfe ob 0
	goto encoder_int_ra			;Zhler B/C noch nicht 0
	btfsc STAT_MOTOR,4			;Zhler B/C = 0, prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	goto $+4					;Zhler C wird verwendet, Rcksetzen auf Zhler B
	bsf STAT_MOTOR,4			;Zhler B wird verwendet, Rcksetzen auf Zhler C
	movfw RA_LOW_C				;hole Wert fr Zhler C nach W
	goto $+3
	bcf STAT_MOTOR,4			;Zhler C wird verwendet, Rcksetzen auf Zhler B
	movfw RA_LOW_B				;hole Wert fr Zhler B nach W
	movwf COUNT_L_RA_TEMP		;Kopiere den Wert B oder C nach Zhler B/C
	goto encoder_int_ra

ra_vollschritt
	decfsz RA_MAX_TEMP			;Reduziere Zhler und prfe ob 0
	goto deklination			;Zhler noch nicht 0
	btfsc OUT_RA_SMOT			;Zhler =0, prfen ob Motorausgang low ist
	goto $+3					;Ausgang ist high, setze Ausgang auf low bei $+3
	bsf	OUT_RA_SMOT				;Ausgang ist low, setze Ausgang auf high 
	goto $+2					;weiter bei Rcksetzen des Zhlers
	bcf	OUT_RA_SMOT				;setze Ausgang auf low

	btfss STAT_RAMPE,6			;Prfe Timer1 berlauf
	goto ra_vs_end				;kein berlauf vorhanden, 
	bcf STAT_RAMPE,6			;Timer1 berlauf, lsche berlauf Bit
	btfsc STAT_RAMPE,0			;prfen ob Bit fr Bremsrampe gesetzt ist
	goto $+5					;Bit ist gesetzt,d.h. Bremsrampe, weiter bei $+
	movfw RA_HS_RAMPE			;Bit nicht gesetzt,d.h. Beschleunigungsrampe, hole Zhlerwert nach W
	btfss STATUS,Z				;  und prfe ob 0
	decf RA_HS_RAMPE			;Zhler noch nicht 0, reduziere Zhler um 1 (Beschleunigungsrampe: Zhler von 15=>0)
	goto ra_vs_end				;Zhler =0, weiter bei ra_vs_end
	incf RA_HS_RAMPE			;Bremsrampe, erhhe Zhler um 1 (Bremsrampe: Zhler von 0=>15)
	btfss RA_HS_RAMPE,4			;prfen, ob Zhler =16 (prfe bit 4)
	goto ra_vs_end				;Zhler <16, weiter bei ra_vs_end
	decf RA_HS_RAMPE			;Zhler =16, reduziere um 1
	bsf STAT_RAMPE,1			;setze Statusbit fr erreichen der unteren Bremsrampenposition

ra_vs_end
	movlw 0x07					;Programmzhler fuer Rampen Tabelle im Bereich 0x07
	movwf PCLATH				;	
	movfw RA_HS_RAMPE			;Hole Zhlerwert nach W
	call tab_rampe				;Zu dem Zhlerwert wird ein Geschwindigkeitswert aus der Tabelle geholt
	movwf RA_MAX_TEMP			;  und ins Highspeed Register kopiert
	movfw RA_HIGH_L				;der Wert fr die Maximale Geschwindigkeit wird nach W geholt
	addwf RA_MAX_TEMP,f			;  und zum Highspeed Register addiert.

encoder_int_ra					;Encoder RA: Auswerten und Rcksetzen der Werte
	btfss STAT_MOTOR,7			;Prfen ob Bit fr interne Positionsencoder gesetzt ist
	goto test_gotopos_ra		;wenn ja berspringe diesen Befehl, wenn nein (Eingang low) weiter bei Int_sz
	clrf RA_ENC_VS_REST			;lsche Register ( wird immer am Ende vom Zhler A abgezogen)

	movfw RA_ENC_C				;hole Zhler C nach W
	btfsc STATUS,Z				;prfen ob Zhler C <0>
	bcf STAT_MOTOR,5			;Zhler C = 0, Zhler B wird verwendet, lsche Bit (B=0, C=1)

	btfss SPEED,2				;prfen, ob Bit fr Drehrichtung rechts gesetzt ist, RA
	goto $+5					;Bit ist nicht gesetzt, d.h. Drehrichtung ist links, RA
	btfsc STAT_ENCODER,2		;Drehrichtung ist rechts, prfen, ob die alte Drehrichtung links war, RA
	goto encoder_int_ra2		;nein, alte Drehrichtung war auch rechts, weiter
	bsf STAT_ENCODER,2			;alte Drehrichtung war links, update Statusregister mit neuer Drehrichtung rechts
	goto $+4
	btfss STAT_ENCODER,2		;Drehrichtung ist links, prfen, ob die alte Drehrichtung rechts war, RA
	goto encoder_int_ra2		;nein, alte Drehrichtung war auch links, weiter
	bcf STAT_ENCODER,2			;alte Drehrichtung war rechts, update Statusregister mit neuer Drehrichtung links
								;Drehrichtung hat sich gendert, d.h. Drehrichtung Encoder mu auch gendert werden
	movfw COUNT_ENC_RA			;subtrahiere Aktuellen Zhler A von max Wert Zhler A
	subwf RA_ENC_A,w
	movwf COUNT_ENC_RA			; und schreibe die Differenz zurck ins Zhler A Register 
	incf COUNT_ENC_RA			;die Differenz muss noch um 1 erhht werden. 
	btfsc STAT_MOTOR,5			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	incf COUNT_ENC_RA			;Zhler C wird verwendet, Erhhe Zhler A um 1

	movfw LOOP_ENC_RA			;hole Zhler B/C nach W	
	btfss STAT_MOTOR,5			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	subwf RA_ENC_B,w			;Zhler B wird verwendet, subtrahiere Aktuellen Zhler B/C von max Wert Zhler B
	btfsc STAT_MOTOR,5			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	subwf RA_ENC_C,w			;Zhler C wird verwendet, subtrahiere Aktuellen Zhler B/C von max Wert Zhler C
	movwf LOOP_ENC_RA			;Differenz wird wieder in Zhler B/C Register zurckgeschrieben
	incf LOOP_ENC_RA			;  und um 1 erhht

encoder_int_ra2
	btfsc OUT_RA_HL				;prfen ob Ausgang low ist, d.h. Vollschritt
	goto encoder_int_ra1		;=>Mikroschritt, berspringe die nchsten Schritte
	movfw COUNT_ENC_RA			;=>Vollschritt, Zhler A um 15 (+1 bei "encoder_int_ra1") reduzieren 
	sublw d'15'					;prfen, ob Zhler A >15: 15-Zhler A
	btfsc STATUS,C				;prfen, ob Ergebnis negativ, d.h. Zhler A >=16
	goto $+4					;Ergebnis positiv d.h. =<15, weiter bei $+4
	movlw d'15'					;Ergebnis negativ, d.h. >15, 
	subwf COUNT_ENC_RA,f		;reduziere Zhler um 15 (+1 bei "encoder_int_ra1")
	goto encoder_int_ra1		;weiter bei "encoder_int_ra1"
	movwf RA_ENC_VS_REST		;Ergebnis positiv d.h.=<15, kopiere Ergebnis der Subtraktion nach "RA_ENC_VS_REST"
	incf RA_ENC_VS_REST			; und Erhhe um 1
	movlw 0x01					;setze Zhler A auf 1
	movwf COUNT_ENC_RA
encoder_int_ra1
	decfsz COUNT_ENC_RA			;reduziere Zhler A und prfe ob 0
	goto encoder_int_ra_end		;Zhler A noch nicht 0
	movfw RA_ENC_A				;Zhler A = 0, Rcksetzen des Zhlers mit Wert A
	movwf COUNT_ENC_RA
	movfw RA_ENC_C				;hole Zhler C nach W
	btfsc STATUS,Z				;prfen ob Zhler C <0>
	goto incdec_ra				;Zhler C =0
	btfsc STAT_MOTOR,5			;Zhler C <0>, Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	incf COUNT_ENC_RA			;Zhler C wird verwendet, Erhhe Zhler A um 1
	decfsz LOOP_ENC_RA			;reduziere Zhler B/C und prfe ob 0
	goto incdec_ra				;Zhler B/C noch nicht 0
	btfsc STAT_MOTOR,5			;Zhler B/C = 0, prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	goto $+4					;Zhler C wird verwendet, Rcksetzen auf Zhler B
	bsf STAT_MOTOR,5			;Zhler B wird verwendet, Rcksetzen auf Zhler C
	movfw RA_ENC_C				;hole Wert fr Zhler C nach W
	goto $+3
	bcf STAT_MOTOR,5			;Zhler C wird verwendet, Rcksetzen auf Zhler B
	movfw RA_ENC_B				;hole Wert fr Zhler B nach W
	movwf LOOP_ENC_RA			;Kopiere den Wert B oder C nach Zhler B/C
incdec_ra
	btfss SPEED,2				;Prfen ob Bit fr Drehrichtung rechts RA gesetzt ist
	call inc_ra					;Bit ist nicht gesetzt, d.h. links => erhhe RA
	btfsc SPEED,2				;Prfen ob Bit fr Drehrichtung rechts RA nicht gesetzt ist
	call dec_ra					;Bit ist gesetzt, d.h. rechts => reduziere RA
encoder_int_ra_end
	movfw RA_ENC_VS_REST
	subwf COUNT_ENC_RA,f

test_gotopos_ra
	btfsc STAT_POS,5			;prfen, ob Goto Enable Bit gesetzt ist
	call pos_goto_ra			;Unterprogramm zum prfen, ob Goto Position erreicht ist


deklination
	btfss STAT_MOTOR,1			;Prfen ob Bit fr DE Motor "Ein" gesetzt ist (=> Enable)
	goto Int_sz					;Bit ist nicht gesetzt, weiter
	btfsc SPEED,6				;prfen ob das Bit fr DE up/down nicht gesetzt ist
	goto Int_sz					;wenn wahr berspringe diesen Befehl, wenn nicht wahr weiter bei Interupt Sternzeit

	btfss STAT_POS,1			;prfen ob +90 Endposition erreicht ist
	goto $+3					;wenn ja berspringe diesen Befehl und prfe die Drehrichtung 
	btfsc SPEED,7				;prfen ob Ausgang auf Down steht,
	goto Int_sz					;wenn ja berspringe diesen Befehl, wenn nein Abbruch und weiter bei Int_sz (+90Endposition erreicht und Ausgang auf Up) 
	btfss STAT_POS,2			;prfen ob -90 Endposition erreicht ist
	goto $+3					;wenn ja berspringe diesen Befehl und prfe die Drehrichtung 
	btfss SPEED,7				;prfen ob Ausgang auf Up steht,
	goto Int_sz					;wenn ja berspringe diesen Befehl, wenn nein Abbruch und weiter bei Int_sz (-90Endposition erreicht und Ausgang auf Down) 

	btfss OUT_DE_HL				;prfen ob Ausgang high ist
	goto de_vollschritt			;wenn wahr berspringe diesen Befehl, wenn nicht wahr (low) weiter bei de_Vollschritt
	decfsz DE_LOW_TEMP			;
	goto Int_sz					;
	btfsc OUT_DE_SMOT			;prfen ob Motorausgang low ist
	goto $+3					;wenn ja berspringe diesen Befehl und setze Ausgang auf high, wenn nein setze Ausgang auf low bei $+3
	bsf	OUT_DE_SMOT				;
	goto $+2					;weier bei Rcksetzen des Zhlers
	bcf	OUT_DE_SMOT				;
	movfw DE_LOW				;Rcksetzen des Zhlers
	movwf DE_LOW_TEMP			;
	goto encoder_int_de			;weiter bei encoder

de_vollschritt
	decfsz DE_MAX_TEMP			;Reduziere Zhler und prfe ob grer 0
	goto Int_sz					;Zhler ist noch nicht 0,
	btfsc OUT_DE_SMOT			;prfen ob Motorausgang low ist
	goto $+3					;wenn ja berspringe diesen Befehl und setze Ausgang auf high, wenn nein setze Ausgang auf low bei $+3
	bsf	OUT_DE_SMOT				;
	goto $+2					;weier bei Rcksetzen des Zhlers
	bcf	OUT_DE_SMOT				;

	btfss STAT_RAMPE,7			;Prfe Timer1 berlauf
	goto de_vs_end				;kein berlauf vorhanden, 
	bcf STAT_RAMPE,7			;Timer1 berlauf, lsche berlauf Bit
	btfsc STAT_RAMPE,2			;prfen ob Bit fr Bremsrampe gesetzt ist
	goto $+5					;Bit ist gesetzt,d.h. Bremsrampe, weiter bei $+
	movfw DE_HS_RAMPE			;Bit nicht gesetzt,d.h. Beschleunigungsrampe, hole Zhlerwert nach W
	btfss STATUS,Z				;  und prfe ob 0
	decf DE_HS_RAMPE			;Zhler noch nicht 0, reduziere Zhler um 1 (Beschleunigungsrampe: Zhler von 15=>0)
	goto de_vs_end				;Zhler =0, weiter bei ra_vs_end
	incf DE_HS_RAMPE			;Bremsrampe, erhhe Zhler um 1 (Bremsrampe: Zhler von 0=>15)
	btfss DE_HS_RAMPE,4			;prfen, ob Zhler =16 (prfe bit 4)
	goto de_vs_end				;Zhler <16, weiter bei ra_vs_end
	decf DE_HS_RAMPE			;Zhler =16, reduziere um 1
	bsf STAT_RAMPE,3			;setze Statusbit fr erreichen der unteren Bremsrampenposition

de_vs_end
	movlw 0x07					;Programmzhler fuer Rampen Tabelle im Bereich 0x07
	movwf PCLATH				;	
	movfw DE_HS_RAMPE			;Hole Zhlerwert nach W
	call tab_rampe				;Zu dem Zhlerwert wird ein Geschwindigkeitswert aus der Tabelle geholt
	movwf DE_MAX_TEMP			;  und ins Highspeed Register kopiert
	movfw DE_HIGH_L				;der Wert fr die Maximale Geschwindigkeit wird nach W geholt
	addwf DE_MAX_TEMP,f			;  und zum Highspeed Register addiert.

encoder_int_de					;Encoder DE: Auswerten und Rcksetzen der Werte	
	btfss STAT_MOTOR,7			;Prfen ob Bit fr interne Positionsencoder gesetzt ist
	goto test_gotopos_de		;wenn ja berspringe diesen Befehl, wenn nein (Eingang low) weiter bei Int_sz
	clrf DE_ENC_VS_REST			;lsche Register ( wird immer am Ende vom Zhler A abgezogen)

	movfw DE_ENC_C				;hole Zhler C nach W
	btfsc STATUS,Z				;prfen ob Zhler C <0>
	bcf STAT_MOTOR,6			;Zhler C = 0, Zhler B wird verwendet, lsche Bit (B=0, C=1)

	btfss SPEED,7				;prfen, ob Bit fr Drehrichtung up gesetzt ist, DE
	goto $+5					;Bit ist nicht gesetzt, d.h. Drehrichtung ist down, DE
	btfsc STAT_ENCODER,3		;Drehrichtung ist up, prfen, ob die alte Drehrichtung down war, DE
	goto encoder_int_de2		;nein, alte Drehrichtung war auch up, weiter
	bsf STAT_ENCODER,3			;alte Drehrichtung war down, update Statusregister mit neuer Drehrichtung up
	goto $+4
	btfss STAT_ENCODER,3		;Drehrichtung ist down, prfen, ob die alte Drehrichtung up war, DE
	goto encoder_int_de2		;nein, alte Drehrichtung war auch down, weiter
	bcf STAT_ENCODER,3			;alte Drehrichtung war up, update Statusregister mit neuer Drehrichtung down
								;Drehrichtung hat sich gendert, d.h. Drehrichtung Encoder mu auch gendert werden
	movfw COUNT_ENC_DE			;subtrahiere Aktuellen Zhler A von max Wert Zhler A
	subwf DE_ENC_A,w
	movwf COUNT_ENC_DE			; und schreibe die Differenz zurck ins Zhler A Register 
	incf COUNT_ENC_DE			;die Differenz muss noch um 1 erhht werden. 
	btfsc STAT_MOTOR,6			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	incf COUNT_ENC_DE			;Zhler C wird verwendet, Erhhe Zhler A um 1

	movfw LOOP_ENC_DE			;hole Zhler B/C nach W	
	btfss STAT_MOTOR,6			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	subwf DE_ENC_B,w			;Zhler B wird verwendet, subtrahiere Aktuellen Zhler B/C von max Wert Zhler B
	btfsc STAT_MOTOR,6			;Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	subwf DE_ENC_C,w			;Zhler C wird verwendet, subtrahiere Aktuellen Zhler B/C von max Wert Zhler C
	movwf LOOP_ENC_DE			;Differenz wird wieder in Zhler B/C Register zurckgeschrieben
	incf LOOP_ENC_DE			;  und um 1 erhht

encoder_int_de2
	btfsc OUT_DE_HL				;prfen ob Ausgang low ist, d.h. Vollschritt
	goto encoder_int_de1		;=>Mikroschritt, berspringe die nchsten Schritte
	movfw COUNT_ENC_DE			;=>Vollschritt, Zhler A um 15 (+1 bei "encoder_int_de1") reduzieren 
	sublw d'15'					;prfen, ob Zhler A >15: 15-Zhler A
	btfsc STATUS,C				;prfen, ob Ergebnis negativ, d.h. Zhler A >=16
	goto $+4					;Ergebnis positiv d.h. =<15, weiter bei $+4
	movlw d'15'					;Ergebnis negativ, d.h. >15, 
	subwf COUNT_ENC_DE,f		;reduziere Zhler um 15 (+1 bei "encoder_int_de1")
	goto encoder_int_de1		;weiter bei "encoder_int_de1"
	movwf DE_ENC_VS_REST		;Ergebnis positiv d.h.=<15, kopiere Ergebnis der Subtraktion nach "DE_ENC_VS_REST"
	incf DE_ENC_VS_REST			; und Erhhe um 1
	movlw 0x01					;setze Zhler A auf 1
	movwf COUNT_ENC_DE
encoder_int_de1
	decfsz COUNT_ENC_DE			;reduziere Zhler A und prfe ob 0
	goto encoder_int_de_end		;Zhler A noch nicht 0
	movfw DE_ENC_A				;Zhler A = 0, Rcksetzen des Zhlers mit Wert A
	movwf COUNT_ENC_DE
	movfw DE_ENC_C				;hole Zhler C nach W
	btfsc STATUS,Z				;prfen ob Zhler C <0>
	goto incdec_de				;Zhler C =0
	btfsc STAT_MOTOR,6			;Zhler C <0>, Prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	incf COUNT_ENC_DE			;Zhler C wird verwendet, Erhhe Zhler A um 1
	decfsz LOOP_ENC_DE			;reduziere Zhler B/C und prfe ob 0
	goto incdec_de				;Zhler B/C noch nicht 0
	btfsc STAT_MOTOR,6			;Zhler B/C = 0, prfen ob Zhler B oder C verwendet wird (B=0, C=1)
	goto $+4					;Zhler C wird verwendet, Rcksetzen auf Zhler B
	bsf STAT_MOTOR,6			;Zhler B wird verwendet, Rcksetzen auf Zhler C
	movfw DE_ENC_C				;hole Wert fr Zhler C nach W
	goto $+3
	bcf STAT_MOTOR,6			;Zhler C wird verwendet, Rcksetzen auf Zhler B
	movfw DE_ENC_B				;hole Wert fr Zhler B nach W
	movwf LOOP_ENC_DE			;Kopiere den Wert B oder C nach Zhler B/C
incdec_de
	btfss SPEED,7				;Prfen ob Bit fr Drehrichtung up DE gesetzt ist
	call dec_de					;Bit ist nicht gesetzt, d.h. down => reduziere DE
	btfsc SPEED,7				;Prfen ob Bit fr Drehrichtung down DE nicht gesetzt ist
	call inc_de					;Bit ist gesetzt, d.h. up => erhhe DE
encoder_int_de_end
	movfw DE_ENC_VS_REST
	subwf COUNT_ENC_DE,f

test_gotopos_de
	btfsc STAT_POS,6			;prfen, ob Goto Enable Bit gesetzt ist
	call pos_goto_de			;Unterprogramm zum prfen, ob Goto Position erreicht ist

Int_sz
	btfss PIR1,TMR1IF			;Abfrage Timer1 berlauf, wird fr Beschleunigungs-/Bremsrampe bentigt
	goto $+4					;kein berlauf, weiter bei $+
	bcf PIR1,TMR1IF				;berlauf vorhanden, lsche berlaufbit
	bsf STAT_RAMPE,6			;setze Timer Bit fr Rampe RA
	bsf STAT_RAMPE,7			;setze Timer Bit fr Rampe DE
	decfsz COUNT_L_SZ,f			;Reduziere Zhler fr Low byte und prfe ob 0
	goto Int_uz					;wenn ja berspringe diesen Befehl, wenn nein Rcksprung aus dem Interupt
	decfsz COUNT_H_SZ,f			;Reduziere Zhler fr High byte und prfe ob 0
	goto Int_uz					;Sekunde noch nicht vorbei

	; eine Sekunde ist vorbei
	; 6 x 7791,168 = (6 x 7791) + 7792       = 6 Sekunden
	; Digit = 5..1	Teilen durch 7791 = 1E 6F
	; Digit =    0  Teilen durch 7792 = 1E 70

	; Sekundentimer wieder einstellen auf 7791
	movlw 0x1F					;High Byte = 31
	movwf COUNT_H_SZ			;
	movlw 0x6F					;Low Byte = 111 (Loop 1-5)
	movwf COUNT_L_SZ
	; bei Digit = 0 mu countL auf 0x70 gesetzt werden
	; falls Digit=0 dann 
	;  Digit von 0 auf 6 setzen
	;  Sekundentimer auf 7792 korrigieren
	decfsz DIGIT_SZ,f			; 6-Sekundenzhler verringern
	goto Int_sz1
	movlw 0x06					; Rcksetzen des Zhlers, Digit:=06	
	movwf DIGIT_SZ				;
	movlw 0x70					;Low Byte = 112 (Loop 6)
	movwf COUNT_L_SZ
Int_sz1
	call inc_sz					;1 Sek Sternzeit ist um, erhhe Sternzeit und RA um eine Sekunde
	call inc_ra
	movfw SEK_RA				;hole Sekunde RA nach W und subtrahiere die alte Sekunde
	subwf SEK_RA_ALT,w
	btfss STATUS,Z				;Prfen, ob Ergebnis =0
	bsf NEU_DISP,2				;Ergebnis >0, setze Status bit fr RA, Sek
	movfw SEK_RA				;kopiere die neue Sek in Register fr alte Sekunde 
	movwf SEK_RA_ALT
	btfsc STAT_POS,5			;prfen, ob Goto Enable Bit gesetzt ist
	call pos_goto_ra			;Unterprogramm zum prfen, ob Goto Position erreicht ist

	
Int_uz
	decfsz COUNT_L_UZ,f			;Reduziere Zhler fr Low byte und prfe ob 0
	return						;wenn ja berspringe diesen Befehl, wenn nein Rcksprung aus dem Interupt
	decfsz COUNT_H_UZ,f			;Reduziere Zhler fr High byte und prfe ob 0
	return				;Sekunde noch nicht vorbei

	; eine Sekunde ist vorbei
	; 2 x 7812,5 = 7812 + 7813       = 2 Sekunden
	; Digit = 1	Teilen durch 7812 = 1E 84
	; Digit = 0 Teilen durch 7813 = 1E 85

	; Sekundentimer wieder einstellen auf 7812
	movlw 0x1F					;High Byte = 31
	movwf COUNT_H_UZ			;
	movlw 0x84					;Low Byte = 132 (Loop 1)
	movwf COUNT_L_UZ
	; bei Digit = 0 mu countL noch 1x erhht werden

	; falls Digit=0 dann 
	;  Digit von 0 auf 2 setzen
	;  Sekundentimer auf 7813 korrigieren
	decfsz DIGIT_UZ,f			; 8-Sekundenzhler verringern
	goto Int_uz1
	bsf	DIGIT_UZ,1				; Rcksetzen des Zhlers, Digit:=2	
	incf COUNT_L_UZ,f
Int_uz1
	call inc_uz		
	return

enc_ext_irp_routine				;Interruptroutine bei externen Encodern
	movfw PORTB					;kopiere Port B in ENCODER_NEU Register
	movwf ENCODER_NEU			;
	btfsc ENCODER_NEU,4			;prfen, ob Eingangstakt RA low ist
	goto deklination_ext		;nicht low
	btfss STAT_ENCODER,5		;prfen, ob alter Eingangstakt RA high war
	goto deklination_ext		;nicht high, d.h. keine nderung
	btfsc STAT_MOTOR,2			;prfen, ob Drehrichtung RA nicht invertiert ist
	goto $+6					;invertiert
	btfsc ENCODER_NEU,5			;prfen ob Eingang low ist	
	call dec_ra					;nein,high,reduziere RA
	btfss ENCODER_NEU,5			;prfen ob Eingang high ist	
	call inc_ra					;nein, low, erhhe RA
	goto deklination_ext
	btfsc ENCODER_NEU,5			;prfen ob Eingang low ist	
	call inc_ra					;nein,high,erhhe RA (invertiert)
	btfss ENCODER_NEU,5			;prfen ob Eingang high ist	
	call dec_ra					;nein, low, reduziere RA (invertiert)
deklination_ext
	btfsc ENCODER_NEU,6			;prfen, ob Eingangstakt DE low ist
	goto enc_ext_irp_end		;nicht low
	btfss STAT_ENCODER,6		;prfen, ob alter Eingangstakt DE high war
	goto enc_ext_irp_end		;nicht high, d.h. keine nderung
	btfsc STAT_MOTOR,3			;prfen, ob Drehrichtung DE nicht invertiert ist
	goto $+6					;invertiert
	btfsc ENCODER_NEU,7			;prfen ob Eingang low ist	
	call dec_de					;nein,high,reduziere RA
	btfss ENCODER_NEU,7			;prfen ob Eingang high ist	
	call inc_de					;nein, low, erhhe RA
	goto enc_ext_irp_end
	btfsc ENCODER_NEU,7			;prfen ob Eingang low ist	
	call inc_de					;nein,high,erhhe RA (invertiert)
	btfss ENCODER_NEU,7			;prfen ob Eingang high ist	
	call dec_de					;nein, low, reduziere RA (invertiert)
enc_ext_irp_end
	bcf STAT_ENCODER,5			;lsche Bits fr alten Zustand der Eingangstakte RA/DE
	bcf STAT_ENCODER,6
	btfsc ENCODER_NEU,4			;prfen, ob aktueller Encodertakt RA low war
	bsf STAT_ENCODER,5			;nein, Encodertakt RA war high, setze Bit
	btfsc ENCODER_NEU,6			;prfen, ob aktueller Encodertakt DE low war
	bsf STAT_ENCODER,6			;nein, Encodertakt RA war high, setze Bit
	bcf	INTCON,RBIF				;Interupt-Flag lschen
	return

inc_ra
	incf	SEK_RA,f			; 1 Sekunden erhhen
	movlw	D'60'
	subwf	SEK_RA,w
	btfss	STATUS,Z	
	return						; 1 Sekunden <> 10	

	clrf	SEK_RA
	bsf	NEU1_DISP,4				;setze Status Bit fr Ra, Min
	incf	MIN_RA,f			; 1 Minuten erhhen
	movlw	D'60'
	subwf	MIN_RA,w
	btfss	STATUS,Z	
	return						;  1 Minuten <> 10

	clrf	MIN_RA
	bsf	NEU1_DISP,5				;setze Status Bit fr Ra, Std
	incf	STD_RA,f			; 1 Stunden erhhen
	movlw	D'24'
	subwf	STD_RA,w
	btfss	STATUS,Z	
	return						; 1 Stunden <> 10

	clrf	STD_RA
	return						

dec_ra
	movfw 	SEK_RA
	btfsc	STATUS, Z	
	goto	$+3
	decf	SEK_RA, f			; 1 Sekunden reduzieren
	return						; 1 Sekunden <> 10	

	movlw	D'59'
	movwf	SEK_RA
	bsf	NEU1_DISP,4				;setze Status Bit fr Ra, Min
	movfw 	MIN_RA
	btfsc	STATUS, Z	
	goto	$+3
	decf	MIN_RA, f			; 1 Minuten reduzieren
	return						; 1 Minuten <> 10

	movlw	D'59'
	movwf	MIN_RA
	bsf	NEU1_DISP,5				;setze Status Bit fr Ra, Std
	movfw 	STD_RA
	btfsc	STATUS, Z	
	goto	$+3
	decf	STD_RA, f			; 1 Stunden reduzieren
	return						; 1 Stunden <> 10

	movlw	D'23'
	movwf	STD_RA
	return						


inc_de
	bcf STAT_POS,2				;lsche Statusbit fr Endposition 0
	bsf NEU_DISP,3				;setze Status Bit fr DE, 1/10tel Min

	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	goto $+0x04					;>=90, weiter
	movf MIN_DE_01,w			;<90, hole 10tel Minuten nach W und prfe ob >0<
	btfsc STATUS,Z				
	bsf NEU1_DISP,6				;setze Status Bit fr DE, Min	
	incf MIN_DE_01, f			; 0,1 Minuten erhhen
	incf MIN_DE_01, f			; 0,1 Minuten erhhen
	movlw D'10'
	subwf MIN_DE_01, w
	btfss STATUS, Z	
	goto inc_de_end				; 0,1 Minuten <> 10	
	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	bsf NEU1_DISP,6				;>90, setze Status Bit fr DE, Min
	clrf MIN_DE_01

	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	goto $+0x04					;>=90, weiter
	movf MIN_DE,w				;<90, hole Minuten nach W und prfe ob >0<
	btfsc STATUS,Z				
	bsf NEU1_DISP,7				;setze Status Bit fr DE, Grd	
	incf MIN_DE, f				; 1 Minuten erhhen
	movlw D'60'
	subwf MIN_DE, w
	btfss STATUS, Z	
	goto inc_de_end				;  1 Minuten <> 10

	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	bsf NEU1_DISP,7				;setze Status Bit fr DE, Grd
	clrf MIN_DE
	incf GRD_DE, f				; 1 Grad erhhen
	movlw d'90'					;Abfrage, ob Position =90
	subwf GRD_DE,w
	btfsc STATUS,Z
	bsf NEU1_DISP,7				;setze Status Bit fr DE, Grd		

	movlw D'180'				;Abfrage, ob Endposition +180 erreicht ist
	subwf GRD_DE, w
	btfss STATUS, Z	
	goto inc_de_end				; 1 Grad <> 10
	bsf STAT_POS,1				;setze Statusbit fr Endposition +180
inc_de_end
	movlw d'90'					;Abfrage, ob Position >=90
	subwf GRD_DE,w
	btfsc STATUS,C
	bcf STAT_POS,0				;Position >=90, setze Status Bit
	return

dec_de
	bcf STAT_POS,1				;lsche statusbit fr +180 Endposition
	bsf NEU_DISP,3				;setze Status Bit fr DE, 1/10tel Min

	movf MIN_DE_01,w
	btfsc STATUS, Z	
	goto dec_de_min				;
	decf MIN_DE_01, f			; 0,1 Minuten reduzieren
	decfsz MIN_DE_01, f			; 0,1 Minuten reduzieren
	goto dec_de_end				; 0,1 Minuten <> 10	
	btfsc STAT_POS,0			;prfen, ob Statusbit fr <90 nicht gesetzt ist
	bsf NEU1_DISP,6				;setze Status Bit fr DE, Min
	goto dec_de_end				
dec_de_min
	movlw D'8'
	movwf MIN_DE_01
	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	bsf NEU1_DISP,6				;setze Status Bit fr DE, Min
	movf MIN_DE,w
	btfsc STATUS, Z	
	goto dec_de_grd				;
	decfsz MIN_DE, f			; 1 Minuten reduzieren
	goto dec_de_end				; 1 Minuten <> 10
	btfsc STAT_POS,0			;prfen, ob Statusbit fr <90 nicht gesetzt ist
	bsf NEU1_DISP,7				;setze Status Bit fr DE, Grd
	goto dec_de_end				
dec_de_grd
	movlw D'59'
	movwf MIN_DE
	btfss STAT_POS,0			;prfen, ob Statusbit fr <90 gesetzt ist
	bsf NEU1_DISP,7				;setze Status Bit fr DE, Grd
	movf GRD_DE,w
	btfsc STATUS, Z	
	goto dec_de_null			;
	decf GRD_DE, f				; 1 Grad reduzieren
	goto dec_de_end				; 1 Grad <> 10
dec_de_null
	bsf STAT_POS,2				;setze Statusbit fr 0 Endposition
dec_de_end
	movlw d'90'
	subwf GRD_DE,w
	btfss STATUS,C
	bsf STAT_POS,0
	return
	

inc_sz
	bsf NEU_DISP,0				;setze Status Bit fr Sternzeit, Sek
	incf	SEK_SZ, f			; 1 Sekunden erhhen
	movlw	D'60'
	subwf	SEK_SZ, w
	btfss	STATUS, Z	
	return						; 1 Sekunden <> 10	

	clrf	SEK_SZ
	bsf	NEU1_DISP,0				;setze Status Bit fr Sternzeit, Min
	incf	MIN_SZ, f			; 1 Minuten erhhen
	movlw	D'60'
	subwf	MIN_SZ, w
	btfss	STATUS, Z	
	return						;  1 Minuten <> 10

	clrf	MIN_SZ
	bsf	NEU1_DISP,1				;setze Status Bit fr Sternzeit, Std
	incf	STD_SZ, f			; 1 Stunden erhhen
	movlw	D'24'
	subwf	STD_SZ, w
	btfss	STATUS, Z	
	return						; 1 Stunden <> 10

	clrf	STD_SZ
	return						


inc_uz
	bsf	NEU_DISP,1				;setze Status Bit fr Uhrzeit, Sek
	incf	SEK_UZ,f			; 1 Sekunden erhhen
	movlw	D'60'
	subwf	SEK_UZ,w
	btfss	STATUS,Z	
	return						; 1 Sekunden <> 10	

	clrf	SEK_UZ
	bsf	NEU1_DISP,2				;setze Status Bit fr Uhrzeit, Min
	incf	MIN_UZ,f			; 1 Minuten erhhen
	movlw	D'60'
	subwf	MIN_UZ,w
	btfss	STATUS,Z	
	return						;  1 Minuten <> 10

	clrf	MIN_UZ
	bsf	NEU1_DISP,3				;setze Status Bit fr Uhrzeit, Std
	incf	STD_UZ,f			; 1 Stunden erhhen
	movlw	D'24'
	subwf	STD_UZ,w
	btfss	STATUS,Z	
	return						; 1 Stunden <> 10

	clrf	STD_UZ
	return						



pos_goto_ra						;Unterprogramm zum prfen, ob Goto Position RA erreicht ist
	movfw STD_RA				;Prfen, ob Goto-Ist (Std RA) = 0
	subwf STD_TEMP,w
	btfss STATUS,Z	
	goto pos_brrampe_ra			;>0<, weiter bei Prfung Bremsrampe
	movfw MIN_RA				;Prfen, ob Goto-Ist (Min RA) = 0
	subwf MIN_TEMP,w
	btfss STATUS,Z	
	goto pos_brrampe_ra			;>0<, weiter bei Prfung Bremsrampe
	movfw SEK_RA				;Prfen, ob Goto-Ist (Sek RA) = 0
	subwf SEK_TEMP,w
	btfss STATUS,Z
	goto pos_brrampe_ra			;>0<, weiter bei Prfung Bremsrampe
	bsf OUT_RA_HL				;=0, setze Ausgang auf high	
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf SPEED,2					;setze bit (Bit 2: RA_X0, 0 = null, 1 = normal)
	bsf SPEED,3					;setze Bit (Bit 3: RA_X2, 0 = x2, 1 = normal)		
	bcf STAT_POS,5				;lsche Goto Enable Bit
pos_brrampe_ra
	movfw STD_RA				;Prfen, ob Bremsrampe-Ist (Std RA) = 0
	subwf STD_TEMP_1,w
	btfss STATUS,Z	
	return						;>0<, zurck
	movfw MIN_RA				;Prfen, ob Bremsrampe-Ist (Min RA) = 0
	subwf MIN_TEMP_1,w
	btfss STATUS,Z	
	return						;>0<, zurck
	movfw SEK_RA				;Prfen, ob Bremsrampe-Ist (Sek RA) = 0
	subwf SEK_TEMP_1,w
	btfss STATUS,Z
	return						;>0<, zurck
	bsf STAT_RAMPE,0			;=0, setze Bit fr Beginn Bremsrampe RA
	return						;

pos_goto_de						;Unterprogramm zum prfen, ob Goto Position DE erreicht ist
	movfw GRD_DE				;Prfen, ob Goto-Ist (Grad DE) = 0
	subwf GRD_GOTO,w
	btfss STATUS,Z	
	goto pos_brrampe_de			;>0<, weiter bei Prfung Bremsrampe
	movfw MIN_DE				;Prfen, ob Goto-Ist (Min DE) = 0
	subwf MIN_GOTO,w
	btfss STATUS,Z	
	goto pos_brrampe_de			;>0<, weiter bei Prfung Bremsrampe
	movfw MIN_DE_01				;Prfen, ob Goto-Ist (10tel Min DE) = 0
	subwf MIN_GOTO_01,w
	btfss STATUS,Z
	goto pos_brrampe_de			;>0<, weiter bei Prfung Bremsrampe
	bsf OUT_DE_HL				;setze Ausgang auf high	
	bsf SPEED,6					;setze Bit
	bcf STAT_POS,6				;lsche Goto Enable Bit
pos_brrampe_de
	movfw GRD_DE				;Prfen, ob Bremsrampe-Ist (Grad DE) = 0
	subwf GRD_TEM_1,w
	btfss STATUS,Z	
	return						;>0<, zurck
	movfw MIN_DE				;Prfen, ob Bremsrampe-Ist (Min DE) = 0
	subwf MIN_TEM_1,w
	btfss STATUS,Z	
	return						;>0<, zurck
	movfw MIN_DE_01				;Prfen, ob Bremsrampe-Ist (10tel Min DE) = 0
	subwf MIN_GOTO_01,w
	btfss STATUS,Z
	return						;>0<, zurck
	bsf STAT_RAMPE,2			;=0, setze Bit fr Beginn Bremsrampe DE
	return						;


hl_ff							;Unterprogramms fr H/L Flip Flop
	btfss SPEED,0				;prfen ob Bit gesetzt ist
	goto $+3					;wenn ja berspringe diesen Befehl, wenn nein setze bit in Register
	bcf SPEED,0					;setze Bit auf 0
	goto $+2					;
	bsf SPEED,0					;setze Bit auf 1
	bsf SPEED,1					;setze Statusbit fr Update Display mit Speed Werten
	bsf NEU_DISP,3
	movlw d'250'				;Wartezeit von 250ms
	bsf PCLATH,4				;Select Page 2
	call wait 		
	bcf PCLATH,4				;Select Page 0
	return




tast_befehl						;Unterprogramm zum seriellen Lesen der Tastaturbefehle aus dem Slave Microcontroller
	bcf INTCON,GIE				;globale Interupts verbieten
	bcf DAT_TX					;setze TX auf 0, Mit dieser negativen Flanke wird im Slave ein Interrupt ausgelst
	bcf STATUS, C				;Carrybit lschen, damit es beim rotieren nicht strt 
	clrf BEFEHL					;lsche Register Befehl
	movlw d'3'					;lade Befehlszhler mit Wert 3 (3 Bits sollen empfangen werden)
	movwf SHIFT_CNT
	bsf DAT_TX					;setze TX auf 1, hiermit wird der Slave angewiesen, die Tastaturbefehle zu bertragen
	movlw d'6'					;Wartezeit 19 Takte
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
tast_befehl_loop
	rlf BEFEHL,f				;verschiebe Registerinhalt nach links
	btfsc DAT_RX				;prfen ob Dateneingang low ist
	bsf BEFEHL,0				;setze bit 0 im Register Befehl
	btfss DAT_RX				;prfen ob Dateneingang high ist
	bcf BEFEHL,0				;lsche bit 0 im Register Befehl
	movlw d'2'					;Wartezeit 7 Takte
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	decfsz SHIFT_CNT			;reduziere Befehlszhler um 1 und prfe ob Zhler 0 ist
	goto tast_befehl_loop		;wenn ja berspringe diesen Befehl, wenn nein zurck zur Schleife
	bsf INTCON,GIE				;globale Interupts erlauben

	movfw BEFEHL				;Die 3 Bit des Befehls (dezimal 0-5)in 6 Bits umwandeln, 
	movwf LOOP_1				;kopiere Befehl nach loop_1
	incf LOOP_1,f				;erhhe loop_1 um 1
	clrf BEFEHL					;lsche befehl
	bsf STATUS,C				;setze Carry Bit
	goto $+2
	bcf STATUS,C				;lsche Carry Bit, damit es beim Rotieren nicht strt
	rlf BEFEHL					;befehl nach links schieben, beim ersten Mal wird das Carry Bit mit Wert 1 eingeschoben
	decfsz LOOP_1				;reduziere loop_1 um 1 und prfe ob 0
	goto $-3					;noch nicht 0, weiter nach links schieben

	return						;loop_1 ist 0, war Befehl =0 ist jetzt Bit0 von Befehl gesetzt
								;war Befehl =5 ist jetzt Bit5 von Befehl gesetzt


	
	ORG 0x07CD

tab_rampe
	addwf PCL,f
	retlw 0x00		;Datenpunkt 0:
	retlw d'1'		;Datenpunkt 1:
	retlw d'2'		;Datenpunkt 2:
	retlw d'4'		;Datenpunkt 3:
	retlw d'6'		;Datenpunkt 4:
	retlw d'8'		;Datenpunkt 5:
	retlw d'11'		;Datenpunkt 6:
	retlw d'15'		;Datenpunkt 7:
	retlw d'22'		;Datenpunkt 8:
	retlw d'30'		;Datenpunkt 9:
	retlw d'40'		;Datenpunkt 10:
	retlw d'51'		;Datenpunkt 11:
	retlw d'64'		;Datenpunkt 12:
	retlw d'80'		;Datenpunkt 13:
	retlw d'100'	;Datenpunkt 14:
	retlw d'125'	;Datenpunkt 15:

	
	ORG 0x07DF
menu_routine1
	movlw 0x07
	movwf PCLATH				;Programmzhler fuer Goto Tabelle im Bereich 0x07E3..0x07FF 
	movfw MENUE_ADR
	addwf PCL,f
	goto copy_goto_eepr		;Adresse 00
	goto copy_goto_man		;Adresse 01
	goto copy_pos_eepr		;Adresse 02
	goto copy_pos_man		;Adresse 03
	goto copy_motor			;Adresse 04	
	goto copy_uz			;Adresse 05
	goto copy_loeschen		;Adresse 06
	goto copy_standort		;Adresse 07
	goto copy_pa_st			;Adresse 08
	goto copy_pa_so			;Adresse 09
	goto copy_pa_lu			;Adresse 10
	goto copy_pa_ramxde		;Adresse 11
	goto copy_pa_patra		;Adresse 12
	goto copy_pa_patde		;Adresse 13
	goto copy_speichern		;Adresse 14
	goto copy_dcfpara		;Adresse 15
	goto copy_pa_ra_aut		;Adresse 16
	goto copy_pa_de_aut		;Adresse 17

;*******************************************************************************************
;	PAGE 1, 	bsf PCLATH,3				;Select Page 1
;				bcf PCLATH,4
;*******************************************************************************************
	ORG 0x800					;PAGE 1





dig_temp_kat				;Unterprogramm zum setzen des Cursor Position im Menuepunkt katalogauswahl
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_temp_kat2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC2				;lade Adresse ins W-Register (3.Zeichen in der 2.Reihe)
	goto dig_temp_katend	;springe zum Ende des Unterprogramms
dig_temp_kat2
	movlw d'2'
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_temp_kat3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC5				;lade Adresse ins W-Register (5.Zeichen in der 2.Reihe)
	goto dig_temp_katend	;springe zum Ende des Unterprogramms
dig_temp_kat3
	movlw d'3'
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_temp_kat4		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC8				;lade Adresse ins W-Register (9.Zeichen in der 2.Reihe)
	goto dig_temp_katend	;springe zum Ende des Unterprogramms
dig_temp_kat4
	movlw d'4'
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_temp_kat5		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xCB				;lade Adresse ins W-Register (12.Zeichen in der 2.Reihe)
	goto dig_temp_katend	;springe zum Ende des Unterprogramms
dig_temp_kat5
	movlw d'5'
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_temp_katend	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xCE				;lade Adresse ins W-Register (15.Zeichen in der 2.Reihe)
dig_temp_katend
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf TEMP_DIG_STAT,0		;lsche das Status bit
	movlw d'250'			;Wartezeit von 250ms
	call wait
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	return

dig_einst_hor				;Unterprogramm zum setzen des Cursor im Einstell Untermen 
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_einst_hor2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC2				;lade Adresse ins W-Register (3.Zeichen in der 2.Reihe)
	goto dig_einst_hor_end	;springe zum Ende des Unterprogramms
dig_einst_hor2
	movlw 0xC6				;lade Adresse ins W-Register (6.Zeichen in der 2.Reihe)
dig_einst_hor_end
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bcf TEMP_DIG_STAT,0		;lsche das Status bit
	movlw d'250'			;Wartezeit von 250ms
	call wait
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	return


menu_rade_man				;Unterprogramm zum Initialisieren der Menpunkte Goto/Position Manuell 
	clrf DIG_REG			;
	bsf DIG_REG,0			;setze bit 0 in Register 
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'12'				;Anzeige hat 12 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw STD_RA			;lade RA/DE Werte in Temp Register
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er	
	movwf STD_TEMP_1		;kopiere 1er Wert ins Temp Register
	movfw TEMP_14
	movwf STD_TEMP			;kopiere 10er Wert ins Temp Register
	movfw MIN_RA
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er	
	movwf MIN_TEMP_1		;kopiere 1er Wert ins Temp Register
	movfw TEMP_14
	movwf MIN_TEMP			;kopiere 10er Wert ins Temp Register
	movfw SEK_RA
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movwf SEK_TEMP_1		;kopiere 1er Wert ins Temp Register
	movfw TEMP_14
	movwf SEK_TEMP			;kopiere 10er Wert ins Temp Register

	call conv_de_int_disp	;Unterprogramm zum Konvertieren von DE Werten 0=>180 in -90=>0=>+90 fr die Anzeige 

	movfw PM_DE
	movwf PM_TEM
	movfw TEMP_18			;Wert fr Grad
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movwf GRD_TEM_1			;kopiere 1er Wert ins Temp Register
	movfw TEMP_14
	movwf GRD_TEM			;kopiere 10er Wert ins Temp Register
	movfw TEMP_17			;Wert fr Minuten
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movwf MIN_TEM_1			;kopiere 1er Wert ins Temp Register
	movfw TEMP_14
	movwf MIN_TEM			;kopiere 10er Wert ins Temp Register
	movfw TEMP_16			;Wert fr 10tel Minuten
	movwf MIN_TEM_01
	call disp_temp_zeit		;Anzeige der Temp Werte RA/DE
	call disp_temp_grad
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	return

menu_eeprom
	clrf DIG_REG			;
	bsf DIG_REG,2			;setze bit 2 in Register 
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movlw d'2'				;prfen, ob Katalog "Own" gewhlt ist
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto $+7				;nein, weiter bei nchster Prfung
	movlw 0x00				;ja, kopiere die erste Katalognummer in die Katalog Register (dez: 0, hex: 0000)
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_H				;1.Katalognummer "Own", High Byte
	movlw 0x00
	movwf KAT_L				;1.Katalognummer "Own", Low Byte
	goto search_katno		;weiter bei Search_Katno
	bcf STATUS,RP0			;Select BANK_0
	movlw d'3'				;prfen, ob Katalog "Messier" gewhlt ist
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto $+7				;nein, weiter bei nchster Prfung
	movlw 0x27				;ja, kopiere die erste Katalognummer in die Katalog Register (dez: 10001, hex: 2711)
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_H				;1.Katalognummer "Messier", High Byte
	movlw 0x11
	movwf KAT_L				;1.Katalognummer "Messier", Low Byte
	goto search_katno		;weiter bei Search_Katno
	bcf STATUS,RP0			;Select BANK_0
	movlw d'4'				;prfen, ob Katalog "NGC" gewhlt ist
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto $+7				;nein, weiter bei nchster Prfung
	movlw 0x4E				;ja, kopiere die erste Katalognummer in die Katalog Register (dez: 20001, hex: 4E21)
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_H				;1.Katalognummer "NGC", High Byte
	movlw 0x21
	movwf KAT_L				;1.Katalognummer "NGC", Low Byte
	goto search_katno		;weiter bei Search_Katno
	movlw d'5'				;prfen, ob Katalog "YBSC" gewhlt ist
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto $+6				;nein, weiter bei nchster Prfung
	movlw 0x75				;ja, kopiere die erste Katalognummer in die Katalog Register (dez: 30002, hex: 7532)
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_H				;1.Katalognummer "YBSC", High Byte
	movlw 0x32
	movwf KAT_L				;1.Katalognummer "YBSC", Low Byte
search_katno
	bcf STATUS,RP0			;Select BANK_0
	movlw d'4'				;Rcksetzen des Zhlers auf 4
	movwf TEMP_DIG_CNT
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_H
	movwf ZAHL_C1
	movfw KAT_L
	movwf ZAHL_C0
	clrf ZAHL_C2
	bsf PCLATH,4			;Select Page 3		
	call convert_hex_dig
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2	
	call disp_kat_no
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	call search_kat
	bcf STATUS,RP0			;Select BANK_0
	return

incdec_test
	call menu_cursor_adr	;Unterprogramm fr Display Adresse, Adresse steht in W und TEMP_DIG_ADR
	btfsc TEMP_DIG_STAT,0	;wenn Bit nicht gesetzt ist, wird Wert verndert. Wenn Bit gesetzt ist, wird Position 
	goto incdec_test_end	; verndert und nur der Cursor neu gesetzt. 
	btfsc DIG_REG,0
	goto incdec_temp1
	btfsc DIG_REG,1
	goto incdec_para1
	btfsc DIG_REG,2
	goto incdec_eepr1
	btfsc DIG_REG,3
	goto incdec_uz1
	btfsc DIG_REG,4
	goto incdec_so1
	btfsc DIG_REG,5
	goto incdec_dcfpara1
	btfsc DIG_REG,6
	goto incdec_motor1
	btfsc DIG_REG,7
	goto incdec_paraaut1

incdec_temp1				;Unterprogramm zur Vernderung der Werte in den Menuepunkten Goto, Position Manuell
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_temp2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw STD_TEMP
	movwf TEMP_13
	movlw d'3'
	movwf TEMP_14
	call incdec_val
	movwf STD_TEMP
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp2				;Unterprogramm zur Erhhung der Werte in den Menuepunkten Goto, Position und Sternzeit
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_temp3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw STD_TEMP_1
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf STD_TEMP_1
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp4
	movfw MIN_TEMP
	movwf TEMP_13
	movlw d'6'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEMP
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp5
	movfw MIN_TEMP_1
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEMP_1
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp5
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp6
	movfw SEK_TEMP
	movwf TEMP_13
	movlw d'6'
	movwf TEMP_14
	call incdec_val
	movwf SEK_TEMP
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp6
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp7
	movfw SEK_TEMP_1
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf SEK_TEMP_1
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp7
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp8
	movlw A'+'
	subwf PM_TEM,w
	btfss STATUS, Z	
	goto $+4	
	movlw A'-'
	movwf PM_TEM
	goto $+3
	movlw A'+'
	movwf PM_TEM
end_dig_temp7
	movfw PM_TEM
	addlw 0xD0
	goto incdec_temp_end
incdec_temp8
	movlw d'8'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp9
	movfw GRD_TEM
	movwf TEMP_13
	movlw d'9'
	movwf TEMP_14
	call incdec_val
	movwf GRD_TEM
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp9
	movlw d'9'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp10
	movfw GRD_TEM_1
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf GRD_TEM_1
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp10
	movlw d'10'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp11
	movfw MIN_TEM
	movwf TEMP_13
	movlw d'6'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEM
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp11
	movlw d'11'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp12
	movfw MIN_TEM_1
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEM_1
	goto incdec_temp_end	;springe zum Ende des Unterprogramms
incdec_temp12
	movlw d'12'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_temp_end
	movfw MIN_TEM_01
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEM_01
incdec_temp_end
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end


incdec_val					
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto dec_dig_temp		;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	incf TEMP_13
	movfw TEMP_14
	subwf TEMP_13,w
	btfsc STATUS, Z		
	clrf TEMP_13
	goto end_dig_temp		;weiter bei kopieren des Wertes
dec_dig_temp
	movfw TEMP_13			;hole Wert ins W-Register
	btfsc STATUS, Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf TEMP_13			;reduziere Zhler um 1
	goto $+4				;schiebe Wert in temporres Display Register
	decf TEMP_14
	movfw TEMP_14				;
	movwf TEMP_13			;schiebe Wert in temporres Register
end_dig_temp
	movfw TEMP_13
	return


incdec_para1				;Unterprogramm zur Vernderung der Werte im MenuepunktParameter
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_para2	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+3				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	incf STD_TEMP			;erhhe Zhler um 1
	goto $+2				;weiter bei kopieren des Wertes
	decf STD_TEMP
	movfw STD_TEMP
	goto incdec_paraend
incdec_para2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_para3
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+3				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	incf MIN_TEMP
	goto $+2				;weiter bei kopieren des Wertes
	decf MIN_TEMP
	movfw MIN_TEMP
	goto incdec_paraend
incdec_para3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_paraend
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+3				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	incf SEK_TEMP
	goto $+2				;weiter bei kopieren des Wertes
	decf SEK_TEMP
	movfw SEK_TEMP
incdec_paraend
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	call convdisp_hex_3dig	;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end


incdec_eepr1				;Unterprogramm zur Vernderung der Katalognr. in den Menuepunkten Goto, Position aus EEPROM
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_L				;kopieren der Werte
	movwf ZAHL_C0
	movfw KAT_H
	movwf ZAHL_C1
	clrf ZAHL_C2
	bsf PCLATH,4			;Select Page 3	
	call convert_hex_dig	;Unterprogramm zur Umwandlung der Katalognummer im Hex Format in 5 Zeichen
	bcf PCLATH,4			;Select Page 1	
	bcf STATUS,RP0			;Select BANK_0			
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_eepr2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_3
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_3
	goto incdec_eepr_end	;springe zum Ende des Unterprogramms
incdec_eepr2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_eepr3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_2
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_2
	goto incdec_eepr_end	;springe zum Ende des Unterprogramms
incdec_eepr3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_eepr4		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_1
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_1
	goto incdec_eepr_end	;springe zum Ende des Unterprogramms
incdec_eepr4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_eepr_end	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_0
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1
	movwf KAT_0
incdec_eepr_end
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_kat_no		;Anzeige der Katalognummer auf dem Display
	bsf PCLATH,3			;Select Page 3	
	bsf PCLATH,4			;	
	call convert_dig_hex	;Unterprogramm zur Umwandlung der 5 Zeichen der Kat.Nr. in das Hex Format
	bcf PCLATH,4			;Select Page 1	
	bsf STATUS,RP0			;Select BANK_1
	movfw ZAHL_C0			;kopieren der Werte
	movwf KAT_L
	movfw ZAHL_C1
	movwf KAT_H	
	bcf STATUS,RP0			;Select BANK_0
	call search_kat			;Aufruf des Unterprogramms zur suche der Katalognummer aus dem externen EEPROM
	goto incdec_test_end



incdec_uz1					;Unterprogramm zur Vernderung der Werte im Menuepunkt Datum/Uhrzeit Manuell
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_uz2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto dec_uz1			;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1	
	incf DATEN_4			;Tag, erhhe Wert
	movlw d'32'				;prfen ob Wert zu hoch
	subwf DATEN_4,w
	btfsc STATUS,Z
	goto $+3				;Wert zu hoch, rcksetzen mit Wert 1
	movfw DATEN_4
	goto incdec_uz_end		;Wert nicht zu hoch
	movlw d'1'				;Wert zu hoch, rcksetzen mit Wert 1
	movwf DATEN_4
	goto incdec_uz_end
dec_uz1
	bsf STATUS,RP0			;Select BANK_1	
	decfsz DATEN_4			;Tag, reduziere Wert und prfe ob 0
	goto $+4				;Wert noch nicht 0
	movlw d'31'				;Wert = 0, rcksetzen mit Wert 31
	movwf DATEN_4
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
	movfw DATEN_4
	goto incdec_uz_end		;Wert noch nicht 0
incdec_uz2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_uz3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto dec_uz2			;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1	
	incf DATEN_6			;Monat, erhhe Wert
	movlw d'13'				;prfen ob Wert zu hoch
	subwf DATEN_6,w
	btfsc STATUS,Z
	goto $+3				;Wert zu hoch, rcksetzen mit Wert 1
	movfw DATEN_6
	goto incdec_uz_end		;Wert nicht zu hoch
	movlw d'1'				;Wert zu hoch, rcksetzen mit Wert 1
	movwf DATEN_6
	goto incdec_uz_end
dec_uz2
	bsf STATUS,RP0			;Select BANK_1	
	decfsz DATEN_6			;Monat, reduziere Wert und prfe ob 0
	goto $+4				;Wert noch nicht 0
	movlw d'12'				;Wert = 0, rcksetzen mit Wert 12
	movwf DATEN_6
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
	movfw DATEN_6
	goto incdec_uz_end		;Wert noch nicht 0
incdec_uz3
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_7			;Jahr,
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'100'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf DATEN_7
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
incdec_uz4
	movlw d'4'				;prfen ob der 4. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz5			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_3			;Stunde,
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'24'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf DATEN_3
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
incdec_uz5
	movlw d'5'				;prfen ob der 5. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz6			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_2			;Minute,
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'60'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf DATEN_2
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
incdec_uz6
	movlw d'6'				;prfen ob der 5. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz7			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_5			;Sekunde,
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'60'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf DATEN_5
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
incdec_uz7
	movlw d'7'				;prfen ob der 5. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz8			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1
	movlw A'+'				;prfen ob aktuelles Vorzeichen = +
	subwf DATEN_8,w			;Vorzeichen Zeitzone,
	btfss STATUS,Z			;	
	goto $+3				;Vorzeichen ist -, ersetze durch +
	movlw A'-'				;Vorzeichen ist +, ersetze durch -
	goto $+2
	movlw A'+'				;Vorzeichen ist -, ersetze durch +
	movwf DATEN_8
	bcf STATUS,RP0			;Select BANK_0
	movfw TEMP_DIG_ADR
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_8			;hole den genderten Wert nach W
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end	;springe zum Ende des Unterprogramms
incdec_uz8
	movlw d'8'				;prfen ob der 5. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_uz_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_1			;Zeitzone,
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'12'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf DATEN_1
	goto incdec_uz_end		;springe zum Ende des Unterprogramms
incdec_uz_end
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end



incdec_so1					;Unterprogramm zur Vernderung der Werte im Menuepunkt Standort
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_so2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw GRD_TEM			;Daten fr Grad
	movwf TEMP_13
	movlw d'180'
	movwf TEMP_14
	call incdec_val
	movwf GRD_TEM
	goto incdec_so_end		;springe zum Ende des Unterprogramms
incdec_so2					;
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_so3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw MIN_TEM			;Daten fr Winkelminuten
	movwf TEMP_13
	movlw d'60'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEM
	goto incdec_so_end		;springe zum Ende des Unterprogramms
incdec_so3
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_so4			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movfw MIN_TEM_01		;Daten fr Winkelsekunden
	movwf TEMP_13
	movlw d'60'
	movwf TEMP_14
	call incdec_val
	movwf MIN_TEM_01
	goto incdec_so_end		;springe zum Ende des Unterprogramms
incdec_so4
	movlw d'4'				;prfen ob der 4. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z			;prfen ob Ergebniss =0
	goto incdec_so_end		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0x00
	btfss PM_TEM,0			;Daten fr Ost=0, West=1
	movlw 0x01
	movwf PM_TEM	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	movfw TEMP_DIG_ADR	
	call disp_befehl	
	movlw A'W'
	btfss PM_TEM,0			;Daten fr Ost=0, West=1
	movlw A'E'
	call disp_daten			;bertrage das Zeichen
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end	;springe zum Ende des Unterprogramms
incdec_so_end
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	movlw 0x01
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto $+8
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	call convdisp_hex_3dig	;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	call convdisp_hex_2dig	;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end



incdec_dcfpara1				;Unterprogramm zur Vernderung der Werte im Menuepunkt 8.3 DCF Parameter
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_dcfpara2	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+4				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1		
	incf DCF_L_MIN			;erhhe Zhler um 1
	goto $+3				;weiter bei kopieren des Wertes
	bsf STATUS,RP0			;Select BANK_1	
	decf DCF_L_MIN
	movfw DCF_L_MIN
	goto incdec_dcfparaend
incdec_dcfpara2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_dcfpara3
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+4				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf DCF_L_MAX
	goto $+3				;weiter bei kopieren des Wertes
	bsf STATUS,RP0			;Select BANK_1
	decf DCF_L_MAX
	movfw DCF_L_MAX
	goto incdec_dcfparaend
incdec_dcfpara3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_dcfpara4
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+4				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf DCF_H_MIN
	goto $+3				;weiter bei kopieren des Wertes
	bsf STATUS,RP0			;Select BANK_1
	decf DCF_H_MIN
	movfw DCF_H_MIN
	goto incdec_dcfparaend
incdec_dcfpara4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto incdec_dcfparaend
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des Wertes gesetzt ist
	goto $+4				;nein, Bit ist nicht gesetzt, gehe zu Reduzierung des Wertes
	bsf STATUS,RP0			;Select BANK_1
	incf DCF_H_MAX
	goto $+3				;weiter bei kopieren des Wertes
	bsf STATUS,RP0			;Select BANK_1
	decf DCF_H_MAX
	movfw DCF_H_MAX
incdec_dcfparaend
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	call convdisp_hex_3dig	;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end

incdec_motor1				;
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_motor2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_14,0			;RA Motor: 0=Aus, 1=Ein, prfen ob Bit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, 
	bcf TEMP_14,0			;Bit ist gesetzt, lsche Bit
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
	bsf TEMP_14,0			;Bit ist nicht gesetzt, setze Bit
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
incdec_motor2
	movlw d'2'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_motor3		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_14,1			;DE Motor: 0=Aus, 1=Ein, prfen ob Bit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, 
	bcf TEMP_14,1			;Bit ist gesetzt, lsche Bit
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
	bsf TEMP_14,1			;Bit ist nicht gesetzt, setze Bit	
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
incdec_motor3
	movlw d'3'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_motor4		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_14,2			;Drehrichtung RA Motor: 0=Normal, 1=Invertiert, prfen ob Bit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, 
	bcf TEMP_14,2			;Bit ist gesetzt, lsche Bit
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
	bsf TEMP_14,2			;Bit ist nicht gesetzt, setze Bit	
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
incdec_motor4
	movlw d'4'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto incdec_motor_end	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfss TEMP_14,3			;Drehrichtung DE Motor: 0=Normal, 1=Invertiert, prfen ob Bit gesetzt ist
	goto $+3				;Bit ist nicht gesetzt, 
	bcf TEMP_14,3			;Bit ist gesetzt, lsche Bit
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
	bsf TEMP_14,3			;Bit ist nicht gesetzt, setze Bit	
	goto incdec_motor_end	;springe zum Ende des Unterprogramms
incdec_motor_end
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_set_motor
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end


incdec_paraaut1
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut2	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_5				;Gesamtbersetzung Ig, 100.000er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_5
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut2
	movlw d'2'				;prfen ob der 2. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut3	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_4				;Gesamtbersetzung Ig, 10.000er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_4
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut3
	movlw d'3'				;prfen ob der 3. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut4	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_3				;Gesamtbersetzung Ig, 1.000er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_3
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut4
	movlw d'4'				;prfen ob der 4. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut5	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_2				;Gesamtbersetzung Ig, 100er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_2
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut5
	movlw d'5'				;prfen ob der 5. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut6	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_1				;Gesamtbersetzung Ig, 10er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_1
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut6
	movlw d'6'				;prfen ob der 6. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut7	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_0				;Gesamtbersetzung Ig, 1er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf KAT_0
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut7
	movlw d'7'				;prfen ob der 7. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z			;prfen ob Ergebniss =0		
	goto incdec_paraaut_end	;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	bsf STATUS,RP0			;Select BANK_1	
	movfw ZZ_UZ				;Nmax, 100er
	bcf STATUS,RP0			;Select BANK_0	
	movwf TEMP_13
	movlw d'10'
	movwf TEMP_14
	call incdec_val
	bsf STATUS,RP0			;Select BANK_1	
	movwf ZZ_UZ
	goto incdec_paraaut_end	;springe zum Ende des Unterprogramms
incdec_paraaut_end
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_DIG_VAL
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl
	movfw TEMP_DIG_VAL		;hole den genderten Wert nach W, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto incdec_test_end



incdec_test_end
	movfw TEMP_DIG_ADR	
	bcf PCLATH,3			;Select Page 2	
	bsf PCLATH,4
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw d'250'			;Wartezeit von 250ms
	call wait
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	clrf TEMP_DIG_STAT		;lsche das Status Register
	return
	

search_kat
	bsf STATUS,RP0			;Select BANK_1	
	clrf ADR_H_MIN			;kleinste Adresse (/16), High Byte
	clrf ADR_L_MIN			;kleinste Adresse (/16), Low Byte
	movlw 0x40				;grte Adresse (/16, dez:16384, hex:4000), High Byte:40
	movwf ADR_H_MAX
	movlw 0x00				;grte Adresse (/16, dez:16384, hex:4000), Low Byte:00
	movwf ADR_L_MAX	
	clrf ADR_H1
	clrf ADR_L1
loop_search_kat
	movfw ADR_L_MIN
	movwf ADR_L
	movfw ADR_H_MIN
	movwf ADR_H
	movfw ADR_L_MAX	
	addwf ADR_L,f
	btfsc STATUS,C
	incf ADR_H
	movfw ADR_H_MAX
	addwf ADR_H,f
	bcf STATUS,C
	rrf ADR_H
	rrf ADR_L
	movfw ADR_H			;
	subwf ADR_H1,w		;prfen ob die neue Adresse sich genber der alten gendert hat
	btfss STATUS,Z
	goto $+5
	movfw ADR_L
	subwf ADR_L1,w
	btfsc STATUS,Z
	goto not_available
	movfw ADR_L
	movwf ADR_L1
	movfw ADR_H
	movwf ADR_H1

	clrf ADR_DEV
	movlw d'4'
	movwf DATEN_1
	bcf STATUS,C
	rlf ADR_L
	rlf ADR_H
	rlf ADR_DEV
	decfsz DATEN_1
	goto $-4
	bcf STATUS,C
	rlf ADR_DEV
	movfw ADR_DEV
	addlw b'10100000'
	movwf ADR_DEV		
	bcf STATUS,RP0			;Select BANK_0	
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call Lese_EEPROM		;Unterprogramm zum Lesen der Katalognummer zur gewhlten EEPROM Adresse!!!!!!!!!!!!!!!!
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
	bsf STATUS,RP0			;Select BANK_1	
	movfw KAT_H
	subwf DATEN_1,w
	btfss STATUS,Z
	goto $+5
	movfw KAT_L
	subwf DATEN_2,w
	btfsc STATUS,Z
	goto search_kat_end
	btfss STATUS,C
	goto val_groesser
	movfw ADR_H1
	movwf ADR_H_MAX
	movfw ADR_L1
	movwf ADR_L_MAX			;
	goto loop_search_kat
val_groesser
	movfw ADR_H1
	movwf ADR_H_MIN
	movfw ADR_L1
	movwf ADR_L_MIN			;
	goto loop_search_kat
not_available
	movfw KAT_L				;kopieren der Werte
	movwf ZAHL_C0
	movfw KAT_H
	movwf ZAHL_C1
	clrf ZAHL_C2
	bsf PCLATH,4			;Select Page 3	
	call convert_hex_dig	;Unterprogramm zur Umwandlung der Katalognummer im Hex Format in 5 Zeichen
	bcf PCLATH,3			;Select Page 2		
	bcf STATUS,RP0			;Select BANK_0	
	call clear_display
	call disp_kat_no		;Anzeige der Katalognummer auf dem Display
	movlw 0x20				;blank	
	call disp_daten			;bertrage das Zeichen			
	movlw A'N'				;ASCII:N
	call disp_daten			;bertrage das Zeichen
	movlw A'/'				;ASCII:/
	call disp_daten			;bertrage das Zeichen
	movlw A'A'				;ASCII:A
	call disp_daten			;bertrage das Zeichen
	bsf PCLATH,3			;Select Page 1	
	bcf PCLATH,4
	goto search_kat_end1
search_kat_end
	bcf STATUS,RP0			;Select BANK_0	
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call Lese_EEPROM_1
	call disp_ee
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4
search_kat_end1
	bcf STATUS,RP0			;Select BANK_0	
	return

quarz1hz_set				;Unterprogramm zur Anzeige der mittleren Abweichung zwischen interner Uhr und DCF Signal
	bcf STATUS,RP0			;Select BANK_0, in ppm, verursacht durch eine zu hohe bzw. zu niedrige Oszillator Frequenz.
	bcf OUT_RA_ENABLE		;Motoren abschalten, um DCF Signal nicht zu stren
	bcf OUT_DE_ENABLE
	movfw STAT_MOTOR		;Register sichern in TEMP_17 Register
	movwf TEMP_17
	bcf STAT_MOTOR,0		;Bits lschen
	bcf STAT_MOTOR,1
							;Lschen der Register
	bsf STATUS,RP0			;Select BANK_1 
	clrf ADR_L				;Wird verwendet fr Sekunden Zhler L Byte
	clrf ADR_H				;Wird verwendet fr Sekunden Zhler H Byte
	bsf PCLATH,4			;Select Page 3
	call clear_zahl_c		;Lsche ZAHL_C Register, werden fr Abweichung genutzt
	bcf PCLATH,4			;Select Page 1
	movlw 0x30				;obere 4Bit fr Anzeige der Werte Auf dem Display
	movwf KAT_0				;kopiere obere 4Bit in Display Register 
	movwf KAT_1				;
	movwf KAT_2				;
	movwf KAT_3				;

	movlw A' '				;ASCII: Blank
	movwf ZZ_UZ				;wird verwendet fr Vorzeichen Abweichung

	bcf STATUS,RP0			;Select BANK_0 
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4
	call clear_display		;Aufruf des Unterprogramms zum lschen des Display
	call cursor_off			;Aufruf des Unterprogramms zum ausschalten des Cursors
	call disp_set_q1hz		;Anzeige von ppm/s auf dem Display
	movlw 0x80				;Displayadresse	fr Vorzeichen
	call disp_set_q1hz_vz	;Anzeige des Vorzeichens auf dem Display
	movlw 0x81				;Displayadresse	fr Abweichung
	call disp_set_q1hz_ppms	;Anzeige der Abweichung auf dem Display
	movlw 0x89				;Displayadresse	fr abgelaufene Sekunden
	call disp_set_q1hz_ppms	;Anzeige der abgelaufenen Sekunden auf dem Display
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4

	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto menu_q1hz_end		;Slave will Befehl senden => springe zum Ende der Programmschleife
	btfss IN_DCF			;Warten auf die positive Flanke des DCF Signals
	goto $-3				;DCF Signal noch negativ, zurck zur Warteschleife
	btfss DAT_RX			;positive Flanke, prfen, ob Slave keine Daten senden will
	goto menu_q1hz_end		;Slave will Befehl senden => springe zum Ende der Programmschleife
	btfsc IN_DCF			;Warten auf die negative Flanke des DCF Signals => Beginn einer neuen Sekunde.
	goto $-3				;DCF Signal noch positiv, zurck zur Warteschleife
	bcf NEU_DISP,7			;negative Flanke
	movlw d'4'				;Mit dieser negativen Flanke des DCF Signals wird die interne Uhr syncronisiert. 
	movwf TMR0				;(4 Takte Adjustment durch Zeitdifferenz) 
							;Rcksetzen des Sekundenzhlers der internen Uhr
	movlw d'31'				;High Byte = 31
	movwf COUNT_H_UZ		;
	movlw d'132'			;Low Byte = 132 (Loop 1)
	movwf COUNT_L_UZ
	movlw 0x02				;Setzen des Zhlers, Digit:= 2	
	movwf DIGIT_UZ			;
	bcf NEU_DISP,1			;lsche Sekunden berlauf Bit
q1hz_loop
	bsf PCLATH,4			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1 
	call clear_zahl_c		;Lsche ZAHL_C Register, werden fr Abweichung genutzt
	bcf STATUS,RP0			;Select BANK_0 
	bcf PCLATH,4			;Select Page 1
q1hz_loop1
	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto menu_q1hz_end		;Slave will Befehl senden => springe zum Ende der Programmschleife
	btfss IN_DCF			;Warten auf die positive Flanke des DCF Signals
	goto q1hz_loop1			;DCF Signal ist noch Low, zurck zur Warteschleife  
q1hz_loop2
	btfss IN_DCF			;Warten auf die negative Flanke des DCF Signals
	goto ext_first			;Negative Flanke liegt an => d.h. das DCF Signal ist "schneller" als die interne Uhr
	btfsc NEU_DISP,1		;Prft ob eine Sekunde der internen Uhr abgelaufen ist 
	goto int_first			;Sekunde ist abgelaufen => d.h. die interne Uhr ist "schneller" als das DCF Signal 
	goto q1hz_loop2			;
int_first
	bcf NEU_DISP,1			;lsche Sekunden- und Timer 0 berlauf Bits
	bcf NEU_DISP,7
int_first1
	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto menu_q1hz_end		;Slave will Befehl senden => springe zum Ende der Programmschleife
	btfsc IN_DCF			;prfen der negativen Flanke des DCF Signals
	goto $+6				;DCF Signal noch positiv, weiter bei Prfung des Timer 0 berlauf Bits
	movfw TMR0				;DCF Signal negativ, sichern des Timer 0 Werts im L Byte des 24 Bit Abweichungs Registers
	bsf STATUS,RP0			;Select BANK_1 	
	movwf ZAHL_C0
	movlw A'+'				;Vorzeichen fr Abweichung
	goto auswert_q1hz		;Weiter zur Auswertung

	btfss NEU_DISP,7		;Prft den Timer 0 berlauf
	goto int_first1			;es liegt kein berlauf vor, zurck zur Schleife
	bcf NEU_DISP,7			;Timer 0 ist bergelaufen, lsche das berlauf Bit
	bsf STATUS,RP0			;Select BANK_1 	
	incfsz ZAHL_C1			;Erhhe das M Byte des 24 Bit Abweichungs Registers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf ZAHL_C2			;M Byte des 24 Bit Abweichungs Registers ist bergelaufen, erhhe H Byte um 1
	movlw d'6'				;wenn nach 0,197sek keine negative Flanke anliegt (z.B. in der 59sten Sek des DCF Protokolls),
	subwf ZAHL_C2,w			;  wird die Messung abgebrochen.
	bcf STATUS,RP0			;Select BANK_0 	
	btfss STATUS,Z
	goto int_first1			;0,197sek Marke noch nicht erreicht, zurck zur int_first Schleife
	goto auswert_q1hz		;0,197sek Marke ist erreicht, Abbruch und weiter bei nchster Messung

ext_first
	movfw TMR0				;sichern des Timer 0 Wert im L Byte des 24 Bit Abweichungs Registers
	bsf STATUS,RP0			;Select BANK_1 	
	movwf ZAHL_C0
	bcf STATUS,RP0			;Select BANK_0 	
ext_first1
	bsf INTCON,GIE			;erlaube globale Interrupts
	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto menu_q1hz_end		;Slave will Befehl senden => springe zum Ende der Programmschleife
	bcf INTCON,GIE			;verbiete globale Interrupts
	btfss NEU_DISP,7		;Prft den Timer 0 berlauf
	goto $+6				;es liegt kein berlauf vor, weiter bei Prfung, ob eine Sekunde abgelaufen ist
	bcf NEU_DISP,7			;Timer 0 ist bergelaufen, lsche das berlauf Bit
	bsf STATUS,RP0			;Select BANK_1 	
	incfsz ZAHL_C1			;Erhhe das M Byte des 24 Bit Abweichungs Registers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf ZAHL_C2			;M Byte des 24 Bit Abweichungs Registers ist bergelaufen, erhhe H Byte um 1
	bcf STATUS,RP0			;Select BANK_0 	
	btfss NEU_DISP,1		;Prft ob eine Sekunde abgelaufen ist 
	goto ext_first1			;Sekunde ist noch nicht um, zurck zur ext_first Schleife
	bsf INTCON,GIE			;Sekunde ist um, erlaube globale Interrupts
	bcf NEU_DISP,1			;lsche Sekunden berlauf Bit
	bsf STATUS,RP0			;Select BANK_1 		
	movfw ZAHL_C0			;Sekunde ist um, Berechnung der Differenz des im L Byte gespeicherten Timer 0
	sublw d'0'				;  Wertes zum Timer 0 berlauf ( = 256 - Timer 0 )
	movwf ZAHL_C0			;Rcksichern des Ergebnisses im L Byte des 24 Bit Abweichungs Registers
	movlw 0x01				;M Byte um 1 reduzieren
	subwf ZAHL_C1
	btfss STATUS,C			;prfen ob M Byte unterlaufen, d.h. <0
	decf ZAHL_C2			;M Byte <0, reduziere H Byte um 1
	movlw A'-'				;Vorzeichen fr Abweichung

auswert_q1hz
	bsf STATUS,RP0			;Select BANK_1 	
	movwf ZZ_UZ				;Sichern des Vorzeichen fr Abweichung
	incfsz ADR_L			;erhhe das Low Byte des Sekundenzhlers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf ADR_H				;Low Byte des Sekundenzhlers ist bergelaufen, erhhe High Byte um 1

	movfw ADR_L				;kopiere die L+H Sekunden Register in die Nenner Register fr die Division
	movwf ZAHL_A0			;   ( Die Werte des 24 Bit Abweichungs Registers ZAHL_C2:C0 stehen schon 
	movfw ADR_H				;    in den Zhler Registern fr die Division)
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	bsf PCLATH,4			;Select Page 3 (Unterprogramm zur Division steht auf Page 3)
	call div32				;Aufruf des Unterprogramms zur Division ( Aufgelaufene Abweichung geteilt durch Abgelaufene Sekunden)
	bcf PCLATH,4			;Select Page 1, Ergebnis der Division steht in ZAHL_C Registern und mu noch
	bcf STATUS,C			;	durch 2 geteilt werden, da 1 Takt von TMR0 0,5us entspricht.
	rrf ZAHL_C2				;Register durch 2 teilen, d.h. nach rechts schieben: => us => ppm Abweichung
	rrf ZAHL_C1
	rrf ZAHL_C0
	btfsc STATUS,C			;Ergebnis runden: prfen ob das letzte ausgeschobene Bit 1 oder 0 war
	incfsz ZAHL_C0			;Bit war 1, L Byte um 1 erhhen und prfen auf berlauf
	goto $+4				;Bit war 0, weiter
	incfsz ZAHL_C1			;berlauf vorhanden, erhhe M Byte und prfe auf berlauf
	goto $+2				;kein berlauf
	incf ZAHL_C2			;berlauf vorhanden, erhhe L Byte
	bsf PCLATH,4			;Select Page 3
	call convert_hex_dig	;Aufruf des Unterprogramms zum zerlegen der Abweichung im Hex Format in 
	bcf PCLATH,4			;einzelne Zahlen zur Anzeige auf dem Display, Select Page 1
	movlw 0x30				;obere 4 bit fr Display Anzeige (=>ASCII Format)
	addwf KAT_0
	addwf KAT_1
	addwf KAT_2
	addwf KAT_3
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	movlw 0x80				;Displayadresse	fr Vorzeichen
	call disp_set_q1hz_vz	;Anzeige des Vorzeichens auf dem Display
	movlw 0x81				;Displayadresse	fr Abweichung
	call disp_set_q1hz_ppms	;Anzeige der Abweichung auf dem Display
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1 
	call clear_zahl_c		;Lsche ZAHL_C Register, werden fr Abweichung genutzt
	bcf PCLATH,4			;Select Page 1
	movfw ADR_L				;kopiere die L+H Sekunden Register in ZAHL_C Register fr die Auftrennung in einzelne Digits
	movwf ZAHL_C0			;
	movfw ADR_H				;
	movwf ZAHL_C1

	bsf PCLATH,4			;Select Page 3
	call convert_hex_dig	;Aufruf des Unterprogramms zum zerlegen der abgelaufenen Sekunden im Hex Format in 
	bcf PCLATH,4			;einzelne Zahlen zur Anzeige auf dem Display, Select Page 1
	movlw 0x30				;obere 4 bit fr Display Anzeige (=>ASCII Format)
	addwf KAT_0
	addwf KAT_1
	addwf KAT_2
	addwf KAT_3
	bcf PCLATH,3			;Select Page 2
	bsf PCLATH,4			;
	movlw 0x89				;Displayadresse	fr Abweichung
	call disp_set_q1hz_ppms	;Anzeige der abgelaufenen Sekunden auf dem Display
	bsf PCLATH,3			;Select Page 1
	bcf PCLATH,4			;	

	bcf STATUS,RP0			;Select BANK_0 
	goto q1hz_loop
menu_q1hz_end
	bcf STATUS,RP0			;Select BANK_0 
	movfw TEMP_17			;Rcksichern des TEMP Registers in Arbeitsregister
	movwf STAT_MOTOR
	btfsc STAT_MOTOR,0		;wenn Statusbits fr Motoren "Ein" gesetzt sind, Ausgnge setzen
	bsf OUT_RA_ENABLE
	btfsc STAT_MOTOR,1
	bsf OUT_DE_ENABLE	
	bcf PCLATH,3			;Select Page 0
	goto menue_end

Zeit_Null						;Unterprogramm zur Rcksetzung der Zeit
	clrf SEK_SZ				;
	clrf MIN_SZ
	clrf STD_SZ
	movlw d'6'
	movwf DIGIT_SZ
	clrf SEK_UZ				;
	clrf MIN_UZ
	clrf STD_UZ
	movlw d'2'
	movwf DIGIT_UZ
	clrf SEK_RA				;
	clrf MIN_RA
	clrf STD_RA
	clrf MIN_DE_01
	clrf MIN_DE
	movlw d'90'
	movwf GRD_DE
	movlw A'+'
	movwf PM_DE
	clrf STAT_POS
	return						;Rcksprung aus dem Unterprogramm


ra_richtung
	movfw STD_TEMP
	subwf STD_RA,w				;Ist - Goto
	btfsc STATUS,Z				;prfe ob Ergebnis <0>
	goto richtung_ra_min		;wenn ja berspringe diesen Befehl, wenn nein weiter bei Test Minuten
	btfss STATUS,C				;prfe ob Ergebnis <0
	addlw d'24'
	sublw d'12'
	goto richtung_ra_set

richtung_ra_min 
	movfw MIN_TEMP
	subwf MIN_RA,w				;ist Position minus Goto-Position
	btfsc STATUS,Z				;prfe ob Ergebnis <0>
	goto richtung_ra_sec		;wenn ja berspringe diesen Befehl, wenn nein weiter bei Test Minuten
	goto richtung_ra_set

richtung_ra_sec
	movfw SEK_TEMP
	subwf SEK_RA,w				;ist Position minus Goto-Position
	btfsc STATUS,Z				;prfe ob Ergebnis <0>
	goto richtung_ra_end		;wenn ja berspringe diesen Befehl, wenn nein weiter bei Test Minuten

richtung_ra_set
	btfsc STATUS,C				;prfe ob Ergebnis <0
	goto ra_rechts_set
ra_links_set					;Ermittlung Startpunkt fr Bremsrampe und setzen des Ausgangs (Rechts/Links)
	movfw SEK_TEMP				;Kopiere Goto Werte in Temp Register
	movwf TEMP_13
	movfw MIN_TEMP
	movwf TEMP_14
	movfw STD_TEMP
	movwf TEMP_15
	movfw SEK_RA				;Kopiere Ist Werte in Temp Register
	movwf SEK_TEMP_1
	movfw MIN_RA
	movwf MIN_TEMP_1
	movfw STD_RA
	movwf STD_TEMP_1
	call calc_brramp_ra			;Berechnung der Differenz zur Goto Position als Startpunkt fr Bremsrampe
	movfw SEK_TEMP				;Kopiere Goto Werte in Temp Register
	movwf SEK_TEMP_1
	movfw MIN_TEMP
	movwf MIN_TEMP_1
	movfw STD_TEMP
	movwf STD_TEMP_1

	movfw TEMP_13				;subtrahiere Diff sek von Goto sek
	subwf SEK_TEMP_1
	btfsc STATUS,C				;prfen, ob Goto - Differenz negativ (sek)
	goto $+0x0F					;positiv, weiter bei Minuten
	movlw d'60'					;negativ, addiere 60 zu Goto sek
	addwf SEK_TEMP_1,f
	movlw d'1'					; und reduziere Minuten um 1
	subwf MIN_TEMP_1,f
	btfsc STATUS,C				;prfen ob Minuten negativ
	goto $+9					;positiv, weiter bei Minuten
	movlw d'60'					;negativ, addiere 60 zu Goto min
	addwf MIN_TEMP_1,f
	movlw d'1'					; und reduziere Stunden um 1
	subwf STD_TEMP_1,f
	btfsc STATUS,C				;prfen ob Stunden negativ
	goto $+3					;positiv, weiter bei Minuten
	movlw d'24'					;negativ, addiere 24 zu Goto std
	addwf STD_TEMP_1,f

	movfw TEMP_14				;subtrahiere Diff min von Goto min
	subwf MIN_TEMP_1,f
	btfsc STATUS,C				;prfen, ob Goto - Differenz negativ (min)
	goto $+9					;positiv, weiter 
	movlw d'60'					;negativ, addiere 60 zu Goto min
	addwf MIN_TEMP_1,f
	movlw d'1'					; und reduziere Stunden um 1
	subwf STD_TEMP_1,f
	btfsc STATUS,C				;prfen ob Stunden negativ
	goto $+3					;positiv, weiter
	movlw d'24'					;negativ, addiere 24 zu Goto std
	addwf STD_TEMP_1,f

	bcf PCLATH,3				;Select Page 0
	call set_ra_links			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf PCLATH,3				;Select Page 1
	return
ra_rechts_set					;Ermittlung Startpunkt fr Bremsrampe und setzen des Ausgangs (Rechts/Links)
	movfw SEK_TEMP				;Kopiere Goto Werte in Temp Register
	movwf SEK_TEMP_1
	movfw MIN_TEMP
	movwf MIN_TEMP_1
	movfw STD_TEMP
	movwf STD_TEMP_1
	movfw SEK_RA				;Kopiere Ist Werte in Temp Register
	movwf TEMP_13
	movfw MIN_RA
	movwf TEMP_14
	movfw STD_RA
	movwf TEMP_15
	call calc_brramp_ra			;Berechnung der Differenz zur Goto Position als Startpunkt fr Bremsrampe
	movfw SEK_TEMP				;Kopiere Goto Werte in Temp Register
	movwf SEK_TEMP_1
	movfw MIN_TEMP
	movwf MIN_TEMP_1
	movfw STD_TEMP
	movwf STD_TEMP_1

	movfw TEMP_13				;addiere Diff sek zu Goto sek
	addwf SEK_TEMP_1
	movlw d'60'
	subwf SEK_TEMP_1,w
	btfss STATUS,C				;prfen, ob Goto - Differenz negativ (sek)
	goto $+0x13					;positiv, weiter bei Minuten
	movlw d'60'					;negativ, addiere 60 zu Goto sek
	subwf SEK_TEMP_1,f
	movlw d'1'					; und reduziere Minuten um 1
	addwf MIN_TEMP_1,f
	movlw d'60'
	subwf MIN_TEMP_1,w
	btfss STATUS,C				;prfen ob Minuten negativ
	goto $+0x0B					;positiv, weiter bei Minuten
	movlw d'60'					;negativ, addiere 60 zu Goto min
	subwf MIN_TEMP_1,f
	movlw d'1'					; und reduziere Stunden um 1
	addwf STD_TEMP_1,f
	movlw d'24'
	subwf STD_TEMP_1,w
	btfss STATUS,C				;prfen ob Stunden negativ
	goto $+3					;positiv, weiter bei Minuten
	movlw d'24'					;negativ, addiere 24 zu Goto std
	subwf STD_TEMP_1,f

	movfw TEMP_14				;subtrahiere Diff min von Goto min
	addwf MIN_TEMP_1,f
	movlw d'60'
	subwf MIN_TEMP_1,w
	btfss STATUS,C				;prfen, ob Goto - Differenz negativ (min)
	goto $+0x0B					;positiv, weiter 
	movlw d'60'					;negativ, addiere 60 zu Goto min
	subwf MIN_TEMP_1,f
	movlw d'1'					; und reduziere Stunden um 1
	addwf STD_TEMP_1,f
	movlw d'24'
	subwf STD_TEMP_1,w
	btfss STATUS,C				;prfen ob Stunden negativ
	goto $+3					;positiv, weiter
	movlw d'24'					;negativ, addiere 24 zu Goto std
	subwf STD_TEMP_1,f

	bcf PCLATH,3				;Select Page 0
	call set_ra_rechts			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf PCLATH,3				;Select Page 1
	return

richtung_ra_end
	bsf STAT_POS,3				;setze Statusbit fr Erreichen der Goto-Position RA
	return


de_richtung						;In diesem Programmteil wird die Drehrichtung DE ermittelt
	btfss STAT_MOTOR,1			;Prfen ob Bit fr DE Motor "Ein" gesetzt ist (=> Enable)
	goto richtung_de_end		;Bit ist nicht gesetzt, setze Statusbit fr Erreichen der Goto Position DE

	movfw GRD_GOTO				;subtrahiere die Goto Position von der Ist Position
	subwf GRD_DE,w
	btfsc STATUS,Z				;prfen ob Ergebnis >0<
	goto richtung_de_min		;Ergebnis =0, gehe zur nchsten Prfung
	goto richtung_de_set		;weiter bei Auswertung
richtung_de_min
	movfw MIN_GOTO				;subtrahiere die Goto Position von der Ist Position
	subwf MIN_DE,w
	btfsc STATUS,Z				;prfen ob Ergebnis >0<
	goto richtung_de_min_01		;Ergebnis =0, gehe zur nchsten Prfung
	goto richtung_de_set		;weiter bei Auswertung
richtung_de_min_01
	movfw MIN_GOTO_01			;subtrahiere die Goto Position von der Ist Position
	subwf MIN_DE_01,w
	btfsc STATUS,Z				;prfen ob Ergebnis >0<
	goto richtung_de_end		;Ergebnis =0, gehe zum Ende des Unterprogramms
richtung_de_set
	btfsc STATUS,C				;prfe ob Ergebnis <0
	goto de_down_set
de_up_set						;Ermittlung Startpunkt fr Bremsrampe und setzen des Ausgangs (Up/Down)
	movfw MIN_GOTO				;Kopiere Goto Werte in Temp Register
	movwf TEMP_14
	movfw GRD_GOTO
	movwf TEMP_15
	movfw MIN_DE				;Kopiere Ist Werte in Temp Register
	movwf MIN_TEM_1
	movfw GRD_DE
	movwf GRD_TEM_1
	call calc_brramp_de			;Berechnung der Differenz zur Goto Position als Startpunkt fr Bremsrampe
	movfw MIN_GOTO				;Kopiere Goto Werte in Temp Register
	movwf MIN_TEM_1
	movfw GRD_GOTO
	movwf GRD_TEM_1

	movfw TEMP_14				;subtrahiere Diff min von Goto min
	subwf MIN_TEM_1,f
	btfsc STATUS,C				;prfen, ob Goto - Differenz negativ (min)
	goto $+9					;positiv, weiter 
	movlw d'60'					;negativ, addiere 60 zu Goto min
	addwf MIN_TEM_1,f
	movlw d'1'					; und reduziere Grad um 1
	subwf GRD_TEM_1,f
	btfsc STATUS,C				;prfen ob Grad negativ
	goto $+3					;positiv, weiter
	movlw d'180'				;negativ, addiere 180 zu Goto Grad
	addwf GRD_TEM_1,f

	bcf PCLATH,3				;Select Page 0
	call set_de_up				;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf PCLATH,3				;Select Page 1
	return

de_down_set
	movfw MIN_GOTO				;Kopiere Goto Werte in Temp Register
	movwf MIN_TEM_1
	movfw GRD_GOTO
	movwf GRD_TEM_1
	movfw MIN_DE				;Kopiere Ist Werte in Temp Register
	movwf TEMP_14
	movfw GRD_DE
	movwf TEMP_15
	call calc_brramp_de			;Berechnung der Differenz zur Goto Position als Startpunkt fr Bremsrampe
	movfw MIN_GOTO				;Kopiere Goto Werte in Temp Register
	movwf MIN_TEM_1
	movfw GRD_GOTO
	movwf GRD_TEM_1

	movfw TEMP_14				;addiere Diff min von Goto min
	addwf MIN_TEM_1,f
	movlw d'60'
	subwf MIN_TEM_1,w
	btfss STATUS,C				;prfen, ob Goto - Differenz negativ (min)
	goto $+0x0B					;positiv, weiter 
	movlw d'60'					;negativ, addiere 60 zu Goto min
	subwf MIN_TEM_1,f
	movlw d'1'					; und reduziere Grad um 1
	addwf GRD_TEM_1,f
	movlw d'180'
	subwf GRD_TEM_1,w
	btfss STATUS,C				;prfen ob Grad negativ
	goto $+3					;positiv, weiter
	movlw d'180'				;negativ, addiere 180 zu Goto std
	subwf GRD_TEM_1,f

	bcf PCLATH,3				;Select Page 0
	call set_de_down			;Unterprogramm zum setzen des Ausgangs bei Drehrichtung Normal/Invertiert 
	bsf PCLATH,3				;Select Page 1
	return
	
richtung_de_end
	bsf STAT_POS,4				;setze Statusbit fr Erreichen der Goto-Position DE
	return



calc_brramp_ra					;Bremsrampe soll 2 min vor Erreichen der Goto Pos Beginnen. 
								;Dafr muss die Differenz Goto - Ist min 4 min betragen (2min Startr. + 2min Bremsr.)
								;Ist die Differenz <4 min, wird Differenz/2 verwendet: Differenz berechnen 
	movfw SEK_TEMP_1			;Ist - Goto, sek
	subwf TEMP_13,f
	btfsc STATUS,C				;prfen ob Differenz negativ
	goto $+0x0F					;positiv, weiter bei min
	movlw d'60'					;negativ, addiere 60 zu sek
	addwf TEMP_13,f
	movlw d'1'					; und reduziere min um 1
	subwf TEMP_14,f
	btfsc STATUS,C				;prfen ob min dadurch negativ
	goto $+9					;nein, positiv, weiter bei min
	movlw d'60'					;negativ, addiere 60 zu min
	addwf TEMP_14,f
	movlw d'1'					; und reduziere Stunden um 1
	subwf TEMP_15,f
	btfsc STATUS,C				;prfen ob Stunden dadurch negativ
	goto $+3					;nein, positiv, weiter bei min
	movlw d'24'					;negativ, addiere 24 zu Stunden
	addwf TEMP_15,f

	movfw MIN_TEMP_1			;Ist - Goto, min
	subwf TEMP_14,f
	btfsc STATUS,C				;prfen ob Differenz negativ
	goto $+9					;positiv, weiter bei Stunden
	movlw d'60'					;negativ, addiere 60 zu min
	addwf TEMP_14,f
	movlw d'1'					; und reduziere Stunden um 1
	subwf TEMP_15,f
	btfsc STATUS,C				;prfen ob Stunden dadurch negativ
	goto $+3					;nein, positiv, weiter bei Stunden
	movlw d'24'					;negativ, addiere 24 zu Stunden
	addwf TEMP_15,f

	movfw STD_TEMP_1			;Ist - Goto, Stunden
	subwf TEMP_15,f
	btfsc STATUS,C				;prfen ob Differenz negativ
	goto $+3					;positiv, weiter bei Prfung der Differenz
	movlw d'24'					;negativ, addiere 24 zu Stunden
	addwf TEMP_15,f
	
	movfw TEMP_15				;prfen, ob Differenz Ist - Goto =0, Prfung Stunden
	btfss STATUS,Z
	goto calc_brramp_ra_end		;Stundendifferenz >0<, d,h. >4 min

	movfw TEMP_14				;prfen, ob Differenz Ist - Goto =0, Prfung Minuten
	btfss STATUS,Z
	goto $+5
	movfw TEMP_13				;prfen, ob Differenz Ist - Goto <2, Prfung Sekunden
	sublw d'2'
	btfsc STATUS,C
	bsf STAT_POS,3				;setze Statusbit fr Erreichen der Goto-Position RA

	movlw d'4'					;prfen, ob Minutendifferenz >4 min
	subwf TEMP_14,w
	btfsc STATUS,C
	goto calc_brramp_ra_end		;Minutendifferenz >=4 min
	bcf STATUS,C				;Minutendifferenz <4 min, lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf TEMP_13,f				;Sekundendifferenz durch 2 teilen
	bcf STATUS,C				;lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf TEMP_14,f				;Minutendifferenz durch 2 teilen
	btfss STATUS,C				;prfen ob das ausgeschobene Bit =1, d.h. Rest = 0,5 Minuten = 30 sek
	return						;ausgeschobene Bit =0, d.h. Rest = 0 Minuten, Rcksprung aus dem Unterprogramm
	movlw d'30'					;ausgeschobene Bit =1, addiere Rest von 30 sek zu Sekundendifferenz
	addwf TEMP_13
	return						;Rcksprung aus dem Unterprogramm
calc_brramp_ra_end				;Minutendifferenz >=4 min
	clrf TEMP_13				;lsche Sekundendifferenz und lade 2 Minuten ins Minutendifferenz Register
	movlw d'2'
	movwf TEMP_14
	return						;Rcksprung aus dem Unterprogramm


calc_brramp_de					;Bremsrampe soll 30 min vor Erreichen der Goto Pos Beginnen. 
								;Dafr muss die Differenz Goto - Ist min. 60 min betragen (30min Startr. + 30min Bremsr.)
								;Ist die Differenz <60 min, wird Differenz/2 verwendet: Differenz berechnen 
	movfw MIN_TEM_1				;Ist - Goto, Minuten
	subwf TEMP_14,f
	btfsc STATUS,C				;prfen ob Differenz negativ
	goto $+9					;positiv, weiter bei Grad
	movlw d'60'					;negativ, addiere 60 zu min
	addwf TEMP_14,f
	movlw d'1'					; und reduziere Grad um 1
	subwf TEMP_15,f
	btfsc STATUS,C				;prfen ob Grad dadurch negativ
	goto $+3					;nein, positiv, weiter bei Grad
	movlw d'180'				;negativ, addiere 180 zu Grad
	addwf TEMP_15,f

	movfw GRD_TEM_1				;Ist - Goto, Grad
	subwf TEMP_15,f
	btfsc STATUS,C				;prfen ob Differenz negativ
	goto $+3					;positiv, weiter bei Prfung der Differenz
	movlw d'180'				;negativ, addiere 180 zu Grad
	addwf TEMP_15,f
	
	movfw TEMP_15				;prfen, ob Differenz Ist - Goto =0, Prfung Grad
	btfss STATUS,Z
	goto calc_brramp_de_end		;Graddifferenz >0<, d,h. >=60 min
	bcf STATUS,C				;Minutendifferenz =<60 min, lsche Carry Bit, damit es beim Rotieren nicht strt
	rrf TEMP_14,f				;Minutendifferenz durch 2 teilen
	return						;Rcksprung aus dem Unterprogramm
calc_brramp_de_end				;Minutendifferenz >=60 min
	movlw d'30'					;lade 30 Minuten ins Minutendifferenz Register
	movwf TEMP_14
	return						;Rcksprung aus dem Unterprogramm



menu_cursor_adr
	movlw 0x0F
	movwf PCLATH				;Programmzhler fuer Cursor Positionstabelle im Bereich 0x0F01..0x0FFF 
	btfsc DIG_REG,0
	movlw d'0'					;Basiswert 
	btfsc DIG_REG,1
	movlw d'12'					;Basiswert fr Menpunkt Parameter
	btfsc DIG_REG,2
	movlw d'15'					;Basiswert fr Menpunkt EEPROM
	btfsc DIG_REG,3
	movlw d'19'					;Basiswert fr Menpunkt Datum/Uhrzeit
	btfsc DIG_REG,4
	movlw d'27'					;Basiswert fr Menpunkt Standort
	btfsc DIG_REG,5
	movlw d'31'					;Basiswert fr Menpunkt DCF Parameter
	btfsc DIG_REG,6
	movlw d'35'					;Basiswert fr Menpunkt Motor
	btfsc DIG_REG,7
	movlw d'39'					;Basiswert fr Menpunkt Parameter Auto
	addwf TEMP_DIG_CNT,w
	call tab_disp_adresse
	movwf TEMP_DIG_ADR
	return


	ORG 0x0FC8

tab_disp_adresse
	addwf PCL,f
	retlw 0x00	;Datenpunkt 0:	Nicht vergeben
	retlw 0x88	;Datenpunkt 1:	Menpunkte GOTO/POSITION manuell Wert 1
	retlw 0x89	;Datenpunkt 2:	Menpunkte GOTO/POSITION manuell Wert 2
	retlw 0x8B	;Datenpunkt 3:	Menpunkte GOTO/POSITION manuell Wert 3
	retlw 0x8C	;Datenpunkt 4:	Menpunkte GOTO/POSITION manuell Wert 4
	retlw 0x8E	;Datenpunkt 5:	Menpunkte GOTO/POSITION manuell Wert 5
	retlw 0x8F	;Datenpunkt 6:	Menpunkte GOTO/POSITION manuell Wert 6
	retlw 0xC8	;Datenpunkt 7:	Menpunkte GOTO/POSITION manuell Wert 7
	retlw 0xC9	;Datenpunkt 8:	Menpunkte GOTO/POSITION manuell Wert 8
	retlw 0xCA	;Datenpunkt 9:	Menpunkte GOTO/POSITION manuell Wert 9
	retlw 0xCC	;Datenpunkt 10:	Menpunkte GOTO/POSITION manuell Wert 10
	retlw 0xCD	;Datenpunkt 11:	Menpunkte GOTO/POSITION manuell Wert 11
	retlw 0xCF	;Datenpunkt 12:	Menpunkte GOTO/POSITION manuell Wert 12
	retlw 0xC2	;Datenpunkt 13:	Menpunkt PARAMETER Wert 1
	retlw 0xC7	;Datenpunkt 14:	Menpunkt PARAMETER Wert 2
	retlw 0xCC	;Datenpunkt 15:	Menpunkt PARAMETER Wert 3
	retlw 0x80	;Datenpunkt 16:	Menpunkt GOTO/POSITION aus EEPROM Wert 1
	retlw 0x81	;Datenpunkt 17:	Menpunkt GOTO/POSITION aus EEPROM Wert 2
	retlw 0x82	;Datenpunkt 18:	Menpunkt GOTO/POSITION aus EEPROM Wert 3
	retlw 0x83	;Datenpunkt 19:	Menpunkt GOTO/POSITION aus EEPROM Wert 4
	retlw 0x84	;Datenpunkt 20:	Menpunkt Datum/Uhrzeit Wert 1, Tag
	retlw 0x87	;Datenpunkt 21:	Menpunkt Datum/Uhrzeit Wert 2, Monat
	retlw 0x8C	;Datenpunkt 22:	Menpunkt Datum/Uhrzeit Wert 3, Jahr
	retlw 0xC4	;Datenpunkt 23:	Menpunkt Datum/Uhrzeit Wert 4, Stunde
	retlw 0xC7	;Datenpunkt 24:	Menpunkt Datum/Uhrzeit Wert 5, Minute
	retlw 0xCA	;Datenpunkt 25:	Menpunkt Datum/Uhrzeit Wert 6, Sekunde
	retlw 0xCD	;Datenpunkt 26:	Menpunkt Datum/Uhrzeit Wert 7, Vorzeichen Zeitzone
	retlw 0xCE	;Datenpunkt 27:	Menpunkt Datum/Uhrzeit Wert 8, Zeitzone
	retlw 0xC2	;Datenpunkt 28:	Menpunkt Standort Wert 1, Grad
	retlw 0xC6	;Datenpunkt 29:	Menpunkt Standort Wert 2, Winkelminuten
	retlw 0xC9	;Datenpunkt 30:	Menpunkt Standort Wert 3, Winkelsekunden
	retlw 0xCD	;Datenpunkt 31:	Menpunkt Standort Wert 4, Ost/West
	retlw 0x84	;Datenpunkt 32:	Menpunkt DCF Parameter Wert 1, Minimalwert fr DCF Bit "Low"
	retlw 0x8C	;Datenpunkt 33:	Menpunkt DCF Parameter Wert 2, Maximalwert fr DCF Bit "Low"
	retlw 0xC4	;Datenpunkt 34:	Menpunkt DCF Parameter Wert 3, Minimalwert fr DCF Bit "High"
	retlw 0xCC	;Datenpunkt 35:	Menpunkt DCF Parameter Wert 4, Maximalwert fr DCF Bit "High"
	retlw 0x83	;Datenpunkt 36:	Menpunkt Motor Wert 1, RA Ein/Aus
	retlw 0x8A	;Datenpunkt 37:	Menpunkt Motor Wert 2, DE Ein/Aus
	retlw 0xC3	;Datenpunkt 38:	Menpunkt Motor Wert 3, Drehrichtung RA Normal/Invertiert
	retlw 0xCA	;Datenpunkt 39:	Menpunkt Motor Wert 4, Drehrichtung DE Normal/Invertiert
	retlw 0x86	;Datenpunkt 40:	Menpunkt Parameter Auto Wert 1, Iges 100.000er
	retlw 0x87	;Datenpunkt 41:	Menpunkt Parameter Auto Wert 2, Iges 10.000er
	retlw 0x88	;Datenpunkt 42:	Menpunkt Parameter Auto Wert 3, Iges 1.000er
	retlw 0x89	;Datenpunkt 43:	Menpunkt Parameter Auto Wert 4, Iges 100er
	retlw 0x8A	;Datenpunkt 44:	Menpunkt Parameter Auto Wert 5, Iges 10er
	retlw 0x8B	;Datenpunkt 45:	Menpunkt Parameter Auto Wert 6, Iges 1er
	retlw 0xC5	;Datenpunkt 46:	Menpunkt Parameter Auto Wert 7, Nmax 100er/10er

;*******************************************************************************************
;	PAGE 2, 	bcf PCLATH,3				;Select Page 2
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1000					;PAGE 2	

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z		
	goto disp_menue_2
	movlw d'5'				;Anzeige hat 5 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_goto_set
	goto disp_menue_end
disp_menue_2
	movlw d'2'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_3	
	movlw d'5'				;Anzeige hat 5 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_pos_set
	goto disp_menue_end
disp_menue_3
	movlw d'3'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_4	
	movlw d'3'				;Anzeige hat 3 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_speed_set
	goto disp_menue_end
disp_menue_4
	movlw d'4'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_5	
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_motor_set
	goto disp_menue_end1
disp_menue_5
	movlw d'5'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_6	
	movlw d'2'				;Anzeige hat 2 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_anzeige_set
	goto disp_menue_end
disp_menue_6
	movlw d'6'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_7	
	movlw d'2'				;Anzeige hat 2 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_uz_set
	goto disp_menue_end
disp_menue_7
	movlw d'7'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_8
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_loeschen
	goto disp_menue_end1
disp_menue_8
	movlw d'8'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS, Z	
	goto disp_menue_end
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_einstell_set
disp_menue_end1
	call cursor_off			;ausschalten des cursor in den Menpunkten 4-6
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf NEU_DISP,4			;lsche das Statusbit
	return
disp_menue_end
	movlw 0xC2
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	call cursor_on
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf NEU_DISP,4			;lsche das Statusbit
	return

disp_ra
	btfss NEU1_DISP,5		;prfen, ob sich Stunden RA verndert haben
	goto disp_ra_min		;nein, weiter bei Minuten
	movlw 0x80				;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'R'				;ASCII:R
	call disp_daten			;bertrage das Zeichen
	movlw A'A'				;ASCII:A
	call disp_daten			;bertrage das Zeichen
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw STD_RA			;Daten fr Stunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,5			;Lsche das Stunden RA Update Bit
	goto $+5
disp_ra_min
	btfss NEU1_DISP,4		;prfen, ob sich Minuten RA verndert haben
	goto disp_ra_sek		;nein, weiter bei Sekunden
	movlw 0x86				;Zeichen 7 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw MIN_RA			;Daten fr Minute
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,4			;Lsche das Minuten RA Update Bit
	goto $+3
disp_ra_sek
	movlw 0x89				;Zeichen 10 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw SEK_RA			;Daten fr Sekunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bcf NEU_DISP,2			;Lsche das Sekunden RA Update Bit
	return

disp_de
	call conv_de_int_disp	;Umwandeln der internen 0-180 Registern in +/-90 fr die Anzeige
	btfss NEU1_DISP,7		;prfen, ob sich Grad DE verndert haben
	goto disp_de_min		;nein, weiter bei Minuten DE
	movlw 0xC0				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'				;ASCII:R
	call disp_daten			;bertrage das Zeichen
	movlw A'E'				;ASCII:A
	call disp_daten			;bertrage das Zeichen
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw PM_DE				;Daten fr Vorzeichen +/-
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_18			;Daten fr Grad DE
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw 0xDF				;Zeichen 
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,7			;Lsche das Grad DE Update Bit
	goto $+5
disp_de_min
	btfss NEU1_DISP,6		;prfen, ob sich Minuten DE verndert haben
	goto disp_de_min01		;nein, weiter bei 1/10 minuten
	movlw 0xC7				;Zeichen 7 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_17			;Daten fr Minuten DE
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'.'				;ASCII:"."
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,6			;Lsche das Minuten DE Update Bit
	goto $+3
disp_de_min01
	movlw 0xCA				;Zeichen 10 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_16			;Daten fr Minute 0.1er, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	bcf NEU_DISP,3			;Lsche das 1/10 Minuten DE Update Bit
	return

disp_sz
	btfss NEU1_DISP,1		;prfen, ob sich Stunden SZ verndert haben
	goto disp_sz_min		;nein, weiter bei Minuten
	movlw 0xC0				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'Z'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw STD_SZ			;Daten fr Stunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,1			;Lsche das Stunden SZ Update Bit
	goto $+5
disp_sz_min
	btfss NEU1_DISP,0		;prfen, ob sich Minuten SZ verndert haben
	goto disp_sz_sek		;nein, weiter bei Sekunden SZ
	movlw 0xC6				;Zeichen 7 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw MIN_SZ			;Daten fr Minute
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,0			;Lsche das Minuten SZ Update Bit
	goto $+3	
disp_sz_sek
	movlw 0xC9				;Zeichen 10 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw SEK_SZ			;Daten fr Sekunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bcf NEU_DISP,0			;Lsche das Sekunden SZ Update Bit
	return

disp_uz
	btfss NEU1_DISP,3		;prfen, ob sich Stunden UZ verndert haben
	goto disp_uz_min		;nein, weiter bei Minuten
	movlw 0x80				;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'				;ASCII:S
	call disp_daten			;bertrage das Zeichen
	movlw A'Z'				;ASCII:Z
	call disp_daten			;bertrage das Zeichen
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	movfw STD_UZ			;Daten fr Stunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,3			;Lsche das Stunden UZ Update Bit
	goto $+5
disp_uz_min
	btfss NEU1_DISP,2		;prfen, ob sich Minuten UZ verndert haben
	goto disp_uz_sek		;nein, weiter bei Sekunden UZ
	movlw 0x86				;Zeichen 7 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw MIN_UZ			;Daten fr Minute
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw b'00111010'		;Daten fr ":"
	call disp_daten			;bertrage das Zeichen
	bcf NEU1_DISP,2			;Lsche das Minuten UZ Update Bit
	goto $+3	
disp_uz_sek
	movlw 0x89				;Zeichen 10 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw SEK_UZ			;Daten fr Sekunde
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bcf NEU_DISP,1			;Lsche das Sekunden UZ Update Bit
	return

disp_speed
	movlw 0x8E					;Zeichen 15 in der 1. Zeile
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc SPEED,4				;prfe ob bit fr Solar nicht gesetzt ist 
	goto $+5					;wenn ja berspringe diesen Befehl
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:o
	call disp_daten		;bertrage das Zeichen
	btfsc SPEED,5				;prfe ob bit fr Solar nicht gesetzt ist 
	goto $+5					;wenn ja berspringe diesen Befehl
	movlw A'L'					;ASCII:L
	call disp_daten		;bertrage das Zeichen
	movlw A'u'					;ASCII:u
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:t
	call disp_daten		;bertrage das Zeichen
	movlw 0xCD					;Zeichen 14 in der 2. Zeile
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	btfsc SPEED,0				;prfe ob bit fr Max Speed nicht gesetzt ist 
	goto disp_sp_clearhl		;wenn ja berspringe diesen Befehl
	movlw A'M'					;ASCII:M
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A'X'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	goto disp_sp_end			;springe zum Ende des Unterprogramms
disp_sp_clearhl
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
disp_sp_end
	bcf SPEED,1					;lsche Statusbit
	return


disp_goto_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'1'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'G'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	call disp_katalog			;Aufruf des Unterprogramms zur Anzeige des zu whlenden Object Katalogs
	return

disp_pos_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'2'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'P'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A's'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'i'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'i'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	call disp_katalog			;Aufruf des Unterprogramms zur Anzeige des zu whlenden Object Katalogs
	return

disp_speed_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'3'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'p'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'd'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'L'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'u'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_motor_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'M'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_anzeige_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'5'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'z'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'i'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'g'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'P'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'Z'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return


disp_uz_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'6'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'U'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'h'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'z'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'i'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'C'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'F'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_loeschen
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'7'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'L'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A's'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'c'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'h'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_einstell_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'i'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A's'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'l'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'l'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'u'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'g'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_einstell				;Anzeige Untermen Einstellungen
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x01
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_2
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_standort_set
	goto disp_einstell_end1
disp_einstell_2
	movlw 0x02
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_3
	movlw d'2'				;Anzeige hat 2 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_para_set
	goto disp_einstell_end
disp_einstell_3
	movlw 0x03
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_4
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_dcfpara_set
	goto disp_einstell_end1
disp_einstell_4
	movlw 0x04
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_5
	movlw d'2'				;Anzeige hat 2 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_encoder_set
	goto disp_einstell_end
disp_einstell_5
	movlw 0x05
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_6
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_q1hz_set
	goto disp_einstell_end1
disp_einstell_6
	movlw 0x06
	subwf MENUE_COUNT,w
	btfss STATUS,Z
	goto disp_einstell_end
	movlw d'1'				;Anzeige hat 1 Stellen waagerecht, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call disp_handbox_set
	goto disp_einstell_end1
disp_einstell_end1
	call cursor_off			;ausschalten des cursor in den Menpunkten 4-6
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf NEU_DISP,4			;lsche das Statusbit
	return
disp_einstell_end
	movlw 0xC2
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	call cursor_on
	bsf TEMP_DIG_STAT,0		;setze das Statusbit fr Rechts/Links Vernderung des Cursor im Main Menue
	bcf NEU_DISP,4			;lsche das Statusbit
	return


disp_standort_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'1'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'd'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_para_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'2'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'P'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'm'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	call disp_aut_man
	return

disp_aut_man
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'M'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_dcfpara_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'3'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'C'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'F'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'P'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'm'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_encoder_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'c'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_handbox_set
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'8'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'6'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_goto
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'G'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'R'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'G'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	return

disp_set_pos
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'R'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	return

disp_set_motor
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	btfss TEMP_14,0				;prfen, ob Bit fr RA Motor Ein gesetzt ist
	call disp_aus				;Bit ist nicht gesetzt, Anzeige "Aus"
	btfsc TEMP_14,0				;prfen, ob Bit fr RA Motor Ein nicht gesetzt ist
	call disp_ein				;Bit ist gesetzt, Anzeige "Ein"
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	btfss TEMP_14,1				;prfen, ob Bit fr RA Motor Ein gesetzt ist
	call disp_aus				;Bit ist nicht gesetzt, Anzeige "Aus"
	btfsc TEMP_14,1				;prfen, ob Bit fr RA Motor Ein nicht gesetzt ist
	call disp_ein				;Bit ist gesetzt, Anzeige "Ein"

	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'R'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	btfss TEMP_14,2				;prfen, ob Bit fr Drehrichtung RA Motor Invertiert gesetzt ist
	call disp_normal			;Bit ist nicht gesetzt, Anzeige "Normal"
	btfsc TEMP_14,2				;prfen, ob Bit fr Drehrichtung RA Motor Invertiert nicht gesetzt ist
	call disp_invert			;Bit ist gesetzt, Anzeige "Invertiert"
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	btfss TEMP_14,3				;prfen, ob Bit fr Drehrichtung DE Motor Invertiert gesetzt ist
	call disp_normal			;Bit ist nicht gesetzt, Anzeige "Normal"
	btfsc TEMP_14,3				;prfen, ob Bit fr Drehrichtung DE Motor Invertiert nicht gesetzt ist
	call disp_invert			;Bit ist gesetzt, Anzeige "Invertiert"
	return

disp_ein
	movlw A'E'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_aus
	movlw A'A'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_normal
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_invert
	movlw A'I'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'v'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_uz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'1'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'1'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'2'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'+'					;ASCII:
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:R
	call disp_daten		;bertrage das Zeichen
	movlw A'0'					;ASCII:A
	call disp_daten		;bertrage das Zeichen
	return

disp_temp_zeit
	movlw 0x88					;Zeichen 9 in der 1. Zeile (=0x80 + 0x08)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw STD_TEMP			;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw STD_TEMP_1			;Daten fr Stunde 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	movfw MIN_TEMP			;Daten fr Minute 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw MIN_TEMP_1			;Daten fr Minute 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten		;bertrage das Zeichen
	movfw SEK_TEMP			;Daten fr Sekunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw SEK_TEMP_1			;Daten fr Sekunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	return

disp_temp_grad
	movlw 0xC8					;Zeichen 9 in der 2. Zeile (=0x80 + 0x48)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw PM_TEM				;Daten fr Stunde 10er, untere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw GRD_TEM			;Daten fr Grad 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw GRD_TEM_1				;Daten fr Grad 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movlw 0xDF					;Zeichen 
	call disp_daten		;bertrage das Zeichen
	movfw MIN_TEM			;Daten fr Minute 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw MIN_TEM_1				;Daten fr Minute 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:"."
	call disp_daten		;bertrage das Zeichen
	movfw MIN_TEM_01			;Daten fr Minute 0.1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten		;bertrage das Zeichen
	return

disp_set_standort
	movlw 0x80				;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'G'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'o'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;blank
	call disp_daten			;bertrage das Zeichen
	movlw A'L'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'a'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'n'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'g'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'e'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw 0xC2				;Zeichen 3 in der 2. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw GRD_TEM			;Daten fr Grad
	call convdisp_hex_3dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw 0xDF				;Zeichen 
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEM			;Daten fr Winkelminuten
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEM_01		;Daten fr Winkelsekunden
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw 0x20				;blank
	call disp_daten			;bertrage das Zeichen
	movlw A'W'
	btfss PM_TEM,0			;Daten fr Ost=0, West=1
	movlw A'E'
	call disp_daten			;bertrage das Zeichen
	return


disp_past_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr Stellar
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'1'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A't'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'l'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'l'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_paso_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr Solar
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'2'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'l'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_palu_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr Lunar
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'3'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'L'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'u'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_ramxde_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr RA max, DE norm, DE max
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'R'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'm'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'x'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A','					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'/'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'm'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'x'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_patra_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr Sekundenteiler RA Mikroschritt / Vollschritt
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'5'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'R'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'A'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'c'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'd'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_patde_set					;Unterprogramm zur Anzeige der berschrift im Menpunkt Parameter fr Gradteiler DE Mikroschritt / Vollschritt
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'6'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'.'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'D'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten		;bertrage das Zeichen
	movlw A'E'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'n'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'c'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'o'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'd'					;ASCII:Z
	call disp_daten		;bertrage das Zeichen
	movlw A'e'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'r'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_set_para_ra
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'R'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII
	call disp_daten				;bertrage das Zeichen
	goto disp_set_para_aut

disp_set_para_de
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw A'E'					;ASCII
	call disp_daten				;bertrage das Zeichen
	goto disp_set_para_aut

disp_set_para_aut
	movlw A':'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw A'I'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	movlw A'g'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	movlw A'='					;ASCII
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_5					;Gesamtbersetzung 100.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_4					;Gesamtbersetzung 10.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_3					;Gesamtbersetzung 1.000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_2					;Gesamtbersetzung 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_1					;Gesamtbersetzung 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw KAT_0					;Gesamtbersetzung 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen	
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'N'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	movlw A'a'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	movlw A'x'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	movlw A':'					;ASCII
	call disp_daten				;bertrage das Zeichen	
	bsf STATUS,RP0				;Select BANK_1
	movfw ZZ_UZ					;Daten fr Nmax, 100er
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw  A'0'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw  A'0'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw  A'H'					;ASCII
	call disp_daten				;bertrage das Zeichen
	movlw  A'z'					;ASCII
	call disp_daten				;bertrage das Zeichen
	return


disp_set_dcfpara
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1
	movfw DCF_L_MIN				;Minimalwert fr DCF Bit "Low"
	call convdisp_hex_3dig		;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er plus Anzeige
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1
	movfw DCF_L_MAX				;Maximalwert fr DCF Bit "Low"
	call convdisp_hex_3dig		;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er plus Anzeige
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1
	movfw DCF_H_MIN				;Minimalwert fr DCF Bit "High"
	call convdisp_hex_3dig		;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er plus Anzeige
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1
	movfw DCF_H_MAX				;Maximalwert fr DCF Bit "High"
	call convdisp_hex_3dig		;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er plus Anzeige
	return

disp_set_q1hz
	movlw 0x85				;Zeichen 5 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'p'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw A'm'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	movlw 0x8D				;Zeichen 5 in der 1. Zeile
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A's'				;ASCII:
	call disp_daten			;bertrage das Zeichen
	return

disp_set_q1hz_vz
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0			;Bank1
	movfw ZZ_UZ				;Vorzeichen der Abweichung
	call disp_daten			;bertrage das Zeichen
	return

disp_set_q1hz_ppms
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0			;Bank1
	movfw KAT_3				;1000er Wert ppm/s
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Bank1
	movfw KAT_2				;100er Wert ppm/s
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Bank1
	movfw KAT_1				;10er Wert ppm/s
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Bank1
	movfw KAT_0				;1er Wert ppm/s
	call disp_daten			;bertrage das Zeichen
	return

disp_para_dat				;Unterprogramm zur Anzeige der Werte im Menpunkt Parameter fr RA max, DE norm, DE max
	movlw 0xC0				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'A'				;ASCII:A
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII::
	call disp_daten			;bertrage das Zeichen
	movfw STD_TEMP			;Daten fr Wert A
	call convdisp_hex_3dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'B'				;ASCII:B
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII::
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEMP			;Daten fr Wert B
	call convdisp_hex_3dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'C'				;ASCII:C
	call disp_daten			;bertrage das Zeichen
	movlw A':'				;ASCII::
	call disp_daten			;bertrage das Zeichen
	movfw SEK_TEMP			;Daten fr Wert C
	call convdisp_hex_3dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	return

disp_katalog
	movlw 0xC0					;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'M'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'a'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'O'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'w'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'M'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'N'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'G'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten		;bertrage das Zeichen
	movlw A'B'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	movlw A'S'					;ASCII:S
	call disp_daten		;bertrage das Zeichen
	return

disp_kat_no					;Unterprogramm zur Anzeige der gelesenen Daten Aus dem externen EEPROM
	movlw 0x80
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_3
	bcf STATUS,RP0			;Select BANK_0
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen	
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_2
	bcf STATUS,RP0			;Select BANK_0
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen	
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_1
	bcf STATUS,RP0			;Select BANK_0
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen	
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_0
	bcf STATUS,RP0			;Select BANK_0
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen	
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	call disp_daten			;bertrage das Zeichen
	return

disp_ee						;Unterprogramm zur Anzeige der gelesenen Daten Aus dem externen EEPROM
	movlw 0x85
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_3			;Sterbild 1.Buchstabe
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_4			;Sterbild 2.Buchstabe
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_5			;Sterbild 3.Buchstabe
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_6			;
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_7			;
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_8			;
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_9			;
	btfsc STATUS,Z			;prfen, ob nicht 0
	movlw 0x20				;wenn nicht 0 berspringe diesen Befehl, wenn 0 => blank
	bcf STATUS,RP0			;Select BANK_0
	call disp_daten			;bertrage das Zeichen
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_10			;Helligkeit in mag*10
	bcf STATUS,RP0			;Select BANK_0
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13			;Daten fr 100er, untere 4 bit
	btfss STATUS,Z			;wenn 0 berspringe nchsten Befehl
	goto $+3
	movlw 0x20				;blank
	goto $+2
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_14			;Daten fr 10er, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movlw A'.'				;
	call disp_daten			;bertrage das Zeichen
	movfw TEMP_15			;Daten fr 1ser, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	movlw 0xC0				;Zeichen 1 in der 2. Zeile (=0x80 + 0x40)
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw STD_TEMP
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A':'
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEMP
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A':'
	call disp_daten			;bertrage das Zeichen
	movfw SEK_TEMP
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movfw GRD_TEM
	btfss GRD_TEM,7
	goto $+5
	movlw A'-'
	movwf PM_TEM
	call disp_daten			;bertrage das Zeichen
	goto $+4
	movlw A'+'
	movwf PM_TEM
	call disp_daten			;bertrage das Zeichen
	bcf GRD_TEM,7
	movfw GRD_TEM	
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw 0xDF				;Zeichen 
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEM	
	call convdisp_hex_2dig	;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	movlw A'.'
	call disp_daten			;bertrage das Zeichen
	movfw MIN_TEM_01	
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten			;bertrage das Zeichen
	return

disp_speich
	call clear_display
	call cursor_off
	movlw 0x82
	call disp_befehl		;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'
	call disp_daten		;bertrage das Zeichen
	movlw A'p'
	call disp_daten		;bertrage das Zeichen
	movlw A'e'
	call disp_daten		;bertrage das Zeichen
	movlw A'i'
	call disp_daten		;bertrage das Zeichen
	movlw A'c'
	call disp_daten		;bertrage das Zeichen
	movlw A'h'
	call disp_daten		;bertrage das Zeichen
	movlw A'e'
	call disp_daten		;bertrage das Zeichen
	movlw A'r'
	call disp_daten		;bertrage das Zeichen
	movlw A'n'
	call disp_daten		;bertrage das Zeichen
	movlw A'?'
	call disp_daten		;bertrage das Zeichen
	return

disp_zahl						;Unterprogramm zur Anzeige der Sekunden und der Bit Lnge beim DCF Empfang
	movlw 0x89					;Zeichen 11 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0				;Select BANK_1		
	movfw DCF_BIT				;Daten fr Sekunden-/bitzhler
	call convdisp_hex_2dig		;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er plus Anzeige
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1	
	movfw DCF_COUNT				;Hole Zhler ins W-Register
	bcf STATUS,RP0				;Select BANK_0	
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0xCC					;Zeichen 13 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	bsf STATUS,RP0				;Select BANK_1	
	movfw TBIT_H				;Daten
	bcf STATUS,RP0				;Select BANK_0	
	btfsc STATUS,Z				;prfen ob >0<
	goto $+3					;= 0, ersetze durch Blank
	addlw b'00110000'			;>0<, addiere obere 4 bit
	goto $+2					;
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	bsf STATUS,RP0				;Select BANK_1	
	movfw TBIT_L				;Daten fr "Bitlnge"
	call convdisp_hex_3dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	return



convdisp_hex_2dig				;Unterprogramms zum Auftrennen in 10er und 1er plus Anzeige
	bcf STATUS,RP0				;Select BANK_0	
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


convdisp_hex_3dig				;Unterprogramms zum Auftrennen in 100er, 10er und 1er plus Anzeige
	bcf STATUS,RP0				;Select BANK_0	
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return


;*******************************************************************************************
;	Unterprogramme Initialisierung/Ansteuerung eines OLED-Moduls EA W162-XLG ber Schieberegister 74HC164 (aktiv)
;*******************************************************************************************

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'7'					;geforderte Wartezeit des Displays von 7ms
	call wait 		
	return


cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call disp_befehl			;
	movlw d'20'					;Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl			;
	movlw d'40'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return



disp_befehl						;Unterprogramm zum Schreiben eines Befehls an das Display 
	call start_routine			;
	nop
	call send_data				;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	goto disp_bef_dat_end
disp_daten						;Unterprogramm zum Schreiben der Daten an das Display 
	call start_routine			;
	bsf DAT_TX
	call send_data				;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
disp_bef_dat_end
	movlw d'40'					;geforderte Wartezeit (min. 26,3 us)
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	return

send_data
	movlw d'8'					;es sollen 8 Bits bertragen werden
	movwf SHIFT_CNT				;Zhler fr die bertragenen Bits
	movlw d'2'
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	nop	
shift_daten
	rlf SHIFT_REG,F				;Das Zeichen wird nach links verschoben. Das Carryflag wird
									;an den Ausgang weitergegeben
	btfss STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bcf DAT_TX					;setze RC7 als Ausgang (RC7 = Datenleitung)
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	bsf DAT_TX					;setze RC7 als Eingang (RC7 = Datenleitung)
	movlw d'2'
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1						;kurze Wartezeit, um die bertragungsgeschwindigkeit zu reduzieren
	decfsz SHIFT_CNT,F			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto shift_daten			;nchstes bit bertragen
	nop
	nop
	nop
	nop
	bsf DAT_TX					;Datenleitung freigeben (High setzen) 
	bsf INTCON,GIE				;globale Interupts erlauben
	return						;Rcksprung aus dem Unterprogramm

start_routine
	bcf STATUS,RP0				;Select BANK_0
	movwf SHIFT_REG				;Die zu bertragenden Daten werden aus dem W-Register in das Reg. SHIFT_REG kopiert
	bcf INTCON,GIE				;globale Interupts verbieten
	bcf DAT_TX					;Startsignal an Slave
	movlw d'5'
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	nop	
	return


;*******************************************************************************************
;	Weitere Unterprogramme
;*******************************************************************************************

convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_13			;lsche Temp Register
	clrf TEMP_14			;lsche Temp Register
dig_100er
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 100 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_10er			;Ergebnis negativ, weiter bei 10er
	incf TEMP_13			;Ergebnis noch positiv, Erhhe 100er um 1 
	movlw d'100'			; und reduziere den zu zerlegenden Werte um 100 im F-Register
	subwf TEMP_15,f			;
	goto dig_100er			;Zurck zur Schleife
dig_10er
	movlw d'10'				;schreibe Wert 10 ins W-Register
	subwf TEMP_15,w			;reduziere den zu zerlegenden Werte um 10 im W-Register (zum Testen)
	btfss STATUS,C			;prfen,ob Ergebnis negativ
	goto dig_1er			;Ergebnis negativ, weiter bei 1er
	incf TEMP_14			;Ergebnis noch positiv, Erhhe 10er um 1 
	movlw d'10'				; und reduziere den zu zerlegenden Werte um 10 im F-Register
	subwf TEMP_15,f			;
	goto dig_10er			;Zurck zur Schleife
dig_1er
	movfw TEMP_15
	return

combine_2dig					;Unterprogramm zur Umwandlung von zB 1,2 in Zahl 12
	movwf TEMP_15				;sichern des 1er Wertes in Temp Register
	bcf STATUS, C				;Carrybit lschen, damit es beim rotieren nicht strt
	movfw TEMP_14				;hole die 10er Zahl ins W-Register und speichere sie im Rechenregister
	movwf TEMP_16				;mit 10 multiplizieren = x2x2x2+1x+1x=x10
	rlf TEMP_16,f				; x2
	rlf TEMP_16,f				; x2
	rlf TEMP_16,f				; x2
	movfw TEMP_14				;hole die 10er Zahl ins W-Register und addiere sie 2x zu dem Wert im Rechenregister
	addwf TEMP_16,f				;+1x
	addwf TEMP_16,f				;+1x, im Rechenregister steht nun der 10-fache Wert
	movfw TEMP_15				;hole die 1er Zahl ins W-Register und addiere sie zu dem Wert im Rechenregister
	addwf TEMP_16,f				;im Rechenregister steht nun der kombinierte Wert aus 10er und 1er 
	movfw TEMP_16
	return

wait
	movwf LOOP_1				;
wait_1
	movlw d'250'				;loop_10 mu 16 Takten entsprechen fr 1ms: 16MHz /4/250/16Takte
	movwf TEMP_13
loop_10
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	decfsz TEMP_13
	goto loop_10
	decfsz LOOP_1
	goto wait_1
	return

conv_de_disp_int			;Unterprogramm zum Konvertieren von DE Werten -90=>0=>+90 in 0=>180
	sublw A'+'				;subtrahiere Wert in W (ASCII: 43 = + oder 45 = -) von 43 (+)
	btfss STATUS,Z			; und prfe, ob null
	goto calc_de_conv		;nicht null, d.h. -
	movlw d'90'				;null, d.h. +, addiere 90 zum Gradwert
	addwf TEMP_18
	return
conv_de_int_disp			;Unterprogramm zum Konvertieren von DE Werten 0=>180 in -90=>0=>+90 fr die Anzeige 
	movfw GRD_DE			;kopiere interne DE Werte (0-180) in Temp16-18 Register
	movwf TEMP_18
	movfw MIN_DE
	movwf TEMP_17
	movfw MIN_DE_01
	movwf TEMP_16

	movlw d'90'				;prfen, ob Gradwert <90
	subwf TEMP_18,w
	btfss STATUS,C
	goto $+6				;Gradwert <90, weiter bei Umrechnung auf negative Werte
	movlw d'90'				;Gradwert >=90, reduziere Gradwert um 90 und setze Vorzeichen auf +
	subwf TEMP_18,f
	movlw A'+'
	movwf PM_DE
	return
	movlw A'-'				;Gradwert <90, setze Vorzeichen auf -
	movwf PM_DE
calc_de_conv
	movfw TEMP_16			;prfen, ob 10tel Minuten =0
	btfsc STATUS,Z
	goto $+0x0A				;10tel Minuten =0, weiter bei Minuten
	sublw d'10'				;10tel Minuten >0<, 10 minus 10tel Minuten
	movwf TEMP_16			;
	incf TEMP_17,f			;erhhe Minuten um 1
	movlw d'60'				;Minuten minus 60
	subwf TEMP_17,w
	btfss STATUS,Z			;prfen ob 0
	goto $+3				;Minuten >0<
	movwf TEMP_17
	incf TEMP_18,f			;negativ, erhhe Grad um 1

	movfw TEMP_17			;prfen, ob Minuten =0
	btfsc STATUS,Z
	goto $+4
	sublw d'60'				;Minuten >0<, 60 minus Minuten
	movwf TEMP_17			;
	incf TEMP_18,f			;erhhe Stunden um 1

	movfw TEMP_18
	sublw d'90'				;Minuten >0<, 60 minus Minuten
	movwf TEMP_18		

	return					;fertig




;*******************************************************************************************
;	Unterprogramme zum Lesen/Schreiben eines externen EEPROMS ber den I2C Bus
;*******************************************************************************************


Lese_EEPROM
	call i2c_on				;Bus aktiv 
	bsf STATUS,RP0			;Select BANK_1
	bcf ADR_DEV,0			;Schreiben: bit 0 =0
	movfw ADR_DEV			;1010 ***0, *** =Bus Adresse
	call i2c_tx				;24C*** zum Schreiben adressieren 

	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_H				;high Teil der Adresse 
	call i2c_tx 
	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_L				;low Teil der  Adresse 
	call i2c_tx 

	call i2c_repeat			; Wiederholte Startbedingung ausgeben 

 	bsf STATUS,RP0			;Select BANK_1
	bsf ADR_DEV,0			;Lesen: bit 0 =1
	movfw ADR_DEV			;1010 ***1, *** =Bus Adresse
	call i2c_tx				;24C*** zum Lesen adressieren 

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_1			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

  	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_2			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_no_ack

	call i2c_off			; Bus freigeben
	return


Lese_EEPROM_1
	call i2c_on				;Bus aktiv 
	bsf STATUS,RP0			;Select BANK_1
	bcf ADR_DEV,0			;Schreiben: bit 0 =0
	movfw ADR_DEV			;1010 ***0, *** =Bus Adresse
	call i2c_tx				;24C*** zum Schreiben adressieren 

	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_H				;high Teil der Adresse 
	call i2c_tx 
	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_L				;low Teil der  Adresse 
	call i2c_tx 

	call i2c_repeat			; Wiederholte Startbedingung ausgeben 

 	bsf STATUS,RP0			;Select BANK_1
	bsf ADR_DEV,0			;Lesen: bit 0 =1
	movfw ADR_DEV			;1010 ***1, *** =Bus Adresse
	call i2c_tx				;24C*** zum Lesen adressieren 

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_1			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_2			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_3			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_4			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_5			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_6			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_7			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_8			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_9			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_10			;Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf STD_TEMP			; Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf MIN_TEMP			; Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf SEK_TEMP			; Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf GRD_TEM			; Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf MIN_TEM			; Byte in Speicherzelle Datenpuffer retten 
	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			; I2C Empfangsregister auslesen 
	movwf MIN_TEM_01		; Byte in Speicherzelle Datenpuffer retten 
	call i2c_no_ack

	call i2c_off			; Bus freigeben
	return

Lese_EEPROM_0
	call i2c_on				;Bus aktiv 
	bsf STATUS,RP0			;Select BANK_1
	bcf ADR_DEV,0			;Schreiben: bit 0 =0
	movfw ADR_DEV			;1010 ***0, *** =Bus Adresse
	call i2c_tx				;24C*** zum Schreiben adressieren 

	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_H				;high Teil der Adresse 
	call i2c_tx 
	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_L				;low Teil der  Adresse 
	addlw 0x02
	call i2c_tx 

	call i2c_repeat			; Wiederholte Startbedingung ausgeben 

 	bsf STATUS,RP0			;Select BANK_1
	bsf ADR_DEV,0			;Lesen: bit 0 =1
	movfw ADR_DEV			;1010 ***1, *** =Bus Adresse
	call i2c_tx				;24C*** zum Lesen adressieren 

	movlw d'14'				;es sollen 14 Bytes auf Wert 0 geprft werden
	movwf TEMP_13
	goto $+2

	call i2c_ack

 	call i2c_rx 
	movfw SSPBUF			;I2C Empfangsregister auslesen 
	btfss STATUS,Z			;Prfen ob Wert = 0
	goto $+4				;Wert >0<
	decfsz TEMP_13			;reduziere Zhler und prfe ob 0
	goto $-6				;Zhler noch nicht 0, zurck zur Schleife
	goto $+2				;Zhler ist 0
	bsf TEMP_13,0			;Setze Bit fr Wert >0<
	call i2c_no_ack			;I2C-Bus No Acknowledge Sequenz ausgeben

	call i2c_off			;Bus freigeben
	return					;Rcksprung aus dem Unterprogramm


Schreibe_EEPROM
	call i2c_on				;Bus aktiv 
	bsf STATUS,RP0			;Select BANK_1
	bcf ADR_DEV,0			;Schreiben: bit 0 =0
	movfw ADR_DEV			;1010 ***0, *** =Bus Adresse
	call i2c_tx				;24C*** zum Schreiben adressieren 

	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_H				;high Teil der Adresse 
	call i2c_tx 
	bsf STATUS,RP0			;Select BANK_1
	movfw ADR_L				;low Teil der  Adresse 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_1			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_2			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_3			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_4			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_5			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_6			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_7			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_8			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_9			;Wert der gespeichert werden soll 
	call i2c_tx 

	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_10			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw STD_TEMP			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw MIN_TEMP			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw SEK_TEMP			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw GRD_TEM			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw MIN_TEM			;Wert der gespeichert werden soll 
	call i2c_tx 

	movfw MIN_TEM_01		;Wert der gespeichert werden soll 
	call i2c_tx 

	call i2c_off			; Bus freigeben 
	return


;*******************************************************************************************
;	Unterprogramme I2C
;*******************************************************************************************
; I2C-Bus im Master-Mode bernehmen 
i2c_on
	bsf SSPCON,5				;Enable interne serielle Schnittstelle bit 5 = SSPEN
	bcf PIR1,SSPIF				; SSPIF Bit lschen 
 	bsf STATUS,RP0 
	bsf SSPCON2,SEN				; Bus bernahme anweisen 
	bcf	STATUS,RP0 
	goto i2c_warte 

; Wiederholte Startbedingung ausgeben
i2c_repeat
	bcf PIR1, SSPIF				; SSPIF Bit lschen 
 	bsf STATUS, RP0 
	bsf SSPCON2, RSEN			; Bus bernahme anweisen 
	bcf	STATUS, RP0 
	goto i2c_warte 

; ein Byte aus W senden 
i2c_tx 
	bcf STATUS,RP0				;Select BANK_0
	movwf SSPBUF				; -> zum I2C-Slave bertragen 
	goto i2c_warte 

;ein Byte vom Slave empfangen (nach SSPBUF) 
i2c_rx 
	bsf STATUS,RP0 
	bsf SSPCON2,RCEN			; Daten Empfang einschalten 
	bcf STATUS,RP0 
	goto i2c_warte 

; I2C-Bus Acknowledge Sequenz ausgeben (Lese Modus)
i2c_ack
	bcf STATUS,RP0				;Select BANK_0
	bcf PIR1,SSPIF				;SSPIF Bit lschen 
	bsf STATUS,RP0				;Select BANK_1
	bcf SSPCON2,ACKDT			;Acknowledge bit auf 0 setzen (Acknowledge)
	bsf SSPCON2,ACKEN			;Acknowledge Sequenz ausgeben (ACKDT wird ausgegeben)
	bcf STATUS,RP0				;Select BANK_0
	goto i2c_warte 

; I2C-Bus No Acknowledge Sequenz ausgeben (Lese Modus)
i2c_no_ack
	bcf STATUS,RP0				;Select BANK_0
	bcf PIR1,SSPIF				;SSPIF Bit lschen 
	bsf STATUS,RP0				;Select BANK_1
	bsf SSPCON2,ACKDT			;Acknowledge bit auf 1 setzen (not Acknowledge)
	bsf SSPCON2,ACKEN			;Acknowledge Sequenz ausgeben (ACKDT wird ausgegeben)
	bcf STATUS,RP0				;Select BANK_0
	goto i2c_warte 

; I2C-Bus wieder freigeben 
i2c_off 
	bsf STATUS, RP0 
	bsf SSPCON2, PEN			; Bus Freigabe anweisen 
	bcf STATUS, RP0 

i2c_warte 
	btfss PIR1,SSPIF			; fertig? 
	goto i2c_warte				; nein, noch nicht 
	bcf PIR1,SSPIF				; ja, alles fertig, nun noch SSPIF zurcksetzen 
	return 






;*******************************************************************************************
;	PAGE 3, 	bsf PCLATH,3				;Select Page 3
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1800					;PAGE 3

update_uhr					;Programmteil zur Auswertung der DCF Signale
	bcf OUT_RA_ENABLE		;Motoren abschalten, um DCF Signal nicht zu stren
	bcf OUT_DE_ENABLE
	call lese_dcfpara		;lesen der Min/Max Werte fr "Bitlnge" aus dem internen EEPROM
	bcf PCLATH,3			;Select Page 2
	call clear_display
	call disp_uz			;Anzeige der Uhrzeit
	movlw d'250'			;Wartezeit von 250ms
	call wait
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	clrf DCF_BIT			;Lsche Register fr Anzahl ausgewerteter Bits
	clrf DCF_COUNT 
startbit_prfen
	call dcf_bit			;Aufruf des Unterprogramms zur Ermittlung der "Bitlnge" (=> 0/1)
	movlw d'60'
	subwf DCF_BIT,w
	btfsc STATUS,C
	goto fehler
	btfss DCF_MESS_STAT,2	;prfe Bit fr Beginn neue Minute
	goto startbit_prfen
	clrf DCF_BIT			;
	clrf DCF_PRUEFBIT
;dcfbit0					;Startbit: ist immer 0
	call dcf_bit
	btfsc DCF_MESS_STAT,1	;prfe Fehler Bit
	goto fehler
	btfsc DCF_MESS_STAT,0	;Prfe gltiges Bit, Startbit mu 0 sein
	goto fehler
	movlw d'16'
	movwf ZAHL_C0
dcfbit1_16					;Allgemeine Infos, werden nicht ausgewertet.
	call dcf_bit
	btfsc DCF_MESS_STAT,1	;prfe Fehler Bit
	goto fehler
	decfsz ZAHL_C0			;
	goto dcfbit1_16
	movlw d'2'
	movwf ZAHL_C0
	clrf DATEN_1			;DATEN_1 wird fr MEZ/MESZ verwendet
dcfbit17_18					;bit 17+18: MEZ/MESZ
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	rlf DATEN_1
	btfss DCF_MESS_STAT,0
	bcf DATEN_1,0
	btfsc DCF_MESS_STAT,0
	bsf DATEN_1,0
	decfsz ZAHL_C0			;
	goto dcfbit17_18
;dcfbit19					;bit 19: Schaltsekunde in den nchsten 59min, wird nicht ausgewertet
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
;dcfbit20					;Startbit fr Zeitinformation, immer 1
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_MESS_STAT,0
	goto fehler
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_2			;DATEN_2 wird fr Minuten 1er verwendet
dcfbit21_24					;Minuten Einer
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_2
	btfss DCF_MESS_STAT,0
	bcf DATEN_2,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_2,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit21_24
	movlw d'3'
	movwf ZAHL_C0
	clrf DATEN_3			;DATEN_3 wird fr Minuten 10er verwendet
dcfbit25_27					;Minuten Zehner
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_3
	btfss DCF_MESS_STAT,0
	bcf DATEN_3,2
	btfsc DCF_MESS_STAT,0
	bsf DATEN_3,2
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit25_27
	movfw DATEN_3			;Kombinieren des 10er und 1er Wertes in eine Zahl
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_14			;
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_2			;
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	call combine_2dig		;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_2			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
;dcfbit28					;Prfbit
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_3			;DATEN_3 wird fr Stunden 1er verwendet
dcfbit29_32					;Stunden 1er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_3
	btfss DCF_MESS_STAT,0
	bcf DATEN_3,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_3,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit29_32
	movlw d'2'
	movwf ZAHL_C0
	clrf DATEN_4			;DATEN_4 wird fr Stunden 10er verwendet
dcfbit33_34					;Stunden 10er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_4
	btfss DCF_MESS_STAT,0
	bcf DATEN_4,1
	btfsc DCF_MESS_STAT,0
	bsf DATEN_4,1
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit33_34
	movfw DATEN_4			;Kombinieren des 10er und 1er Wertes in eine Zahl
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_14			;
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_3			;
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	call combine_2dig		;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_3			;Rcksichern des kombinierten Wertes im DCF77 Stunden Register
;dcfbit35					;Prfbit
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_4			;DATEN_4 wird fr Tage 1er verwendet
dcfbit36_39					;Tag 1er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_4
	btfss DCF_MESS_STAT,0
	bcf DATEN_4,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_4,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit36_39
	movlw d'2'
	movwf ZAHL_C0
	clrf DATEN_5			;DATEN_5 wird fr Tage 10er verwendet
dcfbit40_41					;Tag 10er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_5
	btfss DCF_MESS_STAT,0
	bcf DATEN_5,1
	btfsc DCF_MESS_STAT,0
	bsf DATEN_5,1
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit40_41
	movfw DATEN_5			;Kombinieren des 10er und 1er Wertes in eine Zahl
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_14			;
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_4			;
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	call combine_2dig		;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_4			;Rcksichern des kombinierten Wertes im DCF77 Tage Register
	movlw d'3'
	movwf ZAHL_C0
	clrf DATEN_5			;DATEN_5 wird fr Wochentag verwendet
dcfbit42_44					;Wochentag
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_5
	btfss DCF_MESS_STAT,0
	bcf DATEN_5,2
	btfsc DCF_MESS_STAT,0
	bsf DATEN_5,2
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit42_44
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_6			;DATEN_6 wird fr Monat 1er verwendet
dcfbit45_48					;Monat 1er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_6
	btfss DCF_MESS_STAT,0
	bcf DATEN_6,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_6,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit45_48
;dcfbit49					;Monat 10er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DATEN_7			;DATEN_7 wird fr Monat 10er verwendet
	btfss DCF_MESS_STAT,0
	bcf DATEN_7,0
	btfsc DCF_MESS_STAT,0
	bsf DATEN_7,0
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	movfw DATEN_7			;Kombinieren des 10er und 1er Wertes in eine Zahl
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_14			;
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_6			;
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	call combine_2dig		;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_6			;Rcksichern des kombinierten Wertes im DCF77 Monats Register
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_7			;DATEN_7 wird fr Jahr 1er verwendet
dcfbit50_53					;Jahr 1er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_7
	btfss DCF_MESS_STAT,0
	bcf DATEN_7,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_7,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit50_53
	movlw d'4'
	movwf ZAHL_C0
	clrf DATEN_8			;DATEN_8 wird fr Jahr 10er verwendet
dcfbit54_57					;Jahr 10er
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf DATEN_8
	btfss DCF_MESS_STAT,0
	bcf DATEN_8,3
	btfsc DCF_MESS_STAT,0
	bsf DATEN_8,3
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	decfsz ZAHL_C0			;
	goto dcfbit54_57
	movfw DATEN_8			;Kombinieren des 10er und 1er Wertes in eine Zahl
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_14			;
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_7			;
	bcf STATUS,RP0			;Select BANK_0
	bcf PCLATH,3			;Select Page 2
	call combine_2dig		;Aufruf des Unterprogramms
	bsf PCLATH,3			;Select Page 3
	bsf STATUS,RP0			;Select BANK_1
	movwf DATEN_7			;Rcksichern des kombinierten Wertes im DCF77 Jahr Register
;dcfbit58					;Prfbit
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_MESS_STAT,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
;dcfbit59					;keine Absenkung, mit der nchsten negativen Flanke beginnt die neue Minute
	call dcf_bit
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_MESS_STAT,2	;prfen ob Bit fr neuen Datensatz gesetzt ist
	goto fehler				;Bit ist nicht gesetzt, Fehler
	bcf STATUS,RP0			;Select BANK_0		
	btfsc IN_DCF			;warten auf die negative Flanke des DCF signals
	goto $-1
	bcf STATUS,RP0			;Select BANK_0	
	clrf TMR0
	movlw 0x1F				;Empfang des DCF Signals war Fehlerfrei, Laden der Konstanten fr den Uhrzeittakt
	movwf COUNT_H_UZ		;
	movlw 0x84
	movwf COUNT_L_UZ
	movlw 0x02				;Setzen des Zhlers, Digit:= 2	
	movwf DIGIT_UZ			;
	clrf SEK_UZ
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_1			;Kopieren der Temporren Register in die aktiven Register
	movwf ZZ_UZ				;Zeitzone
	movfw DATEN_2			;Minute
	bcf STATUS,RP0			;Select BANK_0		
	movwf MIN_UZ
	bsf STATUS,RP0			;Select BANK_1	
	movfw DATEN_3			;Stunde
	bcf STATUS,RP0			;Select BANK_0	
	movwf STD_UZ
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_4			;Tag
	movwf TAG_UZ
	movfw DATEN_5			;Wochentag
	movwf WTAG_UZ
	movfw DATEN_6			;Monat
	movwf MONAT_UZ
	movfw DATEN_7			;Jahr
	movwf JAHR_UZ
	clrf DATEN_5
	call sternzeit			;Aufruf des Unterprogramms zur Berechnung der lokalen Sternzeit
update_uhr_end
	bcf STATUS,RP0			;Select BANK_0	
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	bcf NEU_DISP,5			;Lsche Statusbit (=>Anzeige UZ/SZ)
	btfsc STAT_MOTOR,0		;wenn Statusbits fr Motoren "Ein" gesetzt sind, Ausgnge setzen
	bsf OUT_RA_ENABLE
	btfsc STAT_MOTOR,1
	bsf OUT_DE_ENABLE	
	goto menue_end	

fehler
	bcf PCLATH,3			;Select Page 2	
	bcf STATUS,RP0			;Select BANK_0		
	call disp_zahl
	movlw d'250'
	call wait
	bsf STATUS,RP0			;Select BANK_1	
	bsf PCLATH,3			;Select Page 3
	clrf DCF_BIT			;
	clrf DCF_PRUEFBIT
	incf DCF_COUNT			;nach 5 Fehlversuchen wird die Update Routine beendet
	movlw d'5'
	subwf DCF_COUNT,w
	btfss STATUS,Z
	goto startbit_prfen	;noch keine 5 Fehlversuche, neuer Start
	bcf STATUS,RP0			;Select BANK_0	
	bcf PCLATH,3			;Select Page 0
	bcf PCLATH,4			;
	btfsc STAT_MOTOR,0		;wenn Statusbits fr Motoren "Ein" gesetzt sind, Ausgnge setzen
	bsf OUT_RA_ENABLE
	btfsc STAT_MOTOR,1
	bsf OUT_DE_ENABLE	
	goto menue_end	

dcf_bit
	bsf STATUS,RP0			;Select BANK_1	
	clrf DCF_MESS_STAT
	clrf TBIT_L
	clrf TBIT_H
	bcf STATUS,RP0			;Select BANK_0	
	bcf NEU_DISP,7
dcf_loop_n					;warten auf die negative Flanke 
	bcf STATUS,RP0			;Select BANK_0	
	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto update_uhr_end		;Slave will Befehl senden => springe zum Ende des Programms
	btfss IN_DCF
	goto dcf_bit1			;weiter bei positiver Flanke
	btfss NEU_DISP,7		;prfen des Timer0 berlaufs
	goto dcf_loop_n
	bcf NEU_DISP,7			;lschen des berlaufbits
	bsf STATUS,RP0			;Select BANK_1	
	incfsz TBIT_L			;erhhe 1,024ms Zhler fr Low byte und prfe ob 0
	goto dcf_loop_n
	incf TBIT_H				;erhhe 262ms Zhler
	movlw d'40'				;prfen ob zhler 40 erreicht ist (entspricht 1,31sek)
	subwf TBIT_H,w
	btfss STATUS,Z
	goto dcf_loop_n	
	bsf DCF_MESS_STAT,2		;wenn ja setze statusbit fr neues Datentelegramm (Bit59)
	return
dcf_bit1
	bsf STATUS,RP0			;Select BANK_1	
	clrf TBIT_L
	clrf TBIT_H
dcf_loop_p					;warten auf die positive Flanke 
	bcf STATUS,RP0			;Select BANK_0
	btfss DAT_RX			;prfen, ob Slave keine Daten senden will
	goto update_uhr_end		;Slave will Befehl senden => springe zum Ende des Programms
	btfsc IN_DCF
	goto dcf_bit_wert		;Auswertung des Signals
	btfss NEU_DISP,7		;prfen des Timer0 berlaufs
	goto dcf_loop_p
	bcf NEU_DISP,7
	bsf STATUS,RP0			;Select BANK_1	
	incfsz TBIT_L			;erhhe 1,024ms Zhler fr Low byte und prfe ob 0
	goto dcf_loop_p
	incf TBIT_H				;erhhe 262ms Zhler
;wenn nach 0,5sek keine positive Flanke ermittelt wurde, wird das Fehlerbit gesetzt, um die Warteschleife abzubrechen
	movlw d'15'				;prfen ob zhler 15 erreicht ist (entspricht 0,5sek)
	subwf TBIT_H,w
	btfss STATUS,Z
	goto dcf_loop_p

dcf_bit_wert				;Auswertung der Zeit
	bsf STATUS,RP0			;Select BANK_1	
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf TBIT_H				;TBIT_H/L durch 8 teilen (3x nach rechts verschieben)
	rrf TBIT_L
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf TBIT_H
	rrf TBIT_L
	bcf STATUS,C			;Carry Bit lschen, damit es beim rotieren nicht strt
	rrf TBIT_H
	rrf TBIT_L
	movfw DCF_H_MAX			;prfen ob TBIT_L > DCF_H_MAX, wenn ja setze fehlerbit
	subwf TBIT_L,w
	btfsc STATUS,C
	bsf DCF_MESS_STAT,1		;setze Fehlerbit
	movfw DCF_H_MIN			;prfen ob TBIT_L < DCF_H_MIN, wenn nein setze bit fr "1", 
	subwf TBIT_L,w	
	btfss STATUS,C
	goto $+3
	bsf DCF_MESS_STAT,0
	goto auswertbit_end		;Bit ist gltig und "1"
	movfw DCF_L_MAX			;prfen ob TBIT_L > DCF_L_MAX, wenn ja setze fehlerbit
	subwf TBIT_L,w
	btfsc STATUS,C
	bsf DCF_MESS_STAT,1		;setze Fehlerbit
	movfw DCF_L_MIN			;prfen ob TBIT_L < DCF_L_MIN, wenn nein lsche bit fr "0", 
	subwf TBIT_L,w
	btfss STATUS,C
	bsf DCF_MESS_STAT,1		;setze Fehlerbit
	bcf DCF_MESS_STAT,0
	goto auswertbit_end		;Bit ist gltig und "0"
auswertbit_end
	bcf PCLATH,3			;Select Page 2	
	bcf STATUS,RP0			;Select BANK_0		
	call disp_zahl
	bsf STATUS,RP0			;Select BANK_1	
	bsf PCLATH,3			;Select Page 3
	incf DCF_BIT
	movlw d'60'
	subwf DCF_BIT,w
	btfsc STATUS,C
	bsf DCF_MESS_STAT,1		;setze Fehlerbit	
	bsf STATUS,RP0			;Select BANK_1		
	return


;*******************************************************************************************
;	Unterprogramme zur Berechnung der lokalen Sternzeit aus Uhrzeit (UTC), Datum, und Position (Lnge O/W)
;*******************************************************************************************


sternzeit
	;fr die Berechnung werden die folgenden temporren Register verwendet:
	;DATEN_1				Zeitzone
	;DATEN_2				Minute
	;DATEN_3				Stunde
	;DATEN_4				Tag
	;DATEN_5				Sekunde
	;DATEN_6				Monat
	;DATEN_7				Jahr
	;DATEN_8				Jahreszahl im Schaltjahrrythmus, Schaltjahr =0, SJ+1 =1, SJ+2 =2, SJ+3 =3
	;ADR_L					Anzahl abgelaufener Tage, Low Byte
	;ADR_H					Anzahl abgelaufener Tage, High Byte
	;KAT_3					Sternzeit in Hundertstel Sekunden, Byte 3
	;KAT_2					Sternzeit in Hundertstel Sekunden, Byte 2
	;KAT_1					Sternzeit in Hundertstel Sekunden, Byte 1
	;KAT_0					Sternzeit in Hundertstel Sekunden, Byte 0
	
;Testbereich Anfang
	;bsf STATUS,RP0			;Select BANK_1	
	;movlw b'00000010'		;laden der Werte in Register
	;movwf DATEN_1			;Zeitzone
	;movlw d'18'				;laden der Werte in Register
	;movwf DATEN_7			;Jahr
	;movlw d'10'				;laden der Werte in Register
	;movwf DATEN_6			;Monat
	;movlw d'01'				;laden der Werte in Register
	;movwf DATEN_4			;Tag
	;movlw d'23'				;laden der Werte in Register
	;movwf DATEN_3			;Stunde
	;movlw d'05'				;laden der Werte in Register
	;movwf DATEN_2			;Minute
	;movlw d'00'				;laden der Werte in Register
	;movwf DATEN_5			;Sekunde
;Testbereich Ende
;Korrektur der lokalen Zeit auf UTC
	bsf STATUS,RP0		;Select BANK_1
	btfsc DATEN_1,7		;prfen ob Wert fr Zeitzone negativ ist
	goto korr_negativ	;
	movfw DATEN_1		;hole die aktuelle Stunde ins W-Register
	subwf DATEN_3,f		;subtrahiere die Stunden versus UTC (MEZ=1,MESZ=2)
	btfsc STATUS,C		;prfe ob Ergebniss negativ
	goto copy_base		;wenn nein, weiter bei copy_base, wenn ja, addiere 24 zu DATEN_3 und reduziere die Tage um 1
	movlw d'24'
	addwf DATEN_3,f		;
	decfsz DATEN_4		;reduziere Tag um 1 und prfe ob 0
	goto copy_base
	decfsz DATEN_6		;reduziere Monat um 1 und prfe ob 0
	goto $+4			;wenn nicht 0 weiter bei Rcksetzen des Tages
	movlw d'12'
	movwf DATEN_6
	decf DATEN_7		;wenn 0 reduziere Jahr um 1

;Ermittlung des Tages, der nach Reduzierung des Monats ins Tagesregister zurckgeschrieben wird
	call monats_tage	;
	movwf DATEN_4
	goto copy_base

korr_negativ
	bcf DATEN_1,7		;lsche Vorzeichen Bit 7
	movfw DATEN_1		;hole die aktuelle Stunde ins W-Register
	addwf DATEN_3,f		;addiere die Stunden versus UTC
	movlw d'24'
	subwf DATEN_3,w		;
	btfss STATUS,C		;prfe ob Ergebniss >23
	goto copy_base		;wenn nein, weiter bei copy_base, wenn ja, subtrahiere 24 zu DATEN_3 und erhhe die Tage um 1
	movwf DATEN_3
	call monats_tage	;hole die maximalen Tage des Monats ins W-Register
	subwf DATEN_4,w		;subtrahiere den aktuellen Tag vom Maximalwert
	btfsc STATUS,Z		;prfe ob Maximalwert schon erreicht
	goto $+3			;Maximalwert ist erreicht => rcksetzen des Tags auf 1 und erhhen des Monats
	incf DATEN_4		;Maximalwert ist noch nicht erreicht => erhhe Tag um 1
	goto copy_base
	movlw d'1'			;Maximalwert ist erreicht => rcksetzen des Tags auf 1
	movwf DATEN_4
	incf DATEN_6		;erhhen des Monats um 1
	movlw d'13'			;prfen ob nach Erhhung der Monat = 13
	subwf DATEN_6,w
	btfss STATUS,Z		;
	goto copy_base		;Wert des Monats ist nicht 13 => weiter bei copy_base
	movlw d'1'			;Wert des Monats ist 13 => Rcksetzen des Monats auf 1 und Erhhung des Jahres
	movwf DATEN_6
	incf DATEN_7

copy_base
	movlw 0xFB			;lade Sternzeit fr UTC 01.01.2000 00:00:00 = 06:39:52,27 
	movwf KAT_0			;	= 2.399.227 hundertstel Sek = 0x00249BFB (hex)
	movlw 0x9B
	movwf KAT_1			;
	movlw 0x24
	movwf KAT_2			;
	clrf KAT_3			;
;ermittlung der Jahreszahl im Schaltjahrrythmus, Schaltjahr =0, SJ+1 =1, SJ+2 =2, SJ+3 =3
	clrf DATEN_8		;lsche Register
	btfsc DATEN_7,0		;prfen ob bit 0=1
	bsf DATEN_8,0		;setze bit0
	btfsc DATEN_7,1		;prfen ob bit 0=1
	bsf DATEN_8,1		;setze bit0
;Korrektur mit 1,85sek/Jahr seit Jahr 2000 = 185hundertstel sek = Jahre *185
	movfw DATEN_7
	movwf ZAHL_A0		;1.Multiplikant
	clrf ZAHL_A1		;
	clrf ZAHL_A2		;
	clrf ZAHL_A3		;
	movlw d'185'
	movwf ZAHL_B0		;2.Multiplikant
	clrf ZAHL_B1		;
	clrf ZAHL_B2		;
	clrf ZAHL_B3		;
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0 und wird addiert
;Addition der Korektur zu Sternzeitregistern : ZAHL_C + ZAHL_B = ZAHL_C
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register
	call add32			;Unterprogramm zur Addition, Ergebnis steht in Registern ZAHL_C7:C0

;Korrektur fr Schaltjahr (SJ) = 0, SJ+1 = 2min57,41sek, SJ+2 = 1min58,27sek, SJ+3 = 0min59,13sek
	clrf ZAHL_B2		;Lsche Additionsregister
	clrf ZAHL_B3		;
	movlw 0x01
	subwf DATEN_8,w
	btfss STATUS,Z
	goto year_02		;nicht SJ+1, weiter bei nchster Prfung
	movlw 0x4D			;SJ+1: addiere 2min57,41sek = 17.741 Hundertstel Sek = 0x0000454D (hex)
	movwf ZAHL_B0		;	Sternzeit steht noch in Registern ZAHL_C7:C0
	movlw 0x45			;
	movwf ZAHL_B1
	call add32			;Addition: ZAHL_C + ZAHL_B = ZAHL_C
	goto year_end
year_02
	movlw 0x02
	subwf DATEN_8,w
	btfss STATUS,Z
	goto year_03		;nicht SJ+2, weiter bei nchster Prfung
	movlw 0x33			;SJ+2: addiere 1min58,27sek = 11.827 Hundertstel Sek = 0x00002E33 (hex)
	movwf ZAHL_B0		;	Sternzeit steht noch in Registern ZAHL_C7:C0
	movlw 0x2E			;
	movwf ZAHL_B1
	call add32			;Addition: ZAHL_C + ZAHL_B = ZAHL_C
	goto year_end
year_03
	movlw 0x03
	subwf DATEN_8,w
	btfss STATUS,Z
	goto year_end		;nicht SJ+3, d.h. SJ => keine Korrektur
	movlw 0x19			;SJ+3: addiere 0min59,13sek = 5.913 Hundertstel Sek = 0x00001719 (hex)
	movwf ZAHL_B0		;	Sternzeit steht noch in Registern ZAHL_C7:C0
	movlw 0x17			;
	movwf ZAHL_B1
	call add32			;Addition: ZAHL_C + ZAHL_B = ZAHL_C
year_end
	call copy_zc_kat	;Rcksichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0

;ermittlung der abgelaufenen Tage im laufenden Jahr
	clrf ADR_H			;lsche Register, werden fr Anzahl volle Tage des laufenden Jahres verwendet
	clrf ADR_L
	movfw DATEN_6		;Monat
	movwf ZAHL_A0
	decfsz ZAHL_A0		;reduziere Monat und prfe ob 0
	goto month_01		;
	goto add_tag
month_01
	movlw 0x1F			;Programmzhler fuer Tagestabelle im Bereich 0x1FF0..0x1FFF 
	movwf PCLATH		;
	movfw ZAHL_A0		;Hole Monat nach W
	call tab_tage		;Aufruf der Tagestabelle mit Tagen des Monats in W
	addwf ADR_L,f
	btfsc STATUS,C
	incf ADR_H
	decfsz ZAHL_A0
	goto month_01
month_end				;bei Schaltjahr und Monate Mrz bis Dezember mu 1 Tag addiert werden
	movfw DATEN_8
	btfss STATUS,Z
	goto add_tag
	movlw d'3'
	subwf DATEN_6,w
	btfsc STATUS,C
	incf ADR_L,f
add_tag					;zu den Tagen der vollen Monate werden noch die Tage des aktuellen Monats addiert
	decf DATEN_4,w
	addwf ADR_L,f
	btfsc STATUS,C
	incf ADR_H,f

;Ermittlung der Verschiebung aus den abgelaufenen vollen Tagen des aktuellen Jahres: = Tage *236,556sek
	movfw ADR_L			;Tage Low Byte
	movwf ZAHL_A0		;1.Multiplikant
	movfw ADR_H			;Tage High Byte
	movwf ZAHL_A1		;
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x0C			;236,556sek = 236.556msek => 0x00039C0C (hex)
	movwf ZAHL_B0		;2.Multiplikant
	movlw 0x9C
	movwf ZAHL_B1		;
	movlw 0x03
	movwf ZAHL_B2		;
	clrf ZAHL_B3		;
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
;Ergebnis von msek auf Hundertstel Sek umrechnen, d.h. durch 10 teilen. msek stehen noch in  
	movlw d'10'			;Registern ZAHL_C7:C0
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32			;Unterprogramm zur Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest in ZAHL_B
;Ergebnis zu Sternzeit addieren, Ergebnis steht noch in Registern ZAHL_C7:C0
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_kat	;Rcksichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0

;Ermittlung der Verschiebung aus den abgelaufenen vollen Stunden des aktuellen Tages: = Stunden *9,85sek
	movfw DATEN_3		;kopiere die vollen Stunden des aktuellen Tages in 1.Multiplikationsregister
	movwf ZAHL_A0		;volle Stunden werden in 100stel Sek umgerechnet und mit 985  Hundertstel Sek korr.
	clrf ZAHL_A1		; 1h = 360.000 Hundertstel Sek
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x19			;360000 + 985 = 360.985 => 0x00058219
	movwf ZAHL_B0
	movlw 0x82			;
	movwf ZAHL_B1
	movlw 0x05			;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
;Ergebnis zu Sternzeit addieren, Ergebnis steht noch in Registern ZAHL_C7:C0
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_kat	;Rcksichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0

;Ermittlung der Verschiebung aus den abgelaufenen vollen Minuten des aktuellen Tages: = Minuten *0,16sek
	movfw DATEN_2		;kopiere die vollen Minuten der aktuellen Stunde in 1.Multiplikationsregister
	movwf ZAHL_A0		;volle Minuten werden in 100stel Sek umgerechnet und mit 16  Hundertstel Sek korr.
	clrf ZAHL_A1		; 1min = 6.000 Hundertstel Sek
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x80			;6.000 + 16 = 6.016 => 0x00001780
	movwf ZAHL_B0
	movlw 0x17			;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
;Ergebnis zu Sternzeit addieren, Ergebnis steht noch in Registern ZAHL_C7:C0
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_kat	;Rcksichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0
;Sekunden in Hundertstel Sek umrechnen und zu Sternzeit addieren
	movfw DATEN_5		;kopiere die Sekunden der aktuellen Minute in 1.Multiplikationsregister
	movwf ZAHL_A0		;Sekunden werden in 100stel Sek umgerechnet
	clrf ZAHL_A1		; 1Sek = 100 Hundertstel Sek
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x64			;100 => 0x00000064
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
;Ergebnis zu Sternzeit addieren, Ergebnis steht noch in Registern ZAHL_C7:C0
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_kat	;Rcksichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0
	nop

;Korrektur der lokalen Sternzeit mit der Geo. Lnge Ost/West
	bcf STATUS, RP0		;Bank 0
	call lese_standort	;lade die Geo. Lnge in die F-Register
	bsf STATUS, RP0		;Bank 1
;Umrechnung von Winkelgrad in Hundertstel Sek: grad / 15 * 360.000 = grad * 24.000
	bcf STATUS, RP0		;Bank 0	
	movfw GRD_TEM		;Schiebe den Grad Wert in Multiplikationsregister 1
	bsf STATUS, RP0		;Bank 1
	movwf ZAHL_A0		;
	clrf ZAHL_A1		;
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0xC0			;24.000 => 0x00005DC0
	movwf ZAHL_B0
	movlw 0x5D
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
	call copy_zc_dat	;Zwischenspeichern in Registern DATEN_4:1
;Umrechnung von Winkelminuten in Hundertstel Sek: w-min / 15 * 6.000 = w-min * 400
	bcf STATUS, RP0		;Bank 0	
	movfw MIN_TEM		;Schiebe den Minuten Wert in Multiplikationsregister 1
	bsf STATUS, RP0		;Bank 1
	movwf ZAHL_A0		;
	clrf ZAHL_A1		;
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x90			;400 => 0x00000190
	movwf ZAHL_B0
	movlw 0x01
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
	call copy_dat_zb	;Kopiere DATEN_4:1 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_dat	;Zwischenspeichern in Registern DATEN_4:1
;Umrechnung von Winkelsekunden in Hundertstel Sek: w-sek * 100 / 15
	bcf STATUS, RP0		;Bank 0	
	movfw MIN_TEM_01	;Schiebe den Sekunden Wert in Multiplikationsregister 1
	bsf STATUS, RP0		;Bank 1
	movwf ZAHL_A0		;
	clrf ZAHL_A1		;
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x64			;100 => 0x00000064
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C7:C0
	movlw 0x0F			;/ 15 => 0x0000000F
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3	
	call div32			;das Ergebniss der Division steht in Registern ZAHL_C7:C0, Rest in ZAHL_B3:B0
	call copy_dat_zb	;Kopiere DATEN_4:1 Register nach ZAHL_B3:B0 Register
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	call copy_zc_dat	;Zwischenspeichern in Registern DATEN_4:1

	bcf STATUS, RP0		;Bank 0
	btfss PM_TEM,0		;Prfen des Vorzeichens (Ost =0, West =1) von Register in Bank 0
	goto korr_ost
	bsf STATUS, RP0		;Bank 1
	movlw 0x00			;Bei westlicher Lnge wird die Differenz zu 24 h addiert,
	movwf ZAHL_C0		; 24h = 8.640.000 Hundertstel Sek => 0x0083D600
	movlw 0xD6			;
	movwf ZAHL_C1		;
	movlw 0x83			;
	movwf ZAHL_C2		;
	clrf ZAHL_C3		;	
	clrf ZAHL_C4		;	
	clrf ZAHL_C5		;	
	clrf ZAHL_C6		;	
	clrf ZAHL_C7		;	
	call copy_dat_zb	;Geo. Lnge in Hundertstel Sek. Kopiere DATEN_4:1 Register nach ZAHL_B3:B0 Register
	call sub32			;32 bit Subtraktion: ZAHL_C - ZAHL_B = ZAHL_C

korr_ost				;addiere die Hundertstel Sek der Geo. Lnge zu Sternzeitregistern
	bsf STATUS, RP0		;Bank 1
	call copy_kat_zb	;Kopiere KAT_3:0 Register nach ZAHL_B3:B0 Register, Sternzeit
	call add32			;das Ergebniss der Addition steht in Registern ZAHL_C7:C0
	movlw 0x64			;durch 100 teilen => Sekunden, Rest sind Hundertstel Sek
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3	
	call div32			;das Ergebniss der Division steht in Registern ZAHL_C7:C0, Rest in ZAHL_B3:B0
	movfw ZAHL_B0		;kopiere Hundertstel Sekunden aus Rest in KAT_0 Register
	movwf KAT_0
	movlw 0x3C			;durch 60 teilen => Minuten, Rest sind Sekunden
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3	
	call div32			;das Ergebniss der Division steht in Registern ZAHL_C7:C0, Rest in ZAHL_B3:B0
	movfw ZAHL_B0		;kopiere Sekunden aus Rest in KAT_1 Register
	movwf KAT_1
	movlw 0x3C			;durch 60 teilen => Stunden, Rest sind Minuten
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3	
	call div32			;das Ergebniss der Division steht in Registern ZAHL_C7:C0, Rest in ZAHL_B3:B0
	movfw ZAHL_B0		;kopiere Minuten aus Rest in KAT_2 Register
	movwf KAT_2
	movfw ZAHL_C0		;kopiere Stunden in KAT_3 Register
	movwf KAT_3
	movlw d'24'			;prfen ob Stunden >=24 sind
	subwf KAT_3,w
	btfss STATUS,C		;prfe Carry Bit
	goto $+4			;Stunden sind <24
	movlw d'24'			;Stunden sind >=24, subtrahiere 24 von Stunden
	subwf KAT_3,f		;
	goto $-6			;erneute Prfung

;100stel Sekunden in Sternzeit Timerwerte umrechnen: (100-REST_SEK) * Timer SZ Register / 100
	movlw d'100'
	movwf ZAHL_A0		;Low byte des 1.Multiplikanten
	movfw KAT_0
	subwf ZAHL_A0,f		;(100-REST_SEK)
	clrf ZAHL_A1		;
	clrf ZAHL_A2		;
	clrf ZAHL_A3		;
	movlw 0x6F			;Timer Sternzeit, L Byte
	movwf ZAHL_B0		;2.Multiplikant
	movlw 0x1F			;Timer Sternzeit, H Byte
	movwf ZAHL_B1		;
	decf ZAHL_B1		;H Byte um 1 reduzieren
	clrf ZAHL_B2		;
	clrf ZAHL_B3		;
	call mul32			;das Ergebniss der Multiplikation steht in Registern ZAHL_C: (100-REST_SEK) * Timer SZ Register
	movlw d'100'		;lade NENNER_L mit Wert 100
	movwf ZAHL_A0		;
	clrf ZAHL_A1		;
	clrf ZAHL_A2		;
	clrf ZAHL_A3		;
	call div32			;das Ergebniss der Division steht in Registern ZAHL_C, der Rest in ZAHL_B
	incf ZAHL_C1		;(100-REST_SEK) * Timer SZ Register / 100, H Byte wieder um 1 erhhen

	bcf INTCON,GIE		;Neue Timer Werte fr Sternzeit kopiern, globale Interupts verbieten
	movfw ZAHL_C0
	bcf STATUS,RP0		;Select BANK_0
	movwf COUNT_L_SZ
	bsf STATUS,RP0		;Select BANK_1
	movfw ZAHL_C1
	bcf STATUS,RP0		;Select BANK_0
	movwf COUNT_H_SZ
	movlw 0x06			;Setzen des Zhlers, Digit:= 6	
	movwf DIGIT_SZ		;
	bsf STATUS,RP0		;Select BANK_1
	movfw KAT_1			;kopieren der Temp Register in die Sternzeit Register
	bcf STATUS,RP0		;Select BANK_0
	movwf SEK_SZ
	bsf STATUS,RP0		;Select BANK_1
	movfw KAT_2			;kopieren der Temp Register in die Sternzeit Register
	bcf STATUS,RP0		;Select BANK_0
	movwf MIN_SZ
	bsf STATUS,RP0		;Select BANK_1
	movfw KAT_3			;kopieren der Temp Register in die Sternzeit Register
	bcf STATUS,RP0		;Select BANK_0
	movwf STD_SZ
	bsf INTCON,GIE		;globale Interupts erlauben
	return				;Ende des "Sternzeit" Unterprogramms

;*******************************************************************************************
; Unterprogramme zum Unterprogramm "Sternzeit"
;*******************************************************************************************

monats_tage					;Unterprogramm zur Ermittlung der Tage eines Monats, Rcksprung mit Anzahl Tage in W
	movlw d'2'				;Ermittlung der Tage des Monats Februar
	subwf DATEN_6,w
	btfss STATUS,Z
	goto $+9				;Monat ist nicht Februar, weiter bei Abfrage der anderen Monate
	btfsc DATEN_7,0			;Monat ist Februar, wenn Bits 0+1 des Jahres = 0 => Schaltjahr
	goto $+5				;Bit 0 = 1 => kein Schaltjahr
	btfsc DATEN_7,1	
	goto $+3				;Bit 1 = 1 => kein Schaltjahr
	movlw d'29'				;Bits 0+1 = 0 => Schaltjahr
	return
	movlw d'28'				;=> kein Schaltjahr
	return
	movlw 0x1F				;Programmzhler fuer Tagestabelle im Bereich 0x1FF0..0x1FFF 
	movwf PCLATH			;
	movfw DATEN_6			;Hole Monat nach W
	call tab_tage			;Aufruf der Tagestabelle mit Tagen des Monats in W
	return


clear_zahl_c			;Unterprogramm zum Lschen der ZAHL_C Register
	clrf ZAHL_C0		;
	clrf ZAHL_C1		;
	clrf ZAHL_C2		;
	clrf ZAHL_C3		;
	clrf ZAHL_C4		;
	clrf ZAHL_C5		;
	clrf ZAHL_C6		;
	clrf ZAHL_C7		;
	return

copy_kat_zb				;Unterprogramm zum kopieren der KAT_3:0 Register nach ZAHL_B3:B0 Register
	movfw KAT_0			;
	movwf ZAHL_B0
	movfw KAT_1			;
	movwf ZAHL_B1
	movfw KAT_2			;
	movwf ZAHL_B2
	movfw KAT_3			;
	movwf ZAHL_B3
	return

copy_zc_kat				;Sichern des Ergebnis aus ZAHL_C3:C0 Registern in Sternzeitregister KAT_3:0
	movfw ZAHL_C0		;
	movwf KAT_0			;
	movfw ZAHL_C1		;
	movwf KAT_1			;
	movfw ZAHL_C2		;
	movwf KAT_2			;
	movfw ZAHL_C3		;
	movwf KAT_3			;
	return

copy_zc_dat				;Sichern des Ergebnis aus ZAHL_C3:C0 Registern in DATEN_4:1 Registern
	movfw ZAHL_C0		;
	movwf DATEN_1		;
	movfw ZAHL_C1		;
	movwf DATEN_2		;
	movfw ZAHL_C2		;
	movwf DATEN_3		;
	movfw ZAHL_C3		;
	movwf DATEN_4		;
	return

copy_dat_zb				;Kopiere DATEN_4:1 Register nach ZAHL_B3:B0 Register
	movfw DATEN_1		;
	movwf ZAHL_B0
	movfw DATEN_2		;
	movwf ZAHL_B1
	movfw DATEN_3		;
	movwf ZAHL_B2
	movfw DATEN_4		;
	movwf ZAHL_B3
	return

convert_hex_dig				;Unterprogramm zum Zerlegen einer 24-Bit Zahl in die einzelnen Digits (max.999.999)zur Anzeige auf dem Display
	bsf STATUS, RP0			;Bank 1
	clrf ZAHL_C3
	clrf ZAHL_C4
	clrf ZAHL_C5
	clrf ZAHL_C6
	clrf ZAHL_C7

	clrf KAT_5				;Lsche Register, von der 24-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	incf KAT_5				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf KAT_5				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf KAT_4				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf KAT_4				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C2 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf KAT_4				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf KAT_3				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf KAT_3				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf KAT_3				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf KAT_2				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf KAT_2				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf KAT_2				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	clrf KAT_1				;Lsche Register, von der 32-Bit Zahl in Register ZAHL_C wird ZAHL_B abgezogen bis Ergebnis negativ
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	incf KAT_1				;erhhe Digit um 1
	call sub32				;ZAHL_C minus ZAHL_B, das Ergebnis steht in ZAHL_C
	movfw ZAHL_C4			;Wenn Ergebnis der Subtraktion noch positiv ist, ist das berlaufregister ZAHL_C4 = 0
	btfsc STATUS,Z			;Prfen ob ZAHL_C4 >0<
	goto $-4				;berlaufregister ZAHL_C4 ist noch 0, zurck zur Schleife
	decf KAT_1				;berlaufregister ZAHL_C4 ist >0<, letzte Erhhung des DISP_ Digit 
	call add32				;   und letzte Subtraktion rckgngig machen

	movfw ZAHL_C0			;
	movwf KAT_0
	return

convert_dig_hex				;Unterprogramm zum Umwandeln der einzelnen Digits (max.999.999) in eine 24-Bit Zahl
	bsf STATUS, RP0			;Bank 1
	call clear_zahl_c		;Lsche Additionsregister ZAHL_C

	movfw KAT_5				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex4		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xA0				;Lade ZAHL_B mit 100.000 (0x000186A0)
	movwf ZAHL_B0
	movlw 0x86				;
	movwf ZAHL_B1
	movlw 0x01				;
	movwf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz KAT_5			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex4
	movfw KAT_4				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex3		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x10				;Lade ZAHL_B mit 10.000 (0x00002710)
	movwf ZAHL_B0
	movlw 0x27				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz KAT_4			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex3
	movfw KAT_3				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex2		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0xE8				;Lade ZAHL_B mit 1.000 (0x000003E8)
	movwf ZAHL_B0
	movlw 0x03				;
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz KAT_3			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex2
	movfw KAT_2				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex1		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x64				;Lade ZAHL_B mit 100 (0x00000064)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz KAT_2			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex1
	movfw KAT_1				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto conv_dig_hex0		;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movlw 0x0A				;Lade ZAHL_B mit 10 (0x0000000A)
	movwf ZAHL_B0
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	decfsz KAT_1			;Reduziere Digit um 1 und Prfe ob 0
	goto $-2				;Digit ist noch nicht 0, zurck zur Schleife
conv_dig_hex0
	movfw KAT_0				;Hole Digit nach W und prfe ob >0<
	btfsc STATUS,Z
	goto $+0x06				;Digit ist 0, weiter bei Auswertung des nchst kleineren Digits
	movwf ZAHL_B0			;Speichere Digit im Additionsregister
	clrf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call add32				;Addiere ZAHL_B zu ZAHL_C
	return


;*******************************************************************************************
; 32 bit Division: ZAHL_C / ZAHL_A = ZAHL_C, Rest steht in ZAHL_B
; Die Werte fr den Zhler mssen in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H) stehen
; Die Werte fr den Nenner mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) stehen
; Das ganzzahlige Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
; Der Rest steht in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
;*******************************************************************************************

div32
	clrf ZAHL_B3			;Rest von vorherigen Berechnungen lschen, ZAHL_B
	clrf ZAHL_B2
	clrf ZAHL_B1
	clrf ZAHL_B0					
	movlw d'64'				;LOOP_2 mit 64 laden, da 64 bit im Zhler
	movwf LOOP_2
div32_loop
	rlf ZAHL_C0,F       	;den gesamtem Zhler um 1 nach links schieben, ZAHL_C
	rlf ZAHL_C1,F
	rlf ZAHL_C2,F
	rlf ZAHL_C3,F
	rlf ZAHL_C4,F
	rlf ZAHL_C5,F
	rlf ZAHL_C6,F
	rlf ZAHL_C7,F

	rlf ZAHL_B0,F			;das Register fr den Rest um 1 nach links schieben
	rlf ZAHL_B1,F
	rlf ZAHL_B2,F
	rlf ZAHL_B3,F

	rlf LOOP_2,F			;das Carrybit wird vorbergehend im LOOP_2 gespeichert
									
	bsf LOOP_2,0
	movf ZAHL_A0,W			;Subtrahiere Nenner _0 von Rest _0 (REST_0-NENNER_0=REST_0)
	subwf ZAHL_B0,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B1,F			;reduziere REST_1 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub2			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub2
	movf ZAHL_A1,W			;Subtrahiere Nenner _1 von Rest _1 (REST_1-NENNER_1=REST_1)
	subwf ZAHL_B1,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B2,F			;reduziere REST_2 um 1
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub3			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub3
	movf ZAHL_A2,W			;Subtrahiere Nenner _2 von Rest _2 (REST_2-NENNER_2=REST_2)
	subwf ZAHL_B2,F			;
	btfsc STATUS, C			;prft den bertrag
	goto div32_sub4			;kein bertrag
	movlw 0x01
	subwf ZAHL_B3,F			;reduziere REST_3 um 1
	btfss STATUS, C			;prft den bertrag
	bcf LOOP_2,0			;bertrag vorhanden, lsche Bit 0 in LOOP_2

div32_sub4
	movf ZAHL_A3,W			;Subtrahiere Nenner _3 von Rest _3 (REST_3-NENNER_3=REST_3)
	subwf ZAHL_B3,F			;

	btfss STATUS, C			;prfe, ob das Ergebnis positiv ist
	bcf LOOP_2,0			;wenn nein, lsche gespeichertes Carrybit in LOOP_2
	btfsc LOOP_2,0			;prfe das gespeicherte Carrybit in LOOP_2
	goto div32_jump			;wenn Carry=1: springe nach div16_jump

	movf ZAHL_A0,W			;wenn Carry=0:	Subtraktion wieder rckgngig machen 
	addwf ZAHL_B0,F			; 
	btfsc STATUS,C
	incfsz ZAHL_B1,F
	goto $+4
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A1,W			;
	addwf ZAHL_B1,F			;
	btfsc STATUS,C
	incfsz ZAHL_B2,F
	goto $+2
	incf ZAHL_B3,F

	movf ZAHL_A2,W			; 
	addwf ZAHL_B2,F			; 
	btfsc STATUS,C
	incf ZAHL_B3,F

	movf ZAHL_A3,W			;
	addwf ZAHL_B3,F			; 

div32_jump
	bcf STATUS,C			;Carrybit lschen und das gespeicherte Carrybit in Register
	rrf LOOP_2,F			;LOOP_2 wieder nach rechts rausschieben.
	decfsz LOOP_2,F 		;LOOP_2 um 1 verringern und prfen ob 0
	goto div32_loop         ;Counter ist noch nicht 0, zurck zur Schleife
	 
	rlf ZAHL_C0,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlf ZAHL_C1,F			;ZAEHLER		das richtige Ergebnis im Zhler steht
	rlf ZAHL_C2,F			;ZAEHLER
	rlf ZAHL_C3,F			;ZAEHLER
	rlf ZAHL_C4,F			;ZAEHLER
	rlf ZAHL_C5,F			;ZAEHLER
	rlf ZAHL_C6,F			;ZAEHLER
	rlf ZAHL_C7,F			;ZAEHLER_H

	return
	
;*******************************************************************************************
; 32 bit Multiplikation: ZAHL_A x ZAHL_B = ZAHL_C
; Die Werte fr die Multiplikation mssen in den Registern ZAHL_A0 (L) bis ZAHL_A3 (H) 
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht dann in den Registern ZAHL_C0 (L) bis ZAHL_C7 (H)
;*******************************************************************************************
mul32
	call clear_zahl_c		;Lschen des vorherigen Ergebnisses (ZAHL_C0 - ZAHL_C7)
	bsf ZAHL_C3,7			;um zu erkennen, wann 32 mal geschoben wurde
mul32_loop
	rrf ZAHL_A3,F			;verschiebe gesammtes Register Multiplikant1 um 1 nach rechts
	rrf ZAHL_A2,F			;
	rrf ZAHL_A1,F			;
	rrf ZAHL_A0,F			;wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul32_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert

	movf ZAHL_B0,W			;Beginn der 32 bit Addition: Addiere Multiplikant2 _0 Register zu Ergebnis _4 Register
	addwf ZAHL_C4,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C5,F		;es ist ein bertrag vorhanden und das Ergebnis _5 Register wird um 1 erhht
	goto $+4
	incfsz ZAHL_C6,F		;wenn Ergebnis _5 Register berluft, erhhe Ergebnis _6 Register um 1
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B1,W			;Addiere Multiplikant2 _1 Register zu Ergebnis _5 Register
	addwf ZAHL_C5,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C6,F		;es ist ein bertrag vorhanden und das Ergebnis _6 Register wird um 1 erhht
	goto $+2
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B2,W			;Addiere Multiplikant2 _2 Register zu Ergebnis _6 Register
	addwf ZAHL_C6,F			;
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C7,F			;wenn Ergebnis _6 Register berluft, erhhe Ergebnis _7 Register um 1

	movf ZAHL_B3,W			;Addiere Multiplikant2 _3 Register zu Ergebnis _7 Register
	addwf ZAHL_C7,F			;

mul32_rotate
	rrf ZAHL_C7,F			;Das 64 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf ZAHL_C6,F
	rrf ZAHL_C5,F
	rrf ZAHL_C4,F
	rrf ZAHL_C3,F
	rrf ZAHL_C2,F
	rrf ZAHL_C1,F
	rrf ZAHL_C0,F
	btfss STATUS,C			;ist das Bit 7 von ZAHL_C3 im Carry angekommen wurde das Ergebnis
	goto mul32_loop			;insgesamt 32 mal rotiert und die Berechnung ist beendet
	return                  ;

;*******************************************************************************************
; 32 bit Addition: ZAHL_C + ZAHL_B = ZAHL_C
; Die Werte fr die Addition mssen in den Registern ZAHL_C0 (L) bis ZAHL_C3 (H)
; und ZAHL_B0 (L) bis ZAHL_B3 (H) stehen
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

add32
	movf ZAHL_B0,W			;Hole Low Byte Zahl2 ins W-Register
	addwf ZAHL_C0,F			;addiere Low Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C1,F		;es ist ein bertrag vorhanden und das L+1 Byte von Zahl2 wird um 1 erhht
	goto $+6				;kein bertrag beim L+1 Byte von Zahl2, 
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B1,W			;Hole L+1 Byte Zahl2 ins W-Register
	addwf ZAHL_C1,F			;addiere L+1 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C2,F		;es ist ein bertrag vorhanden und das L+2 Byte von Zahl2 wird um 1 erhht
	goto $+4				;kein bertrag beim L+2 Byte von Zahl2, 
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B2,W			;Hole L+2 Byte Zahl2 ins W-Register
	addwf ZAHL_C2,F			;addiere L+2 Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz ZAHL_C3,F		;es ist ein bertrag vorhanden und das L+3 Byte von Zahl2 wird um 1 erhht
	goto $+2				;kein bertrag beim L+3 Byte von Zahl2, 
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht

	movf ZAHL_B3,W			;Hole High Byte Zahl2 ins W-Register
	addwf ZAHL_C3,F			;addiere High Byte Zahl1
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incf ZAHL_C4,F			;es ist ein bertrag vorhanden und das L+4 Byte von Zahl2 wird um 1 erhht
	return

;*******************************************************************************************
; 32 bit Subtraktion: ZAHL_C - ZAHL_B = ZAHL_C
; Die Werte fr die Subtraktion mssen in den Registern ZAHL_B0 (L) bis ZAHL_B3 (H)
; und ZAHL_C0 (L) bis ZAHL_C4 (H) stehen, wobei ZAHL_B von ZAHL_C subtrahiert wird 
; Das Ergebnis steht in den Registern ZAHL_C0 (L) bis ZAHL_C4 (H)
;*******************************************************************************************

sub32
	movfw ZAHL_B0			;Hole 0/7-Byte von B nach W
	subwf ZAHL_C0,F			;Subtrahiere B von C, 0/7-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 8/15-Byte
	subwf ZAHL_C1,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_2			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 8/15-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_2
	movfw ZAHL_B1			;Hole 8/15-Byte von B nach W
	subwf ZAHL_C1,F			;Subtrahiere B von C, 8/15-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 16/23-Byte
	subwf ZAHL_C2,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_3			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 16/23-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_3
	movfw ZAHL_B2			;Hole 16/23-Byte von B nach W
	subwf ZAHL_C2,F			;Subtrahiere B von C, 16/23-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 24/31-Byte
	subwf ZAHL_C3,F
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_4			;Ergebnis ist positiv (Carry Bit = H) weiter bei Subtraktion der 24/31-Bytes
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_4
	movfw ZAHL_B3			;Hole 24/31-Byte von B nach W
	subwf ZAHL_C3,F			;Subtrahiere B von C, 24/31-Byte
	btfsc STATUS,C			;prfe ob Ergebnis negativ
	goto sub32_end			;Ergebnis ist positiv (Carry Bit = H) weiter zum Ende des Unterprogramms
	movlw 0x01				;Ergebnis ist negativ (Carry Bit = L), subtrahiere 1 von C, 32/39-Byte
	subwf ZAHL_C4,F
sub32_end
	return

runden_div32					;Unterprogramm zum ganzzahligen Runden des Ergebnisses der Division aus div32
	bcf STATUS,C				;lsche Carry Bit, damit es beim rotieren nicht strt
	rlf ZAHL_B0					;ZAHL_B 1x nach links schieben, d.h. mit 2 malnehmen 
	rlf ZAHL_B1
	rlf ZAHL_B2
	rlf ZAHL_B3
	btfsc STATUS,C
	goto runden_div32_inc	

	movfw ZAHL_A3				;wenn Rest >= ZAHL_A/2 ist, d.h. >=0,5 ist, erhhe ZAHL_C um 1
	subwf ZAHL_B3,w				;subtrahiere ZAHL_A/2 von Rest
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Ergebnis negativ (STATUS,C=0), Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0  (STATUS,Z=1)
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A2				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A2 ins W-Reg.
	subwf ZAHL_B2,w				;subtrahiere W-Reg. von ZAHL_B2
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A1				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A1 ins W-Reg.
	subwf ZAHL_B1,w				;subtrahiere W-Reg. von ZAHL_B1
	btfss STATUS,C				;prfen, ob Ergebnis negativ, d.h. <0
	return						;Ergebnis negativ, Rcksprung
	btfss STATUS,Z				;prfen, ob Ergebnis = 0
	goto runden_div32_inc		;Ergebnis positiv, erhhe ZAHL_C um 1
	movfw ZAHL_A0				;Ergebnis = 0, nchst kleineres Byte prfen, hole ZAHL_A0 ins W-Reg.
	subwf ZAHL_B0,w				;subtrahiere W-Reg. von ZAHL_B0
	btfss STATUS,C				;prfen, ob Ergebnis 0 oder positiv (STATUS,C=1)
	return						;Rest (ZAHL_B) < ZAHL_A/2, Rcksprung aus dem Unterprogramm
runden_div32_inc	
	incfsz ZAHL_C0				;Rest (ZAHL_B) >= ZAHL_A/2, erhhe ZAHL_C0 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C1				;berlauf vorhanden,  erhhe ZAHL_C1 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incfsz ZAHL_C2				;berlauf vorhanden,  erhhe ZAHL_C2 um 1 und prfe den berlauf
	return						;kein berlauf, Rcksprung aus dem Unterprogramm
	incf ZAHL_C3				;berlauf vorhanden,  erhhe ZAHL_C3 um 1
	return						;Rcksprung aus dem Unterprogramm



;*******************************************************************************************
; Unterprogramme zum Lesen/Schreiben aus/ins interne EEPROM
;*******************************************************************************************

lese_parameter				;laden der Parameter aus dem internen EEPROM in die F-Register
	movlw 0x09				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_HIGH_L			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x0A				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_ENC_A			;Kopieren der Daten des W-Registers ins F-Register
	movwf COUNT_ENC_RA		;
	movlw 0x0B				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_ENC_B			;Kopieren der Daten des W-Registers ins F-Register
	movwf LOOP_ENC_RA		;
	movlw 0x0C				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_ENC_C			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x0D				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DE_NORM_L			;Kopieren der Daten des W-Registers ins F-Register
	movwf DE_LOW			;
	movlw 0x0E				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DE_HIGH_L			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x0F				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DE_ENC_A			;Kopieren der Daten des W-Registers ins F-Register
	movwf COUNT_ENC_DE	;
	movlw 0x10				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DE_ENC_B			;Kopieren der Daten des W-Registers ins F-Register
	movwf LOOP_ENC_DE		;
	movlw 0x11				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf DE_ENC_C			;Kopieren der Daten des W-Registers ins F-Register
	movlw 0x1B				;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STAT_MOTOR		;Kopieren der Daten des W-Registers ins F-Register

	movlw 0x1A				;EEPROM Adresse fr Nachfhrgeschwindigkeit
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register	
	movwf TEMP_13
lese_parameter1
	movlw d'1'				;
	subwf TEMP_13,w
	btfss STATUS,Z			;
	goto $+3
	movlw 0x03				;1.Adresse fr stellare Parameter Zhler A (B/C)
	goto lese_parameter2
	movlw d'2'				;
	subwf TEMP_13,w
	btfss STATUS,Z			;
	goto $+4
	bcf SPEED,4				;Lsche Bit => solar
	movlw 0x00				;1.Adresse fr solare Parameter Zhler A (B/C)
	goto lese_parameter2
	bcf SPEED,5				;Lsche Bit => lunar
	movlw 0x06				;1.Adresse fr lunare Parameter Zhler A (B/C)
lese_parameter2
	movwf TEMP_14			;EEPROM Adresse
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_NORM_A			;Kopieren der Daten des W-Registers ins F-Register
	movwf RA_LOW_A			;
	movwf RA_LOW_TEMP		;
	movfw TEMP_14
	addlw 0x01				;EEPROM Adresse (TEMP_14 + 1)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_NORM_B			;Kopieren der Daten des W-Registers ins F-Register
	movwf RA_LOW_B			;
	movwf COUNT_L_RA_TEMP	;
	movfw TEMP_14
	addlw 0x02				;EEPROM Adresse (TEMP_14 + 2)
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf RA_NORM_C			;Kopieren der Daten des W-Registers ins F-Register
	movwf RA_LOW_C			;
	return

schreibe_speed
	movfw TEMP_DIG_CNT
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x1A				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

lese_pa_st
	movlw 0x03				;Adresse fr stellare Parameter Zhler A
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STD_TEMP
	movlw 0x04				;Adresse fr stellare Parameter Zhler B
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEMP
	movlw 0x05				;Adresse fr stellare Parameter Zhler C
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SEK_TEMP
	return

schreibe_pa_st				;Unterprogramm zum Schreiben der Daten in Stellar Register
	movfw STD_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x03				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x04				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x05				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

lese_pa_so
	movlw 0x00				;Adresse fr solare Parameter Zhler A
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STD_TEMP
	movlw 0x01				;Adresse fr solare Parameter Zhler B
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEMP
	movlw 0x02				;Adresse fr solare Parameter Zhler C
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SEK_TEMP
	return

schreibe_pa_so				;Unterprogramm zum Schreiben der Daten in Solar Register
	movfw STD_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x00				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x01				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x02				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

lese_pa_lu
	movlw 0x06				;Adresse fr lunare Parameter Zhler A
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf STD_TEMP
	movlw 0x07				;Adresse fr lunare Parameter Zhler B
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEMP
	movlw 0x08				;Adresse fr lunare Parameter Zhler C
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf SEK_TEMP
	return

schreibe_pa_lu				;Unterprogramm zum Schreiben der Daten in Lunar Register
	movfw STD_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x06				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x07				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x08				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

schreibe_pa_ramxde			;Unterprogramm zum Schreiben der Daten in RA max, DE norm, DE max Register
	movfw STD_TEMP
	movwf RA_HIGH_L
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x09				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf DE_NORM_L
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0D				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf DE_HIGH_L
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0E				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

schreibe_pa_patra			;Unterprogramm zum Schreiben der Daten in RA Register
	movfw STD_TEMP
	movwf RA_ENC_A
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0A				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf RA_ENC_B
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0B				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf RA_ENC_C
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0C				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

schreibe_pa_patde			;Unterprogramm zum Schreiben der Daten in DE Register
	movfw STD_TEMP
	movwf DE_ENC_A
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0F				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw MIN_TEMP
	movwf DE_ENC_B
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x10				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	movfw SEK_TEMP
	movwf DE_ENC_C
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x11				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return


lese_standort					;Unterprogramm zum lesen der Standortdaten (geogr. Lnge) aus dem internen EEPROM
	movlw 0x12					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf PM_TEM				;Kopieren der Daten des W-Registers ins F-Register, Ost=0, West=1
	movlw 0x13					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf GRD_TEM				;Kopieren der Daten des W-Registers ins F-Register, Winkelgrad
	movlw 0x14					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEM				;Kopieren der Daten des W-Registers ins F-Register, Winkelminuten
	movlw 0x15					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEM_01			;Kopieren der Daten des W-Registers ins F-Register, Winkelsekunden
	return

schreibe_standort				;Unterprogramm zum schreiben der Positionsdaten (geogr. Lnge) in das interne EEPROM
	movfw PM_TEM				;Kopieren der Daten ins W-Register, Ost=0, West=1
	movwf TEMP_13
	movlw 0x12					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	movfw GRD_TEM				;Kopieren der Daten ins W-Register, Winkelgrad
	movwf TEMP_13
	movlw 0x13					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	movfw MIN_TEM				;Kopieren der Daten ins W-Register, Winkelminuten
	movwf TEMP_13
	movlw 0x14					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	movfw MIN_TEM_01			;Kopieren der Daten ins W-Register, Winkelsekunden
	movwf TEMP_13
	movlw 0x15					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	return


lese_dcfpara					;Unterprogramm zum lesen der DCF parameter aus dem internen EEPROM
	movlw 0x16					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf DCF_H_MAX				;Kopieren der Daten des W-Registers ins F-Register, Maximalwert fr DCF Bit "High"
	movlw 0x17					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf DCF_H_MIN				;Kopieren der Daten des W-Registers ins F-Register, Minimalwert fr DCF Bit "High"
	movlw 0x18					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf DCF_L_MAX				;Kopieren der Daten des W-Registers ins F-Register, Maximalwert fr DCF Bit "Low"
	movlw 0x19					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf DCF_L_MIN				;Kopieren der Daten des W-Registers ins F-Register, Minimalwert fr DCF Bit "Low"
	bcf STATUS,RP0 				;Bank 0
	return

schreibe_dcfpara				;Unterprogramm zum schreiben der DCF parameter ins interne EEPROM
	bsf STATUS,RP0 				;Bank 1
	movfw DCF_H_MAX				;Kopieren der Daten ins W-Register, Ost=0, West=1
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x16					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw DCF_H_MIN				;Kopieren der Daten ins W-Register, Winkelgrad
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x17					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw DCF_L_MAX				;Kopieren der Daten ins W-Register, Winkelminuten
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x18					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw DCF_L_MIN				;Kopieren der Daten ins W-Register, Winkelsekunden
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x19					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	return

schreibe_mot_stat				;Unterprogramm zum schreiben der Motor Daten ins interne EEPROM
	movfw STAT_MOTOR			;Kopieren der Daten ins W-Register, Motoren RA/DE Ein/Aus, Normal/Invertiert
	movwf TEMP_13
	movlw 0x1B					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	return

lese_ratio_ra					;Unterprogramm zum lesen des bersetzungsverhltnis RA Achse aus dem internen EEPROM
	movlw 0x1C					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C2				;Kopieren der Daten des W-Registers ins F-Register, I RA Achse H-Byte
	movlw 0x1D					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C1				;Kopieren der Daten des W-Registers ins F-Register, I RA Achse M-Byte
	movlw 0x1E					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C0				;Kopieren der Daten des W-Registers ins F-Register,  I RA Achse L-Byte
	movlw 0x1F					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZZ_UZ					;Kopieren der Daten des W-Registers ins F-Register, 1/100stel n-max RA Achse
	bcf STATUS,RP0 				;Bank 0
	return

schreibe_ratio_ra				;Unterprogramm zum schreiben des bersetzungsverhltnis RA Achse in das interne EEPROM
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C2				;Kopieren der Daten ins W-Register, I RA Achse H-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x1C					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C1				;Kopieren der Daten ins W-Register, I RA Achse M-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x1D					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C0				;Kopieren der Daten ins W-Register, I RA Achse L-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x1E					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZZ_UZ					;Kopieren der Daten ins W-Register, 1/100stel n-max RA Achse
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x1F					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	return

lese_ratio_de					;Unterprogramm zum lesen des bersetzungsverhltnis DE Achse aus dem internen EEPROM
	movlw 0x20					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C2				;Kopieren der Daten des W-Registers ins F-Register, I DE Achse H-Byte
	movlw 0x21					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C1				;Kopieren der Daten des W-Registers ins F-Register, I DE Achse M-Byte
	movlw 0x22					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZAHL_C0				;Kopieren der Daten des W-Registers ins F-Register,  I DE Achse L-Byte
	movlw 0x23					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	bsf STATUS,RP0 				;Bank 1
	movwf ZZ_UZ					;Kopieren der Daten des W-Registers ins F-Register, 1/100stel n-max DE Achse
	bcf STATUS,RP0 				;Bank 0
	return

schreibe_ratio_de				;Unterprogramm zum schreiben des bersetzungsverhltnis DE Achse in das interne EEPROM
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C2				;Kopieren der Daten ins W-Register, I DE Achse H-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x20					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C1				;Kopieren der Daten ins W-Register, I DE Achse M-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x21					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZAHL_C0				;Kopieren der Daten ins W-Register, I DE Achse L-Byte
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x22					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS,RP0 				;Bank 1
	movfw ZZ_UZ					;Kopieren der Daten ins W-Register, 1/100stel n-max DE Achse
	bcf STATUS,RP0 				;Bank 0
	movwf TEMP_13
	movlw 0x23					;EEPROM Adresse
	call writeEE				;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	return


readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht
	bsf STATUS, RP0 			;EECON1 liegt in der Bank 3 
	bsf STATUS, RP1
	btfsc EECON1, WR			;prfen, ob der letzte Schreibvorgang beendet ist
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf STATUS, RP0 			;EEADR liegt in der Bank 2 
 	movwf EEADR 				;schreibe die Adresse in EEADR 
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Access data memory
	bsf EECON1, RD 				;EEPROM Leseproze starten 
	bcf STATUS, RP0 			;Bank 2 
	movf EEDATA, W 				;Die Daten der EEPROM Zelle nach W kopieren 
	bcf STATUS, RP1 			;Bank 0 
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht
	bsf STATUS, RP0				;Bank 3 (EECON1 ist in Bank 3)
	bsf STATUS, RP1
	btfsc EECON1, WR			;prfen, ob das EEPROM beschrieben werden kann
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf STATUS, RP0				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEADR					;Die Adresse an die die Daten geschrieben werden sollen ist im W-Register
	bcf STATUS, RP1				;Bank 0
	movfw TEMP_13				;hole zu speichernden Wert ins W-Register
	bsf STATUS, RP1				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEDATA				;Wert wird in EEADR geschrieben.
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Das Bit EEPGD lschen, um das interne Daten-EEPROM auszuwhlen. Wird das Bit gesetzt, wird der Progamm-Flash ausgewhlt
	bsf EECON1, WREN			;erlaubt das Beschreiben des Daten-EEPROM
	bcf INTCON, GIE				;falls Interrupts verwendet werde, mssen diese abgeschaltet werden	
	movlw 0x55					;Um das EEPROM zu beschreiben, mssen die folgenden 5 Befehle
	movwf EECON2				;genau in dieser Reihenfolge aufgerufen werden.
	movlw 0xAA					;
	movwf EECON2				;
	bsf EECON1, WR				;Jetzt beginnt der eigentliche Schreibvorgang.
	bsf INTCON, GIE				;Die Interrupts knnen jetzt wieder freigegeben werden.
	bcf EECON1, WREN			;Das Beschreiben des EEPROM verbieten.
	bcf STATUS, RP0				;Bank 0
	bcf STATUS, RP1
	return						;Zurck ins Hauptprogramm

calc_para_ra
	call copy_kat_zahla		;Solar: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	clrf TBIT_H 			;lsche Register
	movlw 0x00				;Positioniergeschwindigkeit Vsolar x 32 x 2^20 
	movwf ZAHL_B0			;	= 15 arcsec/sec x 32 x 2^20 = 503.316.480 = 0x1E000000
	movlw 0x00				;Kopiere 0x1E000000 in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0x00
	movwf ZAHL_B2
	movlw 0x1E
	movwf ZAHL_B3
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf STATUS,RP0			;Select BANK_0	
	call schreibe_pa_so		;Unterprogramm zum Schreiben der Parameter in Solar Register des internen EEPROMS

	call copy_kat_zahla		;Stellar: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	movlw 0xBD				;Positioniergeschwindigkeit Vstellar x 32 x 2^20 
	movwf ZAHL_B0			;	= 15,041067 arcsec/sec x 32 x 2^20 = 504.694.461 = 0x1E1506BD
	movlw 0x06				;Kopiere 0x1E1506BD in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0x15
	movwf ZAHL_B2
	movlw 0x1E
	movwf ZAHL_B3
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf STATUS,RP0			;Select BANK_0	
	call schreibe_pa_st		;Unterprogramm zum Schreiben der Parameter in Stellar Register des internen EEPROMS

	call copy_kat_zahla		;Lunar: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	movlw 0x41				;Positioniergeschwindigkeit Vlunar x 32 x 2^20 
	movwf ZAHL_B0			;	= 14,492052 arcsec/sec x 32 x 2^20 = 486.272.577 = 0x1CFBEE41
	movlw 0xEE				;Kopiere 0x1CFBEE41 in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0xFB
	movwf ZAHL_B2
	movlw 0x1C
	movwf ZAHL_B3
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf STATUS,RP0			;Select BANK_0	
	call schreibe_pa_lu		;Unterprogramm zum Schreiben der Parameter in Lunar Register des internen EEPROMS

	call copy_kat_zahla		;Int. Encoder RA: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	movlw 0x00				;Auflsung Encoder RA Rra x 4 x 16 x 2^20 
	movwf ZAHL_B0			;	= 15 arcsec x 4 x 16 x 2^20 = 503.316.480 = 0x1E000000
	movlw 0x00				;Kopiere 0x1E000000 in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0x00
	movwf ZAHL_B2
	movlw 0x1E
	movwf ZAHL_B3
	bsf TBIT_H,0 			;setze Bit (zum berspringen einiger Berechnungen in der Berechnungsroutine
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf TBIT_H,0 			;lsche Bit
	bcf STATUS,RP0			;Select BANK_0	
	call schreibe_pa_patra	;Unterprogramm zum Schreiben der Parameter in Encoder RA Register des internen EEPROMS

	bsf STATUS,RP0			;Select BANK_1
	movfw ZZ_UZ				;Maximale Frequenz RA Motor in Hz, 100er
	call calc_para_nmax
	movfw ZAHL_C0
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x09				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return

calc_para_de
	call copy_kat_zahla		;Deklination: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	clrf TBIT_H 			;lsche Register
	bsf TBIT_H,1			;setze Bit fr Berechnungsende nach Ermiitlung Zhler A
	movlw 0x00				;Positioniergeschwindigkeit Vsolar x 32 x 2^20 
	movwf ZAHL_B0			;	= 15 arcsec/sec x 32 x 2^20 = 503.316.480 = 0x1E000000
	movlw 0x00				;Kopiere 0x1E000000 in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0x00
	movwf ZAHL_B2
	movlw 0x1E
	movwf ZAHL_B3
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf STATUS,RP0			;Select BANK_0
	btfsc STATUS,C			;Abfrage berlaufbit aus vorheriger Division durch 2 (rrf)
	incf STD_TEMP			;Bit ist gesetzt, Erhhe Zhler A um 1 (Ganzzahliges Runden)
	movfw STD_TEMP
	movwf  TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0D				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM

	call copy_kat_zahla		;Int. Encoder DE: Kopiere Gesamtbersetzung Ig in Multiplikationsregister ZAHL_A
	movlw 0x00				;Auflsung Encoder DE Rde x 4 x 16 x 2^20 
	movwf ZAHL_B0			;	= 12 arcsec x 4 x 16 x 2^20 = 402.653.184 = 0x18000000
	movlw 0x00				;Kopiere 0x18000000 in 2. Multiplikationsregister ZAHL_B
	movwf ZAHL_B1
	movlw 0x00
	movwf ZAHL_B2
	movlw 0x18
	movwf ZAHL_B3
	clrf TBIT_H 			;lsche Register	
	bsf TBIT_H,0 			;setze Bit (zum berspringen einiger Berechnungen in der Berechnungsroutine
	call calc_routine		;Aufruf der Berechnungsroutine
	bcf TBIT_H,0 			;lsche Bit
	bcf STATUS,RP0			;Select BANK_0	
	call schreibe_pa_patde	;Unterprogramm zum Schreiben der Parameter in Encoder RA Register des internen EEPROMS

	bsf STATUS,RP0			;Select BANK_1
	movfw ZZ_UZ				;Maximale Frequenz RA Motor in Hz, 100er
	call calc_para_nmax
	movfw ZAHL_C0
	bcf STATUS,RP0			;Select BANK_0
	movwf TEMP_13			;Kopiere den Wert im Temp Register fr EEPROM
	movlw 0x0E				;EEPROM Adresse
	call writeEE			;Aufruf des Unterprogramms zum Schreiben der Daten ins EEPROM
	return


copy_kat_zahla				;Kopieren der KAT Register in die ZAHL_A Register
	bsf STATUS,RP0			;Select BANK_1
	movfw KAT_0
	movwf ZAHL_A0
	movfw KAT_1
	movwf ZAHL_A1
	movfw KAT_2
	movwf ZAHL_A2
	clrf ZAHL_A3
	return


calc_routine
	call mul32
	movlw 0x80				;arcsec/360 = 360 x 60 x 60 = 1.296.000 = 0x0013C680
	movwf ZAHL_A0
	movlw 0xC6
	movwf ZAHL_A1
	movlw 0x13
	movwf ZAHL_A2
	clrf ZAHL_A3
	call div32
	call runden_div32		;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnis aus div32
	btfsc TBIT_H,0
	goto calc_routine1
	movfw ZAHL_C0
	movwf ZAHL_A0
	movfw ZAHL_C1
	movwf ZAHL_A1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movfw ZAHL_C3
	movwf ZAHL_A3
	call clear_zahl_c		;lsche ZAHL_C Register
	movlw 0x80				;interne Timerfrequenz x 2^40 = 7.812,5 x 2^40 = 8.589.934.592.000.000
	movwf ZAHL_C4			;	= 0x001E848000000000
	movlw 0x84
	movwf ZAHL_C5
	movlw 0x1E
	movwf ZAHL_C6
	call div32
	call runden_div32		;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnis aus div32
calc_routine1				;Rest (x2^20) sichern in 
	movfw ZAHL_C0
	movwf DATEN_1
	movwf ADR_L_MIN			;Erforderliche Schritte (x2^20) Bit 0-7, wird fr Fehlerberechnung bentigt
	movfw ZAHL_C1
	movwf DATEN_2
	movwf ADR_L_MAX			;Erforderliche Schritte (x2^20) Bit 8-15, wird fr Fehlerberechnung bentigt
	movfw ZAHL_C2
	movwf DATEN_3
	movwf ADR_H_MIN			;Erforderliche Schritte (x2^20) Bit 16-23, wird fr Fehlerberechnung bentigt
	movfw ZAHL_C3
	movwf ADR_H_MAX			;Erforderliche Schritte (x2^20) Bit 24-31, wird fr Fehlerberechnung bentigt
	bcf DATEN_3,4
	bcf DATEN_3,5
	bcf DATEN_3,6
	bcf DATEN_3,7	
	;Zhler A:
	movfw ZAHL_C2
	movwf DATEN_4	
	movfw ZAHL_C3
	movwf DATEN_5
	movlw 0x04
	movwf LOOP_2
	bcf STATUS,C
	rrf DATEN_5
	rrf DATEN_4				;=>Zhler A
	decfsz LOOP_2
	goto $-4
	movfw DATEN_4			;Kopiere Zhler A nach STD_TEMP Register
	bcf STATUS,RP0			;Select BANK_0
	movwf STD_TEMP
	bsf STATUS,RP0			;Select BANK_1
	btfsc TBIT_H,1			;Abfrage Bit1. Bei Zhler A DE ist die Berechnung hier zu Ende
	return
	movlw 0xFF				;Daten_7 - 9 werden fr Zwischenspeicherung des kleinsten Fehlers verwendet
	movwf DATEN_7
	movwf DATEN_8
	movwf DATEN_9
	movlw 0x01
	movwf DCF_BIT			;wird fr Loops Gesamt ( = Zhler B + Zhler C) verwendet 
loop_bruch
	movfw DCF_BIT
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	movfw DATEN_1
	movwf ZAHL_B0
	movfw DATEN_2
	movwf ZAHL_B1
	movfw DATEN_3
	movwf ZAHL_B2
	clrf ZAHL_B3
	call mul32

	movfw ZAHL_C2			;teilen durch 2^20
	movwf DATEN_5
	movfw ZAHL_C3
	movwf DATEN_6
	movlw 0x04
	movwf LOOP_2
	bcf STATUS,C
	rrf DATEN_6
	rrf DATEN_5
	decfsz LOOP_2
	goto $-4
	btfsc STATUS,C			;
	incf DATEN_5			;Zhler C steht in DATEN_5 Register
	movfw DATEN_5
	subwf DCF_BIT,w
	movwf DATEN_6			;Zhler B: sichern in DATEN_6 Register
	call clear_zahl_c
	movfw DATEN_5
	movwf ZAHL_C2
	movlw 0x04
	movwf LOOP_2
	bcf STATUS,C
	rlf ZAHL_C2
	rlf ZAHL_C3
	decfsz LOOP_2
	goto $-4
	movfw DCF_BIT
	movwf ZAHL_A0
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call div32
	call runden_div32		;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnis aus div32
	movfw DATEN_1
	movwf ZAHL_B0
	movfw DATEN_2
	movwf ZAHL_B1
	movfw DATEN_3
	movwf ZAHL_B2
	clrf ZAHL_B3
	call sub32
	clrf ZAHL_A3			;Zahl_A3 wird zur Zwischenspeicherung des Vorzeichens verwendet, 0=+, 1=-
	btfss ZAHL_C4,7
	goto test_kleiner		;
	bsf ZAHL_A3,0			;Vorzeichen ist minus
	call add32
	movfw ZAHL_C0
	movwf ZAHL_B0
	movfw ZAHL_C1
	movwf ZAHL_B1
	movfw ZAHL_C2
	movwf ZAHL_B2
	movfw ZAHL_C3
	movwf ZAHL_B3
	call clear_zahl_c
	movfw DATEN_1
	movwf ZAHL_C0
	movfw DATEN_2
	movwf ZAHL_C1
	movfw DATEN_3
	movwf ZAHL_C2
	call sub32
test_kleiner
	movfw ZAHL_C0			;Zwischenspeichern des Fehlers in ZAHL_A0 - A2 Registern
	movwf ZAHL_A0
	movfw ZAHL_C1
	movwf ZAHL_A1
	movfw ZAHL_C2
	movwf ZAHL_A2
	movfw DATEN_7
	movwf ZAHL_B0
	movfw DATEN_8
	movwf ZAHL_B1
	movfw DATEN_9
	movwf ZAHL_B2
	clrf ZAHL_B3
	call sub32
	btfss ZAHL_C4,7			;prfen ob Fehler Neu - Fehler Alt < 0, d.h. Fehler Neu < Fehler Alt
	goto test_klein_end
	movfw ZAHL_A0
	movwf DATEN_7	
	movfw ZAHL_A1
	movwf DATEN_8
	movfw ZAHL_A2
	movwf DATEN_9
	movfw ZAHL_A3
	movwf DATEN_10
	movfw DATEN_6			;Zhler B: sichern in MIN_TEMP Register
	bcf STATUS,RP0			;Select BANK_0
	movwf MIN_TEMP
	bsf STATUS,RP0			;Select BANK_1
	movfw DATEN_5			;Zhler C: sichern in SEK_TEMP Register
	bcf STATUS,RP0			;Select BANK_0
	movwf SEK_TEMP
test_klein_end
	bsf STATUS,RP0			;Select BANK_1
	incfsz DCF_BIT			;erhhe Zhler (B+C) und prfe auf berlauf
	goto loop_bruch			;noch kein berlauf, zurck zur Schleife
	bcf STATUS,RP0			;Select BANK_0
	movfw MIN_TEMP			;Prfen, ob Zhler B >0
	btfsc STATUS,Z
	goto $+3				;Zhler B =0: => Zhler A und B um 1 erhhen, Zhler C auf 0 setzen
	bsf STATUS,RP0			;Select BANK_1	
	return
	incf STD_TEMP			;Zhler A um 1 erhhen
	incf MIN_TEMP			;Zhler B um 1 erhhen
	clrf SEK_TEMP			;Zhler C lschen
	bsf STATUS,RP0			;Select BANK_1	
	return

calc_para_nmax				;  interne Frequenz x 2 / 4 / Nmax
	movwf ZAHL_A0			; 	=> 7812,5 x 2 / (4 x Nmax)
	clrf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	movlw 0x90				;Multiplizieren von Nmax (100er) mit 400 => hex 0x0190
	movwf ZAHL_B0
	movlw 0x01
	movwf ZAHL_B1
	clrf ZAHL_B2
	clrf ZAHL_B3
	call mul32				;Multiplikation, Ergebnis steht in ZAHL_C
	movfw ZAHL_C0			;kopiere Ergebnis in Divisionsregister ZAHL_A
	movwf ZAHL_A0
	movfw ZAHL_C1
	movwf ZAHL_A1
	clrf ZAHL_A2
	clrf ZAHL_A3
	call clear_zahl_c		;lsche Register ZAHL_C
	movlw 0x09				;lade Wert 7812,5 x 2 = 15.625 (hex: 0x3D09) in Divisionsregister ZAHL_C
	movwf ZAHL_C0
	movlw 0x3D
	movwf ZAHL_C1
	call div32				;Division ZAHL_C / ZAHL_A = ZAHL_C	
	call runden_div32		;Aufruf des Unterprogramms zum ganzzahligen Runden des Ergebnis aus div32	
	return

	
	ORG 0x1FF0

tab_tage
	addwf PCL,f
	retlw 0x00	;Datenpunkt 0:	Nicht vergeben
	retlw d'31'	;Datenpunkt 1:	Januar
	retlw d'28'	;Datenpunkt 2:	Februar
	retlw d'31'	;Datenpunkt 3:	Mrz
	retlw d'30'	;Datenpunkt 4:	April
	retlw d'31'	;Datenpunkt 5:	Mai
	retlw d'30'	;Datenpunkt 6:	Juni
	retlw d'31'	;Datenpunkt 7:	Juli
	retlw d'31'	;Datenpunkt 8:	August
	retlw d'30'	;Datenpunkt 9:	September
	retlw d'31'	;Datenpunkt 10:	Oktober
	retlw d'30'	;Datenpunkt 11:	November
	retlw d'31'	;Datenpunkt 12:	Dezember

	END                	; Ende des Programms

